@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2024 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -136,7 +136,7 @@ struct argp_option program_options[] =
       "Internal name (FITS images: EXTNAME keyword).",
       GAL_OPTIONS_GROUP_OUTPUT,
       &p->metaname,
-      GAL_TYPE_STRING,
+      GAL_TYPE_STRLL,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
@@ -149,7 +149,7 @@ struct argp_option program_options[] =
       "Internal units (FITS images: BUNIT keyword).",
       GAL_OPTIONS_GROUP_OUTPUT,
       &p->metaunit,
-      GAL_TYPE_STRING,
+      GAL_TYPE_STRLL,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
@@ -162,7 +162,7 @@ struct argp_option program_options[] =
       "Internal comments (FITS images: COMMENT keyword).",
       GAL_OPTIONS_GROUP_OUTPUT,
       &p->metacomment,
-      GAL_TYPE_STRING,
+      GAL_TYPE_STRLL,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
@@ -180,6 +180,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "append",
+      UI_KEY_APPEND,
+      0,
+      0,
+      "If output file exists, append to existing HDUs.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->append,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
     {0}
   };
