@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2024 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -87,22 +87,14 @@ pop_number_of_operands(struct arithmeticparams *p, int op,
     case GAL_ARITHMETIC_OP_SIGCLIP_STD:
     case GAL_ARITHMETIC_OP_MADCLIP_MAD:
     case GAL_ARITHMETIC_OP_SIGCLIP_MAD:
+    case GAL_ARITHMETIC_OP_SIGCLIP_ALL:
+    case GAL_ARITHMETIC_OP_MADCLIP_ALL:
     case GAL_ARITHMETIC_OP_MADCLIP_MEAN:
     case GAL_ARITHMETIC_OP_SIGCLIP_MEAN:
     case GAL_ARITHMETIC_OP_MADCLIP_MEDIAN:
     case GAL_ARITHMETIC_OP_SIGCLIP_MEDIAN:
-    case GAL_ARITHMETIC_OP_MADCLIP_NUMBER:
-    case GAL_ARITHMETIC_OP_SIGCLIP_NUMBER:
-    case GAL_ARITHMETIC_OP_MADCLIP_FILL_MAD:
-    case GAL_ARITHMETIC_OP_SIGCLIP_FILL_MAD:
-    case GAL_ARITHMETIC_OP_MADCLIP_FILL_STD:
-    case GAL_ARITHMETIC_OP_SIGCLIP_FILL_STD:
-    case GAL_ARITHMETIC_OP_MADCLIP_FILL_MEAN:
-    case GAL_ARITHMETIC_OP_SIGCLIP_FILL_MEAN:
-    case GAL_ARITHMETIC_OP_MADCLIP_FILL_MEDIAN:
-    case GAL_ARITHMETIC_OP_SIGCLIP_FILL_MEDIAN:
-    case GAL_ARITHMETIC_OP_MADCLIP_FILL_NUMBER:
-    case GAL_ARITHMETIC_OP_SIGCLIP_FILL_NUMBER:
+    case GAL_ARITHMETIC_OP_MADCLIP_MASKFILLED:
+    case GAL_ARITHMETIC_OP_SIGCLIP_MASKFILLED:
       numparams=2;
       break;
     }
@@ -228,8 +220,8 @@ arithmetic_filter(void *in_prm)
   size_t *tsize, *dsize=input->dsize, *fsize=afp->fsize;
   size_t i, j, coord[ARITHMETIC_FILTER_DIM], ndim=input->ndim;
   size_t start[ARITHMETIC_FILTER_DIM], end[ARITHMETIC_FILTER_DIM];
-  gal_data_t *tile=gal_data_alloc(NULL, input->type, ndim, afp->fsize, NULL,
-                                  0, -1, 1, NULL, NULL, NULL);
+  gal_data_t *tile=gal_data_alloc(NULL, input->type, ndim, afp->fsize,
+                                  NULL, 0, -1, 1, NULL, NULL, NULL);
 
   /* Prepare the tile. */
   free(tile->array);
@@ -287,6 +279,16 @@ arithmetic_filter(void *in_prm)
       /* Do the necessary calculation. */
       switch(afp->operator)
         {
+        case ARITHMETIC_OP_FILTER_MINIMUM:
+          result=gal_statistics_minimum(tile);
+          break;
+
+
+        case ARITHMETIC_OP_FILTER_MAXIMUM:
+          result=gal_statistics_maximum(tile);
+          break;
+
+
         case ARITHMETIC_OP_FILTER_MEDIAN:
           result=gal_statistics_median(tile, 0);
           break;
@@ -297,22 +299,55 @@ arithmetic_filter(void *in_prm)
           break;
 
 
+        case ARITHMETIC_OP_FILTER_SIGCLIP_STD:
+        case ARITHMETIC_OP_FILTER_MADCLIP_STD:
+        case ARITHMETIC_OP_FILTER_SIGCLIP_MAD:
+        case ARITHMETIC_OP_FILTER_MADCLIP_MAD:
         case ARITHMETIC_OP_FILTER_SIGCLIP_MEAN:
+        case ARITHMETIC_OP_FILTER_MADCLIP_MEAN:
         case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN:
-          /* The median is always available with a sigma-clip, but the mean
-             needs to be explicitly requested. */
-          if(afp->operator == ARITHMETIC_OP_FILTER_SIGCLIP_MEAN)
+        case ARITHMETIC_OP_FILTER_MADCLIP_MEDIAN:
+
+          /* The mean needs to be explicitly requested. */
+          if(   afp->operator == ARITHMETIC_OP_FILTER_SIGCLIP_MEAN
+             || afp->operator == ARITHMETIC_OP_FILTER_MADCLIP_MEAN )
             clipflags = GAL_STATISTICS_CLIP_OUTCOL_OPTIONAL_MEAN;
-          sigclip=gal_statistics_clip_sigma(tile, afp->sclip_multip,
+
+          /* The STD must be explicitly requested for MAD-clipping. */
+          if( afp->operator == ARITHMETIC_OP_FILTER_MADCLIP_STD )
+            clipflags = GAL_STATISTICS_CLIP_OUTCOL_OPTIONAL_STD;
+
+          /* The MAD must be explicitly requested for sigma-clipping. */
+          if( afp->operator == ARITHMETIC_OP_FILTER_SIGCLIP_MAD )
+            clipflags = GAL_STATISTICS_CLIP_OUTCOL_OPTIONAL_STD;
+
+          /* Do the main operation. */
+          if(    afp->operator==ARITHMETIC_OP_FILTER_SIGCLIP_STD
+              || afp->operator==ARITHMETIC_OP_FILTER_SIGCLIP_MAD
+              || afp->operator==ARITHMETIC_OP_FILTER_SIGCLIP_MEAN
+              || afp->operator==ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN )
+            sigclip=gal_statistics_clip_sigma(tile, afp->sclip_multip,
+                                              afp->sclip_param, clipflags,
+                                              0, 1);
+          else
+            sigclip=gal_statistics_clip_mad(tile, afp->sclip_multip,
                                             afp->sclip_param, clipflags,
                                             0, 1);
 
           /* Set the required index. */
           switch(afp->operator)
             {
+            case ARITHMETIC_OP_FILTER_SIGCLIP_STD:
+            case ARITHMETIC_OP_FILTER_MADCLIP_STD:
+              sind = GAL_STATISTICS_CLIP_OUTCOL_STD; break;
+            case ARITHMETIC_OP_FILTER_SIGCLIP_MAD:
+            case ARITHMETIC_OP_FILTER_MADCLIP_MAD:
+              sind = GAL_STATISTICS_CLIP_OUTCOL_MAD; break;
             case ARITHMETIC_OP_FILTER_SIGCLIP_MEAN:
+            case ARITHMETIC_OP_FILTER_MADCLIP_MEAN:
               sind = GAL_STATISTICS_CLIP_OUTCOL_MEAN; break;
             case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN:
+            case ARITHMETIC_OP_FILTER_MADCLIP_MEDIAN:
               sind = GAL_STATISTICS_CLIP_OUTCOL_MEDIAN; break;
             default:
               error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
@@ -324,8 +359,7 @@ arithmetic_filter(void *in_prm)
           /* Allocate the output and write the value into it. */
           result=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL,
                                 0, -1, 1, NULL, NULL, NULL);
-          ((float *)(result->array))[0] =
-            ((float *)(sigclip->array))[sind];
+          ((float *)(result->array))[0]=((float *)(sigclip->array))[sind];
 
           /* Clean up. */
           gal_data_free(sigclip);
@@ -335,7 +369,7 @@ arithmetic_filter(void *in_prm)
         default:
           error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s "
                 "to fix the problem. 'afp->operator' code %d is not "
-                "recognized", PACKAGE_BUGREPORT, __func__,
+                "recognized", __func__, PACKAGE_BUGREPORT,
                 afp->operator);
         }
 
@@ -378,8 +412,14 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
   size_t fsize[ARITHMETIC_FILTER_DIM];
   gal_data_t *tmp, *tmp2, *zero, *comp, *params_list=NULL;
   size_t hnfsize[ARITHMETIC_FILTER_DIM], hpfsize[ARITHMETIC_FILTER_DIM];
-  int issigclip=(operator==ARITHMETIC_OP_FILTER_SIGCLIP_MEAN
-                 || operator==ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN);
+  int isclip=(   operator==ARITHMETIC_OP_FILTER_SIGCLIP_STD
+              || operator==ARITHMETIC_OP_FILTER_MADCLIP_STD
+              || operator==ARITHMETIC_OP_FILTER_SIGCLIP_MAD
+              || operator==ARITHMETIC_OP_FILTER_MADCLIP_MAD
+              || operator==ARITHMETIC_OP_FILTER_SIGCLIP_MEAN
+              || operator==ARITHMETIC_OP_FILTER_MADCLIP_MEAN
+              || operator==ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN
+              || operator==ARITHMETIC_OP_FILTER_MADCLIP_MEDIAN );
 
 
   /* Get the input's number of dimensions. */
@@ -405,7 +445,7 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
 
   /* Based on the first popped operand's dimensions and the operator, of
      pop the necessary number of operands. */
-  nparams = ndim + (issigclip ? 2 : 0 );
+  nparams = ndim + (isclip ? 2 : 0 );
   for(i=0;i<nparams;++i)
     {
       /* Add this to the list of parameters. */
@@ -422,7 +462,7 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
 
   /* If this is a sigma-clipping filter, the top two operands are the
      sigma-clipping parameters. */
-  if(issigclip)
+  if(isclip)
     {
       /* Read the sigma-clipping multiple (first element in the list). */
       tmp=gal_list_data_pop(&params_list);
@@ -507,6 +547,7 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
             { hnfsize[i]=fsize[i]/2; hpfsize[i]=fsize[i]/2-1; }
         }
 
+
       /* For a test.
       printf("fsize: %zu, %zu\n", fsize[0], fsize[1]);
       printf("hnfsize: %zu, %zu\n", hnfsize[0], hnfsize[1]);
@@ -518,19 +559,27 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
       switch(operator)
         {
         case ARITHMETIC_OP_FILTER_MEDIAN:
+        case ARITHMETIC_OP_FILTER_MINIMUM:
+        case ARITHMETIC_OP_FILTER_MAXIMUM:
         case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN:
+        case ARITHMETIC_OP_FILTER_MADCLIP_MEDIAN:
           type=afp.input->type;
           break;
 
         case ARITHMETIC_OP_FILTER_MEAN:
+        case ARITHMETIC_OP_FILTER_SIGCLIP_STD:
+        case ARITHMETIC_OP_FILTER_MADCLIP_STD:
+        case ARITHMETIC_OP_FILTER_SIGCLIP_MAD:
+        case ARITHMETIC_OP_FILTER_MADCLIP_MAD:
         case ARITHMETIC_OP_FILTER_SIGCLIP_MEAN:
+        case ARITHMETIC_OP_FILTER_MADCLIP_MEAN:
           type=GAL_TYPE_FLOAT64;
           break;
 
         default:
-          error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to "
-                "fix the problem. The 'operator' code %d is not recognized",
-                PACKAGE_BUGREPORT, __func__, operator);
+          error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s "
+                "to fix the problem. The 'operator' code %d is not "
+                "recognized", __func__, PACKAGE_BUGREPORT, operator);
         }
 
 
@@ -639,12 +688,14 @@ arithmetic_erode_dilate(struct arithmeticparams *p, char *token, int op)
   /* Do the operation. */
   switch(op)
     {
-    case ARITHMETIC_OP_ERODE:  gal_binary_erode(in,  1, conn_int, 1); break;
-    case ARITHMETIC_OP_DILATE: gal_binary_dilate(in, 1, conn_int, 1); break;
+    case ARITHMETIC_OP_ERODE:  gal_binary_erode(in,  1, conn_int, 1);
+      break;
+    case ARITHMETIC_OP_DILATE: gal_binary_dilate(in, 1, conn_int, 1);
+      break;
     default:
-      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
-            "problem. The operator code %d not recognized", __func__,
-            PACKAGE_BUGREPORT, op);
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
+            "fix the problem. The operator code %d not recognized",
+            __func__, PACKAGE_BUGREPORT, op);
     }
 
   /* Push the result onto the stack. */
@@ -739,10 +790,10 @@ arithmetic_invert(struct arithmeticparams *p, char *token)
   /* Do the inversion based on type. */
   switch(in->type)
     {
-    case GAL_TYPE_UINT8:  do *u8  = UINT8_MAX-*u8;   while(++u8<u8f);   break;
-    case GAL_TYPE_UINT16: do *u16 = UINT16_MAX-*u16; while(++u16<u16f); break;
-    case GAL_TYPE_UINT32: do *u32 = UINT32_MAX-*u32; while(++u32<u32f); break;
-    case GAL_TYPE_UINT64: do *u64 = UINT64_MAX-*u64; while(++u64<u64f); break;
+    case GAL_TYPE_UINT8:  do *u8=UINT8_MAX-*u8;   while(++u8<u8f);   break;
+    case GAL_TYPE_UINT16: do *u16=UINT16_MAX-*u16;while(++u16<u16f); break;
+    case GAL_TYPE_UINT32: do *u32=UINT32_MAX-*u32;while(++u32<u32f); break;
+    case GAL_TYPE_UINT64: do *u64=UINT64_MAX-*u64;while(++u64<u64f); break;
     default:
       error(EXIT_FAILURE, 0, "'invert' operand has %s type. 'invert' can "
             "only take unsigned integer types.\n\nYou can use any of the "
@@ -767,17 +818,20 @@ arithmetic_invert(struct arithmeticparams *p, char *token)
     for(i=0;i<in->size;++i)                                             \
       {                                                                 \
         if( l[i]>0 )                                                    \
-          GAL_DIMENSION_NEIGHBOR_OP(i, in->ndim, in->dsize, in->ndim,   \
+          GAL_DIMENSION_NEIGHBOR_OP(i, in->ndim, in->dsize, ocon[0],    \
                                     dinc,                               \
             {                                                           \
-              if(in->type==GAL_TYPE_FLOAT32 || in->type==GAL_TYPE_FLOAT64) \
+              if(   in->type==GAL_TYPE_FLOAT32                          \
+                 || in->type==GAL_TYPE_FLOAT64 )                        \
                 { if( a[nind] OP m[l[i]] ) m[l[i]]=a[nind]; }           \
               else                                                      \
-                { if( a[nind]!=b && a[nind] OP m[l[i]] ) m[l[i]]=a[nind]; } \
+                { if( a[nind]!=b && a[nind] OP m[l[i]] )                \
+                    m[l[i]]=a[nind]; }                                  \
             });                                                         \
       }                                                                 \
     for(i=0;i<in->size;++i) if( l[i]>0 ) { a[i]=m[l[i]]; }              \
 }
+
 #define INTERPOLATE_REGION_OP(TYPE) {                                   \
     switch(operator)                                                    \
       {                                                                 \
@@ -799,38 +853,44 @@ arithmetic_interpolate_region(struct arithmeticparams *p,
   int32_t *l;
   gal_data_t *minmax;
   gal_data_t *lab=NULL, *flag;
-  size_t i, *con, *dinc, numlabs;
+  size_t i, *icon, *ocon, *dinc, numlabs;
 
   /* First pop the number of nearby neighbors.*/
-  gal_data_t *connectivity = operands_pop(p, token);
+  gal_data_t *outconn = operands_pop(p, token);
+  gal_data_t *inconn  = operands_pop(p, token);
 
   /* Then pop the actual dataset to interpolate. */
   gal_data_t *in = operands_pop(p, token);
 
   /* Do proper sanity checks on 'con'. */
-  if(connectivity->size!=1)
-    error(EXIT_FAILURE, 0, "the first popped operand to the "
-          "'interpolate-XXXofregion' operators must be a single number. "
-          "However, it has %zu elements", connectivity->size);
-  if( connectivity->type==GAL_TYPE_FLOAT32
-      || connectivity->type==GAL_TYPE_FLOAT64)
-    error(EXIT_FAILURE, 0, "the first popped operand to "
-          "'interpolate-XXXofregion' operators is the connectivity to "
-          "define connected blank regions (a counter, an integer, with "
-          "a maximum of the number of dimensions of the input). It must "
-          "NOT be a floating point.\n\n"
-          "If its already an integer, but in a floating point container, "
-          "you can use the 'int32' operator to convert it to a 32-bit "
-          "integer for example");
+  if(inconn->size!=1 || outconn->size!=1)
+    error(EXIT_FAILURE, 0, "the first and second popped operands to the "
+          "'interpolate-*ofregion' operators must be a single number. "
+          "However, it has %zu elements", inconn->size);
+  if(    inconn->type==GAL_TYPE_FLOAT32
+      || inconn->type==GAL_TYPE_FLOAT64
+      || outconn->type==GAL_TYPE_FLOAT32
+      || outconn->type==GAL_TYPE_FLOAT64 )
+    error(EXIT_FAILURE, 0, "the first and second popped operands to "
+          "'interpolate-*ofregion' operators are the connectivity to "
+          "define connected regions. The first popped is used to the "
+          "define the connectivity of the outer neighbors and the "
+          "second popped operand is used to define the region. They "
+          "should both be a counter (a positive integer) with a maximum "
+          "of the number of dimensions of the input). They cannot NOT "
+          "be a floating point. If its already an integer, but in a "
+          "floating point container, you can use the 'int32' operator "
+          "to convert it to a 32-bit integer");
 
   /* Convert connectivity to an integer type and make sure its not larger
      than dimensions of the input. */
-  connectivity=gal_data_copy_to_new_type_free(connectivity,
-                                              GAL_TYPE_SIZE_T);
-  con=connectivity->array;
-  if(con[0]>in->ndim)
-    error(EXIT_FAILURE, 0, "the first popped operand to "
-          "'interpolate-XXXofregion' operators must not be larger than "
+  inconn=gal_data_copy_to_new_type_free(inconn, GAL_TYPE_SIZE_T);
+  outconn=gal_data_copy_to_new_type_free(outconn, GAL_TYPE_SIZE_T);
+  icon=inconn->array;
+  ocon=outconn->array;
+  if(icon[0]>in->ndim || ocon[0]>in->ndim)
+    error(EXIT_FAILURE, 0, "the first and second popped operands to "
+          "'interpolate-*ofregion' operators must not be larger than "
           "the number of dimensions of the input. The connectivity is "
           "used to define connected blank regions. For example in a "
           "2D dataset, a connectivity of 1 corresponds to 4-connected "
@@ -845,7 +905,7 @@ arithmetic_interpolate_region(struct arithmeticparams *p,
   /* Build a binary image with the blank regions masked and label them,
      then free the flagged array. */
   flag=gal_blank_flag(in);
-  numlabs=gal_binary_connected_components(flag, &lab, con[0]);
+  numlabs=gal_binary_connected_components(flag, &lab, icon[0]);
   gal_data_free(flag);
 
   /* Allocate array to keep maximum values for each region. Just note that
@@ -884,7 +944,7 @@ arithmetic_interpolate_region(struct arithmeticparams *p,
   free(dinc);
   gal_data_free(lab);
   gal_data_free(minmax);
-  gal_data_free(connectivity);
+  gal_data_free(inconn);
 
   /* Push the interpolated dataset onto the stack. */
   operands_add(p, NULL, in);
@@ -1377,8 +1437,10 @@ arithmetic_add_dimension(struct arithmeticparams *p, char *token,
           break;
         case ARITHMETIC_OP_ADD_DIMENSION_FAST:
           for(j=0;j<tmp->size;++j)
-            memcpy(gal_pointer_increment(out->array, j*num+num-i-1, out->type),
-                   gal_pointer_increment(tmp->array, j, out->type), nbytes);
+            memcpy(gal_pointer_increment(out->array, j*num+num-i-1,
+                                         out->type),
+                   gal_pointer_increment(tmp->array, j, out->type),
+                   nbytes);
 
           break;
         }
@@ -1465,10 +1527,26 @@ arithmetic_set_operator(char *string, size_t *num_operands, int *inlib)
         { op=ARITHMETIC_OP_FILTER_MEAN;           *num_operands=0; }
       else if (!strcmp(string, "filter-median"))
         { op=ARITHMETIC_OP_FILTER_MEDIAN;         *num_operands=0; }
+      else if (!strcmp(string, "filter-minimum"))
+        { op=ARITHMETIC_OP_FILTER_MINIMUM;        *num_operands=0; }
+      else if (!strcmp(string, "filter-maximum"))
+        { op=ARITHMETIC_OP_FILTER_MAXIMUM;        *num_operands=0; }
+      else if (!strcmp(string, "filter-sigclip-std"))
+        { op=ARITHMETIC_OP_FILTER_SIGCLIP_STD;   *num_operands=0; }
+      else if (!strcmp(string, "filter-sigclip-mad"))
+        { op=ARITHMETIC_OP_FILTER_SIGCLIP_MAD;   *num_operands=0; }
+      else if (!strcmp(string, "filter-madclip-std"))
+        { op=ARITHMETIC_OP_FILTER_MADCLIP_STD;   *num_operands=0; }
+      else if (!strcmp(string, "filter-madclip-mad"))
+        { op=ARITHMETIC_OP_FILTER_MADCLIP_MAD;   *num_operands=0; }
       else if (!strcmp(string, "filter-sigclip-mean"))
         { op=ARITHMETIC_OP_FILTER_SIGCLIP_MEAN;   *num_operands=0; }
+      else if (!strcmp(string, "filter-madclip-mean"))
+        { op=ARITHMETIC_OP_FILTER_MADCLIP_MEAN;   *num_operands=0; }
       else if (!strcmp(string, "filter-sigclip-median"))
         { op=ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN; *num_operands=0; }
+      else if (!strcmp(string, "filter-madclip-median"))
+        { op=ARITHMETIC_OP_FILTER_MADCLIP_MEDIAN; *num_operands=0; }
       else if (!strcmp(string, "erode"))
         { op=ARITHMETIC_OP_ERODE;                 *num_operands=0; }
       else if (!strcmp(string, "dilate"))
@@ -1502,7 +1580,7 @@ arithmetic_set_operator(char *string, size_t *num_operands, int *inlib)
       else if (!strcmp(string, "collapse-mean"))
         { op=ARITHMETIC_OP_COLLAPSE_MEAN;         *num_operands=0; }
       else if (!strcmp(string, "collapse-number"))
-        { op=ARITHMETIC_OP_COLLAPSE_MEDIAN;       *num_operands=0; }
+        { op=ARITHMETIC_OP_COLLAPSE_NUMBER;       *num_operands=0; }
       else if (!strcmp(string, "collapse-median"))
         { op=ARITHMETIC_OP_COLLAPSE_MEDIAN;       *num_operands=0; }
       else if (!strcmp(string, "collapse-madclip-mad"))
@@ -1742,8 +1820,16 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
         {
         case ARITHMETIC_OP_FILTER_MEAN:
         case ARITHMETIC_OP_FILTER_MEDIAN:
+        case ARITHMETIC_OP_FILTER_MINIMUM:
+        case ARITHMETIC_OP_FILTER_MAXIMUM:
+        case ARITHMETIC_OP_FILTER_SIGCLIP_STD:
+        case ARITHMETIC_OP_FILTER_MADCLIP_STD:
+        case ARITHMETIC_OP_FILTER_SIGCLIP_MAD:
+        case ARITHMETIC_OP_FILTER_MADCLIP_MAD:
         case ARITHMETIC_OP_FILTER_SIGCLIP_MEAN:
+        case ARITHMETIC_OP_FILTER_MADCLIP_MEAN:
         case ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN:
+        case ARITHMETIC_OP_FILTER_MADCLIP_MEDIAN:
           wrapper_for_filter(p, operator_string, operator);
           break;
 
@@ -1913,6 +1999,44 @@ arithmetic_final_data(struct arithmeticparams *p)
 
 
 
+/* Add any metadata in case the user wanted to do so. We are treating each
+   metadata entry separately because the user may not want to specify them
+   all for all outputs and the looping is not computationally expensive.*/
+static void
+arithmetic_out_metadata(struct arithmeticparams *p, gal_data_t *data)
+{
+  gal_data_t *d;
+  gal_list_str_t *s;
+
+  /* Loop over 'metaname' and add each item to the output dataset(s). */
+  d=data;
+  for(s=p->metaname; s!=NULL && d!=NULL; s=s->next, d=d->next)
+    {
+      if(d->name) free(d->name);
+      gal_checkset_allocate_copy(s->v, &d->name);
+    }
+
+  /* Similar to 'metaname' but for meta units. */
+  d=data;
+  for(s=p->metaunit; s!=NULL && d!=NULL; s=s->next, d=d->next)
+    {
+      if(d->unit) free(d->unit);
+      gal_checkset_allocate_copy(s->v, &d->unit);
+    }
+
+  /* Similar to 'metacomment' but for meta comments. */
+  d=data;
+  for(s=p->metacomment; s!=NULL && d!=NULL; s=s->next, d=d->next)
+    {
+      if(d->comment) free(d->comment);
+      gal_checkset_allocate_copy(s->v, &d->comment);
+    }
+}
+
+
+
+
+
 /* This function implements the reverse polish algorithm as explained
    in the Wikipedia page.
 
@@ -2031,27 +2155,32 @@ reversepolish(struct arithmeticparams *p)
     {
       /* If the user has requested a name, units or comments for the FITS
          file, insert them into the dataset here. */
-      if(p->metaname)
-        { if(data->name) free(data->name);
-          gal_checkset_allocate_copy(p->metaname, &data->name); }
-      if(p->metaunit)
-        { if(data->unit) free(data->unit);
-          gal_checkset_allocate_copy(p->metaunit, &data->unit); }
-      if(p->metacomment)
-        { if(data->comment) free(data->comment);
-          gal_checkset_allocate_copy(p->metacomment, &data->comment); }
+      arithmetic_out_metadata(p, data);
 
       /* Put a copy of the WCS structure from the reference image, it will
          be freed while freeing 'data'. But start the file with the 0-th
          HDU keywords.*/
-      gal_fits_key_write(p->cp.ckeys, p->cp.output, "0", "NONE", 1, 1);
       if(data->ndim==1 && p->onedasimage==0)
-        gal_table_write(data, NULL, NULL, p->cp.tableformat,
-                        p->onedonstdout ? NULL : p->cp.output,
-                        "ARITHMETIC", 0, 0);
+        {
+          /* The keywords HDU should only be created when a FITS file is to
+             be made; not when the output should go to a text file or the
+             standard output. */
+          if( p->onedonstdout==0
+              && gal_fits_name_is_fits(p->cp.output)==0 )
+            gal_fits_key_write(p->cp.ckeys, p->cp.output, "0", "NONE",
+                               1, 1);
+          gal_table_write(data, NULL, NULL, p->cp.tableformat,
+                          p->onedonstdout ? NULL : p->cp.output,
+                          "ARITHMETIC", 0, 0);
+        }
       else
-        for(tmp=data; tmp!=NULL; tmp=tmp->next)
-          gal_fits_img_write(tmp, p->cp.output, NULL, 0);
+        {
+          if(p->append==0) /* 0-HDU metadata only when not appending. */
+            gal_fits_key_write(p->cp.ckeys, p->cp.output, "0",
+                               "NONE", 1, 1);
+          for(tmp=data; tmp!=NULL; tmp=tmp->next)
+            gal_fits_img_write(tmp, p->cp.output, NULL, 0);
+        }
 
       /* Let the user know that the job is done. */
       if(!p->cp.quiet)
