@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2024 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,8 +33,16 @@ enum arithmetic_prog_operators
 {
   ARITHMETIC_OP_FILTER_MEDIAN = GAL_ARITHMETIC_OP_LAST_CODE,
   ARITHMETIC_OP_FILTER_MEAN,
+  ARITHMETIC_OP_FILTER_MINIMUM,
+  ARITHMETIC_OP_FILTER_MAXIMUM,
+  ARITHMETIC_OP_FILTER_SIGCLIP_STD,
+  ARITHMETIC_OP_FILTER_MADCLIP_STD,
+  ARITHMETIC_OP_FILTER_SIGCLIP_MAD,
+  ARITHMETIC_OP_FILTER_MADCLIP_MAD,
   ARITHMETIC_OP_FILTER_SIGCLIP_MEAN,
+  ARITHMETIC_OP_FILTER_MADCLIP_MEAN,
   ARITHMETIC_OP_FILTER_SIGCLIP_MEDIAN,
+  ARITHMETIC_OP_FILTER_MADCLIP_MEDIAN,
   ARITHMETIC_OP_ERODE,
   ARITHMETIC_OP_DILATE,
   ARITHMETIC_OP_NUMBER_NEIGHBORS,
