@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -28,6 +28,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdio.h>
 #include <string.h>
 
+#include <gnuastro/txt.h>
 #include <gnuastro/wcs.h>
 #include <gnuastro/list.h>
 #include <gnuastro/fits.h>
@@ -79,9 +80,9 @@ doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" will do arithmetic "
   "\n\n"PROGRAM_NAME" recognizes a large collection of standard operators, "
   "including basic arithmetic (e.g., +, -, x, /), mathematical (e.g., abs, "
   "pow, sqrt, log), statistical (minvalue, min, max, average), comparison "
-  "(e.g., lt, le, gt), logical (e.g., and, or, not), the full set of bitwise "
-  "operators, and numeric type conversion operators to all known types. "
-  "Please run the command below for a complete list describing all "
+  "(e.g., lt, le, gt), logical (e.g., and, or, not), the full set of "
+  "bitwise operators, and numeric type conversion operators to all known "
+  "types. Please run the command below for a complete list describing all "
   "operators (press the 'SPACE' keyboard key, or arrow keys, to go down "
   "and 'q' to return to the command-line):\n\n"
   "     $ info gnuastro \"Arithmetic operators\"\n"
@@ -229,8 +230,24 @@ parse_opt(int key, char *arg, struct argp_state *state)
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
+static int
+ui_check_dash_is_negative(char *string)
+{
+  if( isdigit(string[1])  /* We already know 'string[0]' is '-' */
+      || !strcmp(string, "-nan")
+      || !strcmp(string, "-inf") )
+    return 1;
+  else return 0;
+}
+
+
+
+
+
+/* Check ONLY the options. When arguments are involved, do the check
+   in 'ui_check_options_and_arguments'. */
 static void
-ui_read_check_only_options(struct arithmeticparams *p)
+ui_check_only_options(struct arithmeticparams *p)
 {
   if(p->wcsfile && strcmp(p->wcsfile,"none"))
     {
@@ -243,6 +260,11 @@ ui_read_check_only_options(struct arithmeticparams *p)
               "to '--wcsfile')! Please use '--wcshdu' to specify a "
               "HDU/extension to read from", p->wcsfile);
     }
+
+  /* In case meta information are given, correct them into simple lists. */
+  gal_options_merge_list_of_csv(&p->metaname);
+  gal_options_merge_list_of_csv(&p->metaunit);
+  gal_options_merge_list_of_csv(&p->metacomment);
 }
 
 
@@ -250,7 +272,7 @@ ui_read_check_only_options(struct arithmeticparams *p)
 
 
 /* Sanity check on options AND arguments. If only option values are to be
-   checked, use 'ui_read_check_only_options'. */
+   checked, use 'ui_check_only_options'. */
 static void
 ui_check_options_and_arguments(struct arithmeticparams *p)
 {
@@ -259,21 +281,31 @@ ui_check_options_and_arguments(struct arithmeticparams *p)
   char *tofilename=NULL, *basename=NULL;
   struct gal_options_common_params *cp=&p->cp;
 
-  /* First, make sure that any tokens are actually given. */
-  if(p->tokens==NULL)
-    error(EXIT_FAILURE, 0, "no input tokens. Please specify a file name "
-          "or number (as operands) along with operator(s) as input. "
-          "Please run any of the following commands for more "
-          "information.\n\n"
-          "    $ astarithmetic --help           # Short info.\n"
-          "    $ info astarithmetic             # Full invocation "
-          "documentation.\n");
-
-  /* The input tokens are put in a lastin-firstout (simple) linked list, so
-     change them to the correct order so the order we pop a token is the
-     same order that the user input a value. Note that for the options this
-     was done in 'gal_options_read_config_set'. */
-  gal_list_str_reverse(&p->tokens);
+  /* First, make sure that tokens are actually given. */
+  if(p->tokens)
+    {
+      /* The input tokens are put in a lastin-firstout (simple) linked
+         list, so change them to the correct order so the order we pop a
+         token is the same order that the user input a value. Note that for
+         the options this was done in 'gal_options_read_config_set'. */
+      gal_list_str_reverse(&p->tokens);
+    }
+  else
+    {
+      if(p->arguments)
+        {
+          p->tokens=gal_txt_read_to_list(p->arguments);
+          free(p->arguments); p->arguments=NULL;
+        }
+      else
+        error(EXIT_FAILURE, 0, "no input tokens. Please specify a file "
+              "name or number (as operands) along with operator(s) as "
+              "input; or use the '--arguments' options. Please run any "
+              "of the following commands for more information.\n\n"
+              "    $ astarithmetic --help           # Short info.\n"
+              "    $ info astarithmetic             # Full invocation "
+              "documentation.\n");
+    }
 
   /* To allow adding extensions to existing files, let the 'keep' flag be
      the same as the 'dontdelete'. */
@@ -331,14 +363,16 @@ ui_check_options_and_arguments(struct arithmeticparams *p)
           /* This token is a number. Check if a negative dash was present
              that has been temporarily replaced with 'NEG_DASH_REPLACE'
              before option parsing. */
-          else if(token->v[0]==NEG_DASH_REPLACE && isdigit(token->v[1]) )
+          else if( token->v[0]==NEG_DASH_REPLACE
+                   && ui_check_dash_is_negative(token->v) )
             token->v[0]='-';
         }
     }
 
   /* In case no output name has been given (can happen with operators like
-     'makenew' when the user doesn't set an output name explicity), use a
+     'makenew' when the user doesn't set an input name explicity), use a
      default name. */
+  if(p->append) cp->keep=1;
   if(cp->output)
     gal_checkset_writable_remove(cp->output, basename, cp->keep,
                                  cp->dontdelete);
@@ -436,7 +470,8 @@ ui_preparations(struct arithmeticparams *p)
 /************         Set the parameters          *************/
 /**************************************************************/
 void
-ui_read_check_inputs_setup(int argc, char *argv[], struct arithmeticparams *p)
+ui_read_check_inputs_setup(int argc, char *argv[],
+                           struct arithmeticparams *p)
 {
   size_t i;
   struct gal_options_common_params *cp=&p->cp;
@@ -462,7 +497,7 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct arithmeticparams *p)
      if any one starts with a dash and is followed by a number, then the
      dash is replaced by NEG_DASH_REPLACE. */
   for(i=0;i<argc;++i)
-    if(argv[i][0]=='-' && isdigit(argv[i][1]))
+    if( argv[i][0]=='-' && ui_check_dash_is_negative(argv[i]) )
       argv[i][0]=NEG_DASH_REPLACE;
 
 
@@ -482,8 +517,12 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct arithmeticparams *p)
   gal_options_print_state(cp);
 
 
+  /* Prepare all the options as FITS keywords to write in output later. */
+  gal_options_as_fits_keywords(&p->cp);
+
+
   /* Sanity check only on options. */
-  ui_read_check_only_options(p);
+  ui_check_only_options(p);
 
 
   /* Check that the options and arguments fit well with each other. Note
@@ -519,15 +558,15 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct arithmeticparams *p)
 /************      Free allocated, report         *************/
 /**************************************************************/
 void
-freeandreport(struct arithmeticparams *p, struct timeval *t1)
+ui_free_report(struct arithmeticparams *p, struct timeval *t1)
 {
   /* Free the simple strings. */
   free(p->cp.output);
   if(p->wcshdu) free(p->wcshdu);
-  if(p->metaname) free(p->metaname);
-  if(p->metaunit) free(p->metaunit);
   if(p->globalhdu) free(p->globalhdu);
-  if(p->metacomment) free(p->metacomment);
+  if(p->metaname) gal_list_str_free(p->metaname, 1);
+  if(p->metaunit) gal_list_str_free(p->metaunit, 1);
+  if(p->metacomment) gal_list_str_free(p->metacomment, 1);
 
   /* If there are any remaining HDUs in the hdus linked list, then
      free them. */
