@@ -5,7 +5,7 @@ ConvertType is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2024 Free Software Foundation, Inc.
+Copyright (C) 2016-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -103,6 +103,32 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       gal_options_parse_csv_strings
     },
+    {
+      "cmapblankcolor",
+      UI_KEY_CMAPBLANKCOLOR,
+      "STR",
+      0,
+      "Blank color name in color maps.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->cmapblankcolor,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+    },
+    {
+      "cmappgfplots",
+      UI_KEY_CMAPPGFPLOTS,
+      0,
+      0,
+      "Color map to include in PGFPlots (in LaTeX).",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->cmappgfplots,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "rgbtohsv",
       UI_KEY_RGBTOHSV,
@@ -171,7 +197,7 @@ struct argp_option program_options[] =
       UI_KEY_FORCEMIN,
       0,
       0,
-      "Force --fluxmin, even when smaller than minimum.",
+      "Force --fluxmin, even when smaller than min.",
       UI_GROUP_FLUX,
       &p->forcemin,
       GAL_OPTIONS_NO_ARG_TYPE,
@@ -184,7 +210,7 @@ struct argp_option program_options[] =
       UI_KEY_FORCEMAX,
       0,
       0,
-      "Force --fluxmax, even when larger than maximum.",
+      "Force --fluxmax, even when larger than max.",
       UI_GROUP_FLUX,
       &p->forcemax,
       GAL_OPTIONS_NO_ARG_TYPE,
