@@ -5,7 +5,7 @@ CosmicCalculator is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2024 Free Software Foundation, Inc.
+Copyright (C) 2016-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -28,8 +28,6 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdio.h>
 #include <string.h>
 
-#include <gsl/gsl_const_mksa.h>
-
 #include <gnuastro/fits.h>
 #include <gnuastro/table.h>
 #include <gnuastro/pointer.h>
@@ -494,7 +492,7 @@ ui_list_lines(struct cosmiccalparams *p)
     {
       /* Get the wavelength (in angstroms) for this line and print it. */
       ang=gal_speclines_line_angstrom(i);
-      printf("%-20g%s\n",
+      printf("%-15.3f %s\n",
              ( p->listlinesatz
                ? ( ang * (1+p->redshift) ) : ang ) * p->lineunitmultip,
              gal_speclines_line_name(i));
@@ -507,7 +505,7 @@ ui_list_lines(struct cosmiccalparams *p)
       /* Get the wavelength (in angstroms) for this line and print it. */
       j=i+GAL_SPECLINES_LIMIT_LYMAN;
       ang=gal_speclines_line_angstrom(j);
-      printf("%-20g%s\n",
+      printf("%-15.3f %s\n",
              ( p->listlinesatz
                ? ( ang * (1+p->redshift) ) : ang ) * p->lineunitmultip,
              gal_speclines_line_name(j));
