@@ -5,7 +5,7 @@ Crop is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -387,6 +387,15 @@ onecrop_name(struct onecropparams *crp)
   struct cropparams *p=crp->p;
   struct gal_options_common_params *cp=&p->cp;
 
+  /* If '--oneelemstdout' was called, then no actual file is created, we
+     just need to the row number.*/
+  if(p->oneelemstdout)
+    {
+      if( asprintf(&crp->name, "%zu", crp->out_ind)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+      return;
+    }
+
   /* Set the output name and crop sides: */
   if(p->catname)
     {
@@ -405,7 +414,7 @@ onecrop_name(struct onecropparams *crp)
           error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
 
       /* Make sure the file doesn't exist. */
-      gal_checkset_writable_remove(crp->name, 0, cp->dontdelete);
+      gal_checkset_writable_remove(crp->name, NULL, cp->keep, cp->dontdelete);
     }
   else
     {
@@ -413,7 +422,8 @@ onecrop_name(struct onecropparams *crp)
       if(p->outnameisfile)            /* An output file was specified. */
         {
           crp->name=cp->output;
-          gal_checkset_writable_remove(crp->name, 0, cp->dontdelete);
+          gal_checkset_writable_remove(crp->name, p->imgs[crp->in_ind].name,
+                                       cp->keep, cp->dontdelete);
         }
       else          /* The output was a directory, use automatic output. */
         crp->name=gal_checkset_automatic_output(cp,
@@ -476,7 +486,8 @@ onecrop_flpixel(struct onecropparams *crp)
                     status, wcs_errmsg[status]);
 
           /* Find the first and last pixels of this crop. */
-          gal_box_border_from_center(pixcrd, ndim, p->iwidth, fpixel, lpixel);
+          gal_box_border_from_center(pixcrd, ndim, p->iwidth, fpixel,
+                                     lpixel);
         }
       break;
 
@@ -502,6 +513,54 @@ onecrop_flpixel(struct onecropparams *crp)
 
 
 
+static void
+one_crop_make_array_0th(struct onecropparams *crp, char *outname,
+                        long *fpixel_i, long *lpixel_i, long *naxes)
+{
+  struct cropparams *p=crp->p;
+
+  int status=0;
+  fitsfile *fptr;
+  char basekeyname[FLEN_KEYWORD-5];
+  size_t i, j=0, ndim=p->imgs->ndim;
+  struct inputimgs *img=&p->imgs[crp->in_ind];
+  char region[FLEN_VALUE], regionkey[FLEN_KEYWORD];
+
+  /* Create the file (after this function returns, the rest assumes that
+     the file is ready). */
+  if(p->primaryimghdu)
+    {
+      if(fits_create_file(&fptr, outname, &status))
+        gal_fits_io_error(status, "creating file");
+      fits_close_file(fptr, &status);
+    }
+  else
+    {
+      /* Write the selected region of this image as a string to include
+         as a FITS keyword. Then we want to delete the last coma ','.*/
+      for(i=0;i<ndim;++i)
+        j += sprintf(&region[j], "%ld:%ld,", fpixel_i[i], lpixel_i[i]);
+      region[j-1]='\0';
+
+      /* Add the specific cropping parameters. */
+      gal_fits_key_list_title_add_end(&p->cp.ckeys, "Crop metadata", 0);
+      sprintf(basekeyname, "ICF%zu", crp->numimg);
+      gal_fits_key_write_filename(basekeyname, img->name, &p->cp.ckeys,
+                                  0, p->cp.quiet);
+      sprintf(regionkey, "%sPIX", basekeyname);
+      gal_fits_key_list_add_end(&p->cp.ckeys, GAL_TYPE_STRING, regionkey,
+                                0, region, 0, "Range of pixels used "
+                                "for this output.", 0, NULL, 0);
+
+      /* Write the 0-th HDU. */
+      gal_fits_key_write(p->cp.ckeys, outname, "0", "NONE", 0, 1);
+    }
+}
+
+
+
+
+
 
 /* Find the size of the final FITS image (irrespective of how many
    crops will be needed for it) and make the image to keep the
@@ -516,27 +575,21 @@ static void
 onecrop_make_array(struct onecropparams *crp, long *fpixel_i,
                    long *lpixel_i, long *fpixel_c, long *lpixel_c)
 {
+  struct cropparams *p=crp->p;
+
   double crpix;
   fitsfile *ofp;
   long naxes[MAXDIM];
   char *outname=crp->name;
-  int status=0, type=crp->p->type;
+  int status=0, type=p->type;
   char **strarr, cpname[FLEN_KEYWORD];
+  size_t i, ndim=p->imgs->ndim, totsize;
   gal_data_t *rkey=gal_data_array_calloc(1);
-  size_t i, ndim=crp->p->imgs->ndim, totsize;
-  char *cp, *cpf, blankrec[80], titlerec[80];
-  struct inputimgs *img=&crp->p->imgs[crp->in_ind];
-
-
-  /* Set the last element of the blank array. */
-  cpf=blankrec+79;
-  *cpf='\0';
-  titlerec[79]='\0';
-  cp=blankrec; do *cp=' '; while(++cp<cpf);
+  struct inputimgs *img=&p->imgs[crp->in_ind];
 
 
   /* Set the size of the output, in WCS mode, noblank==0. */
-  if(crp->p->noblank && crp->p->mode==IMGCROP_MODE_IMG)
+  if(p->noblank && p->mode==IMGCROP_MODE_IMG)
     for(i=0;i<ndim;++i)
       {
         fpixel_c[i] = 1;
@@ -549,16 +602,12 @@ onecrop_make_array(struct onecropparams *crp, long *fpixel_i,
 
   /* Create the FITS file with a blank first extension, then close it, so
      with the next 'fits_open_file', we build the image in the second
-     extension. This way, atleast for Gnuastro's outputs, we can
-     consistently use '-h1' (something like how you count columns, or
-     generally everything from 1). */
-  if(fits_create_file(&ofp, outname, &status))
-    gal_fits_io_error(status, "creating file");
-  if(crp->p->primaryimghdu==0)
-    {
-      fits_create_img(ofp, SHORT_IMG, 0, naxes, &status);
-      fits_close_file(ofp, &status);
-    }
+     extension. But only if the user didn't want the append the crop to an
+     existing file or if the file doesn't exist at all. This way, at least
+     for Gnuastro's outputs, we can consistently use '-h1' (something like
+     how you count columns, or generally everything from 1). */
+  if(p->append==0 || gal_checkset_check_file_return(outname)==0)
+    one_crop_make_array_0th(crp, outname, fpixel_i, lpixel_i, naxes);
 
 
   /* Create the output crop image. */
@@ -581,6 +630,12 @@ onecrop_make_array(struct onecropparams *crp, long *fpixel_i,
   status=0;
 
 
+  /* Name of extension. */
+  fits_update_key(ofp, TSTRING, "EXTNAME", p->metaname,
+                  "Name of HDU (extension).", &status);
+  gal_fits_io_error(status, "writing EXTNAME");
+
+
   /* Read the units of the input dataset and store them in the output. */
   rkey->next=NULL;
   rkey->name="BUNIT";
@@ -599,8 +654,8 @@ onecrop_make_array(struct onecropparams *crp, long *fpixel_i,
 
   /* Write the blank value as a FITS keyword if necessary. */
   if( type!=GAL_TYPE_FLOAT32 && type!=GAL_TYPE_FLOAT64 )
-    if(fits_write_key(ofp, gal_fits_type_to_datatype(crp->p->type), "BLANK",
-                      crp->p->blankptrwrite, "Pixels with no data.",
+    if(fits_write_key(ofp, gal_fits_type_to_datatype(type), "BLANK",
+                      p->blankptrwrite, "Pixels with no data.",
                       &status) )
       gal_fits_io_error(status, "adding Blank");
   totsize = naxes[0]*naxes[1] * (ndim==3?naxes[2]:1);
@@ -613,15 +668,7 @@ onecrop_make_array(struct onecropparams *crp, long *fpixel_i,
   if(img->wcs)
     {
       /* Write the WCS title and common WCS information. */
-      if(fits_write_record(ofp, blankrec, &status))
-        gal_fits_io_error(status, NULL);
-      sprintf(titlerec, "%sWCS information", GAL_FITS_KEY_TITLE_START);
-      for(i=strlen(titlerec);i<79;++i)
-        titlerec[i]=' ';
-      fits_write_record(ofp, titlerec, &status);
-      for(i=0;i<img->nwcskeys-1;++i)
-        fits_write_record(ofp, &img->wcstxt[i*80], &status);
-      gal_fits_io_error(status, NULL);
+      gal_fits_key_write_wcsstr(ofp, img->wcs, img->wcstxt, img->nwcskeys);
 
       /* Correct the CRPIX keywords. */
       for(i=0;i<ndim;++i)
@@ -632,14 +679,6 @@ onecrop_make_array(struct onecropparams *crp, long *fpixel_i,
           gal_fits_io_error(status, NULL);
         }
     }
-
-
-  /* Add the Crop information. */
-  sprintf(titlerec, "%sCrop information", GAL_FITS_KEY_TITLE_START);
-  for(i=strlen(titlerec);i<79;++i)
-    titlerec[i]=' ';
-  if(fits_write_record(ofp, titlerec, &status))
-    gal_fits_io_error(status, NULL);
 }
 
 
@@ -664,13 +703,11 @@ onecrop(struct onecropparams *crp)
   struct inputimgs *img=&p->imgs[crp->in_ind];
 
   void *array;
-  int returnvalue=1;
+  char *stdoutstring;
   int status=0, anynul=0;
-  fitsfile *ifp=crp->infits, *ofp;
-  char basekeyname[FLEN_KEYWORD-5];     /* '-5': avoid gcc 8.1+ warnings! */
-  gal_fits_list_key_t *headers=NULL;    /* See above comment for more.    */
-  size_t i, j, cropsize=1, ndim=img->ndim;
-  char region[FLEN_VALUE], regionkey[FLEN_KEYWORD];
+  int returnvalue=1, hasoneelem=1;
+  fitsfile *ifp=crp->infits, *ofp;      /* '-5': avoid gcc 8.1+ warnings! */
+  size_t i, cropsize=1, ndim=img->ndim; /* See above comment for more.    */
   long fpixel_o[MAXDIM], lpixel_o[MAXDIM], inc[MAXDIM];
   long naxes[MAXDIM], fpixel_i[MAXDIM], lpixel_i[MAXDIM];
 
@@ -692,10 +729,19 @@ onecrop(struct onecropparams *crp)
   /* Find the overlap and apply it if there is any overlap. */
   if( gal_box_overlap(naxes, fpixel_i, lpixel_i, fpixel_o, lpixel_o, ndim) )
     {
-      /* Make the output FITS image and initialize it with an array of
-         NaN or BLANK values. */
-      if(crp->outfits==NULL)
+      /* See if the output only has a single element or not. */
+      for(i=0;i<ndim;++i)
+        if(fpixel_o[i]!=1 || lpixel_o[i]!=1)
+          {hasoneelem=0; break;}
+
+      /* Make the output FITS image and initialize it with an array of NaN
+         or BLANK values. But only when '--oneelemstdout' isn't called and
+         the output is single-element. */
+      crp->outfits=NULL;
+      crp->outinstdout=0;
+      if(crp->outfits==NULL && !( p->oneelemstdout && hasoneelem) )
         onecrop_make_array(crp, fpixel_i, lpixel_i, fpixel_o, lpixel_o);
+      else crp->outinstdout=1;
       ofp=crp->outfits;
 
 
@@ -719,8 +765,8 @@ onecrop(struct onecropparams *crp)
         onecrop_zero_to_nan(array, cropsize, p->type);
 
 
-      /* If a polygon is given, remove all the pixels within or
-         outside of it.*/
+      /* If a polygon is given, remove all the pixels within or outside of
+         it.*/
       if(p->polygon)
         {
           /* A small sanity check until this part supports 3D. */
@@ -737,30 +783,30 @@ onecrop(struct onecropparams *crp)
         }
 
 
-      /* Write the array into the image. */
-      status=0;
-      if( fits_write_subset(ofp, gal_fits_type_to_datatype(p->type),
-                            fpixel_o, lpixel_o, array, &status) )
-        gal_fits_io_error(status, NULL);
+      /* Write the output (either to a file or standard output if its a
+         single element and the user asked for it). */
+      if(crp->outfits)
+        {
+          /* Write the array into the file. */
+          status=0;
+          if( fits_write_subset(ofp, gal_fits_type_to_datatype(p->type),
+                                fpixel_o, lpixel_o, array, &status) )
+            gal_fits_io_error(status, NULL);
+        }
 
 
-      /* Write the selected region of this image as a string to include as
-         a FITS keyword. Then we want to delete the last coma ','.*/
-      j=0;
-      for(i=0;i<ndim;++i)
-        j += sprintf(&region[j], "%ld:%ld,", fpixel_i[i], lpixel_i[i]);
-      region[j-1]='\0';
-
-
-      /* A section has been added to the cropped image from this input
-         image, so save the information of this image. */
-      sprintf(basekeyname, "ICF%zu", crp->numimg);
-      gal_fits_key_write_filename(basekeyname, img->name, &headers, 0);
-      sprintf(regionkey, "%sPIX", basekeyname);
-      gal_fits_key_list_add_end(&headers, GAL_TYPE_STRING, regionkey,
-                                0, region, 0, "Range of pixels used for "
-                                "this output.", 0, NULL, 0);
-      gal_fits_key_write_in_ptr(&headers, ofp);
+      /* The output should be printed in standard output. */
+      else
+        {
+          /* Convert the value into a string. If we only have a single
+             output, then print it without anything else (no identifier is
+             necessary!). Otherwise, use the desired output filename as the
+             identifier. */
+          stdoutstring=gal_type_to_string(array, p->type, 0);
+          if(p->numout==1) printf("%s\n", stdoutstring);
+          else             printf("%s %s\n", crp->name, stdoutstring);
+          free(stdoutstring);
+        }
 
 
       /* Free the allocated array. */
