@@ -5,7 +5,7 @@ Match is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2024 Free Software Foundation, Inc.
+Copyright (C) 2017-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -49,6 +49,7 @@ struct argp_option program_options[] =
 
 
 
+
     /* Outputs. */
     {
       "logasoutput",
@@ -81,14 +82,28 @@ struct argp_option program_options[] =
       UI_KEY_OUTCOLS,
       "STR",
       0,
-      "Out cols in CSV, 'a': first, 'b': second input.",
+      "Out cols in CSV, 'a': 1st, 'b': 2nd input.",
       GAL_OPTIONS_GROUP_OUTPUT,
       &p->outcols,
-      GAL_OPTIONS_NO_ARG_TYPE,
-      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_TYPE_STRLL,
+      GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET,
-      gal_options_parse_csv_strings
+      gal_options_parse_csv_strings_append
+    },
+    {
+      "arrange",
+      UI_KEY_ARRANGE,
+      "STR",
+      0,
+      "inner, outer, outer-within-aperture or full.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->arrange,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_parse_match_type,
     },
 
 
@@ -166,7 +181,7 @@ struct argp_option program_options[] =
       &p->kdtree,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET,
     },
     {
