@@ -5,7 +5,7 @@ Match is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2024 Free Software Foundation, Inc.
+Copyright (C) 2017-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -63,10 +63,11 @@ struct matchparams
   char            *input1name;  /* First input filename.                */
   char            *input2name;  /* Second input filename.               */
   char                  *hdu2;  /* Second input's HDU.                  */
+  uint8_t             arrange;  /* Type of matching: inner, outer, full */
   gal_data_t           *ccol1;  /* Array of first input column names.   */
   gal_data_t           *ccol2;  /* Array of second input column names.  */
   gal_data_t           *coord;  /* Array of manual coordinate values.   */
-  gal_data_t         *outcols;  /* Array of second input column names.  */
+  gal_list_str_t     *outcols;  /* List of given columns.               */
   gal_data_t        *aperture;  /* Acceptable matching aperture.        */
   char                *kdtree;  /* The mode to use k-d tree mode.       */
   char             *kdtreehdu;  /* k-d tree HDU when its a (FITS) file. */
@@ -74,20 +75,17 @@ struct matchparams
   uint8_t          notmatched;  /* Output is rows that don't match.     */
 
   /* Internal */
-  int                    mode;  /* Mode of operation: image or catalog. */
   gal_data_t           *cols1;  /* Column values of first input.        */
   gal_data_t           *cols2;  /* Column values of second input.       */
   gal_list_str_t       *acols;  /* Output columns from first input.     */
   gal_list_str_t       *bcols;  /* Output columns from second input.    */
   size_t                 anum;  /* Number of columns in first input.    */
   size_t                 bnum;  /* Number of columns in second input.   */
-  char               *logname;  /* Name of log file.                    */
-  char              *out1name;  /* Name of first matched output.        */
-  char              *out2name;  /* Name of second matched output.       */
   gal_list_str_t  *stdinlines;  /* Lines given by Standard input.       */
   int              kdtreemode;  /* The k-d tree mode.                   */
   gal_data_t      *kdtreedata;  /* The k-d tree data.                   */
   size_t           kdtreeroot;  /* The root node of the k-d tree.       */
+  uint8_t               *flag;  /* Problematic matches of second input. */
 
   /* Output: */
   time_t              rawtime;  /* Starting time of the program.        */
