@@ -5,7 +5,7 @@ Match is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2024 Free Software Foundation, Inc.
+Copyright (C) 2017-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -29,6 +29,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 
 #include <gnuastro/fits.h>
+#include <gnuastro/match.h>
 #include <gnuastro/threads.h>
 
 #include <gnuastro-internal/timing.h>
@@ -61,10 +62,10 @@ static char
 args_doc[] = "ASTRdata";
 
 const char
-doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" matches catalogs of objects "
-  "and (by default) will return the re-arranged matching inputs. The "
-  "optional log file will return low-level information about the match "
-  "(indexs and distances).\n"
+doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" matches catalogs of "
+  "objects and (by default) will return the re-arranged matching "
+  "inputs. The optional log file will return low-level information "
+  "about the match (indexs and distances).\n"
   GAL_STRINGS_MORE_HELP_INFO
   /* After the list of options: */
   "\v"
@@ -195,6 +196,70 @@ parse_opt(int key, char *arg, struct argp_state *state)
 
 
 
+void *
+ui_parse_match_type(struct argp_option *option, char *arg,
+                    char *filename, size_t lineno, void *junk)
+{
+  char *str=NULL;
+  uint8_t value=GAL_MATCH_ARRANGE_INVALID;
+
+  /* If we are printing values. */
+  if(lineno==-1)
+    {
+      /* The output must be an allocated string (will be 'free'd later). */
+      value=*(uint8_t *)(option->value);
+      switch(value)
+        {
+        case GAL_MATCH_ARRANGE_FULL:
+          gal_checkset_allocate_copy("full", &str);
+          break;
+        case GAL_MATCH_ARRANGE_INNER:
+          gal_checkset_allocate_copy("inner", &str);
+          break;
+        case GAL_MATCH_ARRANGE_OUTER:
+          gal_checkset_allocate_copy("outer", &str);
+          break;
+        case GAL_MATCH_ARRANGE_OUTERWITHINAPERTURE:
+          gal_checkset_allocate_copy("outer-within-aperture", &str);
+          break;
+        default:
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' "
+                "to fix the problem. %u is not a recognized match type",
+                __func__, PACKAGE_BUGREPORT, value);
+        }
+      return str;
+    }
+
+  /* If we are reading/parsing values. */
+  else
+    {
+      /* If the option is already set, just return. */
+      if(option->set) return NULL;
+
+      /* Read the value. */
+      if(      !strcmp(arg, "full") )  value = GAL_MATCH_ARRANGE_FULL;
+      else if( !strcmp(arg, "inner") ) value = GAL_MATCH_ARRANGE_INNER;
+      else if( !strcmp(arg, "outer") ) value = GAL_MATCH_ARRANGE_OUTER;
+      else if( !strcmp(arg, "outer-within-aperture") )
+        value = GAL_MATCH_ARRANGE_OUTERWITHINAPERTURE;
+      else
+        error_at_line(EXIT_FAILURE, 0, filename, lineno, "'%s' (value "
+                      "to '%s' option) couldn't be recognized as a known "
+                      "mathing type. Acceptable values are 'full', "
+                      "'inner', 'outer' or 'outer-within-aperture'", arg,
+                      option->name);
+      *(uint8_t *)(option->value)=value;
+
+      /* For no un-used variable warning. This function doesn't need the
+         pointer. */
+      return junk=NULL;
+    }
+}
+
+
+
+
+
 
 
 
@@ -213,47 +278,95 @@ parse_opt(int key, char *arg, struct argp_state *state)
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
+#define UI_ONLY_FITS_ERR \
+  "Only output FITS tables are supported in Gnuastro's Match " \
+  "program. This is because depending on the run-time options, " \
+  "the number of output tables can be very different and it is " \
+  "easiest to manage them (for the user and developer!) as " \
+  "sparate HDUs of a single FITS file. If you need a final " \
+  "output table as plain-text, you can use Gnuastro's Table " \
+  "program with a command like this: 'asttable --hdu=N table.fits " \
+  "--output=table.txt' (to extract the N-th HDU (counting from " \
+  "0) and write it as a plain-text table. You can see the list " \
+  "of HDUs in a FITS file with 'astfits table.fits'"
+
+
+
+
+
 /* Check ONLY the options. When arguments are involved, do the check
    in 'ui_check_options_and_arguments'. */
 static void
 ui_check_only_options(struct matchparams *p)
 {
-  /* k-d tree specific sanity checks. */
-  if(p->kdtree)
-  {
-    /* Set the k-d tree mode. */
-    if(      !strcmp(p->kdtree,"build")    ) p->kdtreemode=MATCH_KDTREE_BUILD;
-    else if( !strcmp(p->kdtree,"internal") ) p->kdtreemode=MATCH_KDTREE_INTERNAL;
-    else if( !strcmp(p->kdtree,"disable")  ) p->kdtreemode=MATCH_KDTREE_DISABLE;
-    else if( gal_fits_name_is_fits(p->kdtree) ) p->kdtreemode=MATCH_KDTREE_FILE;
-    else
-      error(EXIT_FAILURE, 0, "'%s' is not valid for '--kdtree'. The "
-            "following values are accepted: 'build' (to build the k-d tree in "
-            "the file given to '--output'), 'internal' (to force internal "
-            "usage of a k-d tree for the matching), 'disable' (to not use a "
-            "k-d tree at all), a FITS file name (the file to read a created "
-            "k-d tree from)", p->kdtree);
+  /* Set the k-d tree mode. */
+  if(      !strcmp(p->kdtree,"build")    )
+    p->kdtreemode=MATCH_KDTREE_BUILD;
+  else if( !strcmp(p->kdtree,"internal") )
+    p->kdtreemode=MATCH_KDTREE_INTERNAL;
+  else if( !strcmp(p->kdtree,"disable")  )
+    p->kdtreemode=MATCH_KDTREE_DISABLE;
+  else if( gal_fits_name_is_fits(p->kdtree) )
+    p->kdtreemode=MATCH_KDTREE_FILE;
+  else
+    error(EXIT_FAILURE, 0, "'%s' is not valid for '--kdtree'. The "
+          "following values are accepted: 'build' (to build the k-d "
+          "tree in the file given to '--output'), 'internal' (to "
+          "force internal usage of a k-d tree for the matching), "
+          "'disable' (to not use a k-d tree at all), a FITS file "
+          "name (the file to read a created k-d tree from)", p->kdtree);
 
-    /* Make sure that the k-d tree build mode is not called with
-       '--outcols'. */
-    if( p->kdtreemode==MATCH_KDTREE_BUILD && (p->outcols || p->coord) )
-      error(EXIT_FAILURE, 0, "the '--kdtree=build' option is incompatible "
-            "with the '--outcols' or '--coord' options (because in the k-d "
-            "tree building mode doesn't involve actual matching. It will "
-            "only build k-d tree and write it to a file so it can be used "
-            "in future matches)");
+  /* In case '--logasoutput' is given, we also need to activate '--log',
+     otherwise, the output will not have a log-table at all. */
+  if(p->logasoutput) p->cp.log=1;
 
-    /* Make sure that a HDU is also specified for the k-d tree when its an
-       external file. */
-    if( p->kdtreemode==MATCH_KDTREE_FILE && p->kdtreehdu==NULL )
-      error(EXIT_FAILURE, 0, "no '--kdtreehdu' specified! When a FITS "
-            "file is given to '--kdtree', you should specify its "
-            "extension/HDU within the file using '--kdtreehdu'. You "
-            "can either use the HDU name, or its number (counting "
-            "from 0). If you aren't sure what HDUs exist in the file, "
-            "you can use the 'astfits %s' command to see the full list",
-            p->kdtree);
-  }
+  /* Make sure that the k-d tree build mode is not called with
+     '--outcols'. */
+  if( p->kdtreemode==MATCH_KDTREE_BUILD && (p->outcols || p->coord) )
+    error(EXIT_FAILURE, 0, "the '--kdtree=build' option is "
+          "incompatible with the '--outcols' or '--coord' options "
+          "(because in the k-d tree building mode doesn't involve "
+          "actual matching. It will only build k-d tree and write "
+          "it to a file so it can be used in future matches)");
+
+  /* Make sure that a HDU is also specified for the k-d tree when its an
+     external file. */
+  if( p->kdtreemode==MATCH_KDTREE_FILE && p->kdtreehdu==NULL )
+    error(EXIT_FAILURE, 0, "no '--kdtreehdu' specified! When a FITS "
+          "file is given to '--kdtree', you should specify its "
+          "extension/HDU within the file using '--kdtreehdu'. You "
+          "can either use the HDU name, or its number (counting "
+          "from 0). If you aren't sure what HDUs exist in the file, "
+          "you can use the 'astfits %s' command to see the full list",
+          p->kdtree);
+
+  /* For an outer match, we only operate in k-d tree mode. */
+  if( (   p->arrange==GAL_MATCH_ARRANGE_OUTER
+       || p->arrange==GAL_MATCH_ARRANGE_OUTERWITHINAPERTURE )
+      && p->kdtreemode==MATCH_KDTREE_DISABLE )
+    error(EXIT_FAILURE, 0, "the 'outer' arrangement only works with the "
+          "k-d tree algorithm (in other words, it conflicts with "
+          "'--kdtree=disable' option). Please set '--kdtree=internal' "
+          "(if you want to build the k-d tree very time), or to "
+          "'--kdtree=file.fits' (where 'file.fits' is an existing k-d "
+          "tree that was built once and can be used many times for "
+          "a faster result on a repeated reference catalog). See the "
+          "description of the '--kdtree' option of the Match program "
+          "in Gnuastro manual for more (by running 'info %s')",
+          PROGRAM_EXEC);
+
+  /* The '--notmatched' option is not compatible with some options. */
+  if(p->notmatched)
+    {
+      if(p->arrange!=GAL_MATCH_ARRANGE_INNER)
+        error(EXIT_FAILURE, 0, "the '--notmatched' option is only "
+              "defined with the '--arrange=inner' match type");
+    }
+
+  /* Match will only produce FITS images. */
+  if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
+    error(EXIT_FAILURE, 0, "the value of '--tableformat' cannot be 'txt'"
+          UI_ONLY_FITS_ERR);
 }
 
 
@@ -347,79 +460,6 @@ ui_check_options_and_arguments(struct matchparams *p)
 /**************************************************************/
 /***************       Preparations         *******************/
 /**************************************************************/
-static void
-ui_set_mode(struct matchparams *p)
-{
-  int tin1, tin2;
-  char *t1=NULL, *t2=NULL;
-
-  /* We will base the mode on the first input, then check with the
-     second. Note that when the first is from standard input (it is
-     'NULL'), then we go into catalog mode because currently we assume
-     standard input is only for plain text and WCS matching is not defined
-     on plain text. */
-  if( p->input1name && gal_fits_file_recognized(p->input1name) )
-    {
-      tin1=gal_fits_hdu_format(p->input1name, p->cp.hdu, "--hdu");
-      p->mode = (tin1 == IMAGE_HDU) ? MATCH_MODE_WCS : MATCH_MODE_CATALOG;
-    }
-  else
-    {
-      tin1=ASCII_TBL; /* For "uninitialized" warning, irrelevant here. */
-      p->mode=MATCH_MODE_CATALOG;
-    }
-
-
-  /* Necessary sanity checks. */
-  if(p->mode==MATCH_MODE_CATALOG && p->cp.searchin==0)
-    error(EXIT_FAILURE, 0, "no '--searchin' option specified. Please run "
-          "the following command for more information:\n\n"
-          "    $ info gnuastro \"selecting table columns\"\n");
-
-
-  /* Now that the mode is set, do some sanity checks on the second
-     catalog. Recall that when '--coord' is given, there is no second input
-     file.*/
-  if(p->input2name)
-    {
-      if(gal_fits_file_recognized(p->input2name))
-        {
-          tin2=gal_fits_hdu_format(p->input2name, p->hdu2, "--hdu2");
-          if(tin1==IMAGE_HDU && tin2!=IMAGE_HDU)
-            {
-              t1 = (tin1==IMAGE_HDU) ? "image" : "catalog";
-              t2 = (tin2==IMAGE_HDU) ? "image" : "catalog";
-            }
-          if(t1)
-            error(EXIT_FAILURE, 0, "%s is a %s, while %s is an "
-                  "%s. Both inputs have to be images or catalogs",
-                  gal_checkset_dataset_name(p->input1name, p->cp.hdu), t1,
-                  gal_checkset_dataset_name(p->input2name, p->hdu2), t2 );
-        }
-      else
-        {
-          if(p->mode==MATCH_MODE_WCS)
-            error(EXIT_FAILURE, 0, "%s is an image, while %s is a catalog! "
-                  "Both inputs have to be images or catalogs",
-                  gal_checkset_dataset_name(p->input1name, p->cp.hdu),
-                  gal_checkset_dataset_name(p->input2name, p->hdu2) );
-        }
-    }
-  else
-    {
-      /* When there is no second-input file name ('--coord' is given), we
-         cannot be in WCS mode (requiring a FITS file). */
-      if(p->mode==MATCH_MODE_WCS)
-        error(EXIT_FAILURE, 0, "%s is an image, while '--coord' is only "
-              "meaningful for catalogs",
-              gal_checkset_dataset_name(p->input1name, p->cp.hdu));
-    }
-}
-
-
-
-
-
 /* The final aperture must have the following values:
 
        p->aperture[0]: Major axis length.
@@ -467,13 +507,15 @@ ui_read_columns_aperture_2d(struct matchparams *p)
     case 3:
       if(oaper[1]>1)
         error(EXIT_FAILURE, 0, "second value to '--aperture' is larger "
-              "than one. When three numbers are given to this option, the "
-              "second is the axis ratio (which must always be less than 1).");
+              "than one. When three numbers are given to this option, "
+              "the second is the axis ratio (which must always be less "
+              "than 1).");
       break;
 
     default:
-      error(EXIT_FAILURE, 0, "%zu values given to '--aperture'. In 2D, this "
-            "option can only take 1, 2, or 3 values", p->aperture->size);
+      error(EXIT_FAILURE, 0, "%zu values given to '--aperture'. In 2D, "
+            "this option can only take 1, 2, or 3 values",
+            p->aperture->size);
     }
 
   /* If a new aperture was defined, then replace it with the exitsting
@@ -600,10 +642,11 @@ ui_read_columns_aperture_3d(struct matchparams *p)
       break;
 
     default:
-      error(EXIT_FAILURE, 0, "%zu values given to '--aperture'. In 3D, this "
-            "option can only take 1, 3, or 6 values. See the description of "
-            "this option in the book for more with this command:\n\n"
-            "    $ info astmatch\n", p->aperture->size);
+      error(EXIT_FAILURE, 0, "%zu values given to '--aperture'. In 3D, "
+            "this option can only take 1, 3, or 6 values. See the "
+            "description of this option in the book for more with this "
+            "command:\n\n"
+            "    $ info %s\n", p->aperture->size, PROGRAM_EXEC);
     }
 
   /* If a new aperture was defined, then replace it with the exitsting
@@ -658,29 +701,49 @@ ui_set_columns_sanity_check_read_aperture(struct matchparams *p)
 
   /* Read/check the aperture values. */
   if(p->aperture)
-    switch(ccol1n)
-      {
-      case 1:
-        if(p->aperture->size>1)
-          error(EXIT_FAILURE, 0, "%zu values given to '--aperture'. In "
-                "a 1D match, this option can only take one value",
-                p->aperture->size);
-        break;
+    {
+      /* If we are in an outer arrangement, an aperture should not be
+         given! */
+      if(p->arrange==GAL_MATCH_ARRANGE_OUTER)
+        error(EXIT_FAILURE, 0, "no aperture should be given in an "
+              "\"outer\" arrangement. For more information, see the "
+              "\"Arranging match output\" section of the manual "
+              "(e.g., by running the command 'info gnuastro "
+              "arranging')");
 
-      case 2: ui_read_columns_aperture_2d(p); break;
-      case 3: ui_read_columns_aperture_3d(p); break;
-      default:
-        error(EXIT_FAILURE, 0, "%zu dimensional matches are not currently "
-              "supported (maximum is 2 dimensions). The number of "
-              "dimensions is deduced from the number of values given to "
-              "'--ccol1' (or '--coord') and '--ccol2'", ccol1n);
-      }
-  else if ( p->kdtreemode != MATCH_KDTREE_BUILD )
-    error(EXIT_FAILURE, 0, "no matching aperture specified. Please use "
-          "the '--aperture' option to define the acceptable aperture for "
-          "matching the coordinates (in the same units as each "
-          "dimension). Please run the following command for more "
-          "information.\n\n    $ info %s\n", PROGRAM_EXEC);
+      /* Read the aperture depending on the number of dimensions. */
+      switch(ccol1n)
+        {
+        case 1:
+          if(p->aperture->size>1)
+            error(EXIT_FAILURE, 0, "%zu values given to '--aperture'. "
+                  "In a 1D match, this option can only take one value",
+                  p->aperture->size);
+          break;
+
+        case 2: ui_read_columns_aperture_2d(p); break;
+        case 3: ui_read_columns_aperture_3d(p); break;
+        default:
+          error(EXIT_FAILURE, 0, "%zu dimensional matches are not "
+                "currently supported (maximum is 2 dimensions). The "
+                "number of dimensions is deduced from the number of "
+                "values given to '--ccol1' (or '--coord') and "
+                "'--ccol2'", ccol1n);
+        }
+    }
+
+  /* No aperture is given; this is only possible in the following
+     modes. Otherwise, we need to abort the program and inform the user to
+     provide an aperture. */
+  else if ( p->kdtreemode != MATCH_KDTREE_BUILD
+            && p->arrange != GAL_MATCH_ARRANGE_OUTER)
+    error(EXIT_FAILURE, 0, "no matching aperture specified. Please "
+          "use the '--aperture' option to define the acceptable "
+          "aperture for matching the coordinates (in the same units "
+          "as each dimension). An aperture is not necessary when you "
+          "are building a k-d tree or when you want an outer match; "
+          "but this is not any of those scenarios. Please run 'info "
+          "%s' for more on the format of '--aperture'", PROGRAM_EXEC);
 
   /* Return the number of dimensions. */
   return ccol1n;
@@ -788,8 +851,9 @@ ui_read_columns_to_double(struct matchparams *p, char *filename,
 #define UI_READ_KDTREE_FIXED_MSG "You can construct a k-d tree "        \
   "for your first input table using the command below (just set your "  \
   "desired output name, and give that file to '--kdtree', in a later "  \
-  "call with the same first input catalog):\n\n"                  \
-  "    astmatch %s -h%s --ccol1=%s,%s --kdtree=build --output=KDTREE.fits\n\n" \
+  "call with the same first input catalog):\n\n"                        \
+  "    astmatch %s -h%s --ccol1=%s,%s --kdtree=build "                  \
+  "--output=KDTREE.fits\n\n"                                            \
   "To learn more about the expected format of k-d trees in Gnuastro, "  \
   "please run this command: 'info gnuastro k-d'"
 
@@ -891,20 +955,23 @@ ui_read_columns(struct matchparams *p)
     ui_read_kdtree(p);
 
   /* If we are in k-d tree based matching and the second dataset is smaller
-     than the first, print a warning to let the user know that the speed
+     than the first, and we are not in an outer-match (where the order of
+     the inputs matter) print a warning to let the user know that the speed
      can be greatly improved if they swap the two. */
   if( !p->cp.quiet
       && p->kdtreemode!=MATCH_KDTREE_BUILD
       && p->kdtreemode!=MATCH_KDTREE_DISABLE
-      && p->cols1->size > (2*p->cols2->size) )
-    error(EXIT_SUCCESS, 0, "TIP: the matching speed will GREATLY IMPROVE "
-          "if you swap the two inputs. Currently the second input has "
-          "fewer rows than the first. In the k-d tree based matching, "
-          "multi-threading will occur on the second input's rows and "
-          "the k-d will be constructed for the first input. Fewer "
-          "first-input rows therefore means a smaller tree, and more "
-          "second-input rows will be better distributed in your "
-          "system's CPU threads");
+      && p->cols1->size > (2*p->cols2->size)
+      && p->arrange!=GAL_MATCH_ARRANGE_OUTER)
+    error(EXIT_SUCCESS, 0, "TIP: the matching speed will improve "
+          "if you swap the two inputs. Recall that for an inner or "
+          "full match, the order of inputs does not matter. "
+          "In the k-d tree based matching, a k-d tree will be "
+          "constructed for the first input and parallelized matching "
+          "will occur on the second input's rows. Fewer first-input "
+          "rows therefore mean a smaller k-d tree (which is easier "
+          "to build), and more second-input rows will be better "
+          "distributed in your system's processing threads");
 
   /* Free the extra spaces. */
   gal_list_str_free(cols1, 1);
@@ -915,25 +982,43 @@ ui_read_columns(struct matchparams *p)
 
 
 
+static void
+ui_preparations_out_cols_read(gal_list_str_t **list, char *string)
+{
+  /* Make sure the string is not empty. */
+  if(string[0]=='\0')
+    error(EXIT_FAILURE, 0, "the value(s) given to '--outcols' should "
+          "a column identifier (counter, name or '_all') after the 'a' "
+          "or 'b'");
+
+  /* Add the string to the list of columns for this column. */
+  gal_list_str_add(list, string, 0);
+}
+
+
+
+
+
 static void
 ui_preparations_out_cols(struct matchparams *p)
 {
+  char *col;
   void *rptr;
   int goodvalue;
   gal_data_t *read;
   uint8_t readtype;
-  char *col, **strarr=p->outcols->array;
-  size_t i, one=1, ndim=p->coord?p->coord->size:0;
+  gal_list_str_t *ocol;
+  size_t one=1, ndim=p->coord?p->coord->size:0;
 
   /* Go over all the values and put the respective column identifier in the
      proper list. */
-  for(i=0;i<p->outcols->size;++i)
+  for(ocol=p->outcols; ocol!=NULL; ocol=ocol->next)
     {
       /* For easy reading. */
-      col=strarr[i];
+      col=ocol->v;
 
-      /* In no-match mode, then the same column will be used from both
-         catalogs so things are easier. */
+      /* In no-match mode, the same column will be used from both catalogs
+         so things are easier. */
       if(p->notmatched)
         {
           gal_list_str_add(&p->acols, col, 0);
@@ -945,7 +1030,9 @@ ui_preparations_out_cols(struct matchparams *p)
       else
         switch(col[0])
           {
-          case 'a': gal_list_str_add(&p->acols, col+1, 0); break;
+          case 'a':
+            ui_preparations_out_cols_read(&p->acols, col+1);
+            break;
           case 'b':
             /* With '--coord', only numbers that are smaller than the
                number of the dimensions are acceptable. */
@@ -976,7 +1063,7 @@ ui_preparations_out_cols(struct matchparams *p)
                         "the number of dimensions (%zu in this case) "
                         "are acceptable", col+1, ndim);
               }
-            gal_list_str_add(&p->bcols, col+1, 0);
+            ui_preparations_out_cols_read(&p->bcols, col+1);
             break;
           default:
             error(EXIT_FAILURE, 0, "'%s' is not a valid value for "
@@ -1002,122 +1089,26 @@ ui_preparations_out_cols(struct matchparams *p)
 
 
 
-#define UI_OUT_SUFFNAME "-matched"
 static void
 ui_preparations_out_name(struct matchparams *p)
 {
   /* To temporarily keep the original value. */
-  char *suffix;
-  uint8_t keepinputdir_orig;
   char *refname = p->input1name ? p->input1name : p->input2name;
 
-  /* Set the output file(s) name(s). */
-  if(p->logasoutput)
+  /* If the output name is set, make sure it is a FITS file, otherwise, set
+     a name from the input. */
+  if(p->cp.output)
     {
-      if(p->cp.output)
-        gal_checkset_allocate_copy(p->cp.output, &p->logname);
-      else
-        {
-          if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
-            p->logname=gal_checkset_automatic_output(&p->cp, refname,
-                                                     UI_OUT_SUFFNAME".txt");
-          else
-            p->logname=gal_checkset_automatic_output(&p->cp, refname,
-                                                     UI_OUT_SUFFNAME".fits");
-        }
-
-      /* Make sure a file with this name doesn't exist. */
-      gal_checkset_writable_remove(p->logname, NULL, 0, p->cp.dontdelete);
-
-      /* The main output name needs to be available in p->out1name (for the
-         final step when we want to write the input configurations as FITS
-         keywords). */
-      gal_checkset_allocate_copy(p->logname, &p->out1name);
+      if( ! gal_fits_name_is_fits(p->cp.output) )
+        error(EXIT_FAILURE, 0, "'%s' is not a recognized FITS file "
+              "name. " UI_ONLY_FITS_ERR, p->cp.output);
     }
   else
-    {
-      if(p->outcols || p->coord || p->kdtreemode==MATCH_KDTREE_BUILD)
-        {
-          if(p->cp.output)
-            gal_checkset_allocate_copy(p->cp.output, &p->out1name);
-          else
-            {
-              suffix = ( p->kdtreemode==MATCH_KDTREE_BUILD
-                         ? "-kdtree.fits"
-                         : ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
-                             ? UI_OUT_SUFFNAME".txt"
-                             : UI_OUT_SUFFNAME".fits") );
-              p->out1name = gal_checkset_automatic_output(&p->cp,
-                                                          refname, suffix);
-            }
-          gal_checkset_writable_remove(p->out1name, refname, 0,
-                                       p->cp.dontdelete);
-        }
-      else
-        {
-          /* Set 'p->out1name' and 'p->out2name'. */
-          if(p->cp.output)
-            {
-              if( gal_fits_name_is_fits(p->cp.output) )
-                {
-                  gal_checkset_allocate_copy(p->cp.output, &p->out1name);
-                  gal_checkset_allocate_copy(p->cp.output, &p->out2name);
-                }
-              else
-                {
-                  /* Here, we are be using the output name as input to the
-                     automatic output generating function (usually it is
-                     the input name, not the output name). Therefore, the
-                     'keepinputdir' variable should be 1. So we will
-                     temporarily change it here, then set it back to what
-                     it was. */
-                  keepinputdir_orig=p->cp.keepinputdir;
-                  p->cp.keepinputdir=1;
-                  p->out1name=gal_checkset_automatic_output(&p->cp,
-                                                            p->cp.output,
-                                                            UI_OUT_SUFFNAME
-                                                            "-1.txt");
-                  p->out2name=gal_checkset_automatic_output(&p->cp,
-                                                            p->cp.output,
-                                                            UI_OUT_SUFFNAME
-                                                            "-2.txt");
-                  p->cp.keepinputdir=keepinputdir_orig;
-                }
-            }
-          else
-            {
-              if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
-                {
-                  p->out1name=gal_checkset_automatic_output(&p->cp, refname,
-                                                            UI_OUT_SUFFNAME
-                                                            "-1.txt");
-                  p->out2name=gal_checkset_automatic_output(&p->cp,
-                                                            p->input2name,
-                                                            UI_OUT_SUFFNAME
-                                                            "-2.txt");
-                }
-              else
-                {
-                  p->out1name=gal_checkset_automatic_output(&p->cp, refname,
-                                                            UI_OUT_SUFFNAME
-                                                            ".fits");
-                  gal_checkset_allocate_copy(p->out1name, &p->out2name);
-                }
-            }
+    p->cp.output=gal_checkset_automatic_output(&p->cp, refname,
+                                               "-matched.fits");
 
-          /* Make sure no file with these names exists. */
-          gal_checkset_writable_remove(p->out1name, refname, 0,
-                                       p->cp.dontdelete);
-          gal_checkset_writable_remove(p->out2name, refname, 0,
-                                       p->cp.dontdelete);
-        }
-
-      /* If a log file is necessary, set its name here. */
-      if(p->cp.log)
-        p->logname=gal_checkset_automatic_output(&p->cp, p->out1name,
-                                                 UI_OUT_SUFFNAME
-                                                 "-log.fits");
-    }
+  /* Make sure a file with this name doesn't exist. */
+  gal_checkset_writable_remove(p->cp.output, NULL, 0, p->cp.dontdelete);
 }
 
 
@@ -1127,20 +1118,41 @@ ui_preparations_out_name(struct matchparams *p)
 static void
 ui_preparations(struct matchparams *p)
 {
-  /* Set the mode of the program. */
-  ui_set_mode(p);
+  int intype;
 
-  /* Currently Match only works on catalogs. */
-  if(p->mode==MATCH_MODE_WCS)
-    error(EXIT_FAILURE, 0, "currently Match only works on catalogs, "
-          "we will implement the WCS matching routines later");
-  else
+  /* Since Match only works on tables, 'searchin' is necessary. */
+  if(p->cp.searchin==0)
+    error(EXIT_FAILURE, 0, "no '--searchin' option specified. Please "
+          "run the following command for more information:\n\n"
+          "    $ info gnuastro \"selecting table columns\"\n");
+
+  /* We will base the mode on the first input, then check with the
+     second. Note that when the first is from standard input (it is
+     'NULL'), then we go into catalog mode because currently we assume
+     standard input is only for plain text and WCS matching is not defined
+     on plain text. */
+  if( p->input1name && gal_fits_file_recognized(p->input1name) )
     {
-      ui_read_columns(p);
-      if(p->outcols)
-        ui_preparations_out_cols(p);
+      intype=gal_fits_hdu_format(p->input1name, p->cp.hdu, "--hdu");
+      if(intype==IMAGE_HDU)
+        error(EXIT_FAILURE, 0, "%s: not a catalog",
+              gal_checkset_dataset_name(p->input1name, p->cp.hdu));
     }
 
+  /* Same check for second input. */
+  if(p->input2name && gal_fits_file_recognized(p->input2name))
+    {
+      intype=gal_fits_hdu_format(p->input2name, p->hdu2, "--hdu2");
+      if(intype==IMAGE_HDU)
+        error(EXIT_FAILURE, 0, "%s: not a catalog",
+              gal_checkset_dataset_name(p->input2name, p->hdu2));
+    }
+
+  /* Read the input columns and prepare output columns. */
+  ui_read_columns(p);
+  if(p->outcols)
+    ui_preparations_out_cols(p);
+
   /* Set the output filename. */
   ui_preparations_out_name(p);
 }
@@ -1219,9 +1231,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct matchparams *p)
 
 
   /* If the output is a FITS table, prepare all the options as FITS
-     keywords to write in output later. */
-  if(gal_fits_name_is_fits(p->out1name))
-    gal_options_as_fits_keywords(&p->cp);
+     keywords to later write in output. */
+  gal_options_as_fits_keywords(&p->cp);
 
 
   /* Let the user know that processing has started. */
@@ -1230,12 +1241,9 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct matchparams *p)
       printf(PROGRAM_NAME" "PACKAGE_VERSION" started on %s",
              ctime(&p->rawtime));
       nthreads=p->kdtreemode==MATCH_KDTREE_DISABLE ? 1 : p->cp.numthreads;
-      printf("  - Using %zu CPU thread%s%s\n", nthreads,
-             nthreads==1 ? "." : "s.",
+      printf("  - Using %zu CPU thread%s\n", nthreads,
              ( p->kdtreemode==MATCH_KDTREE_DISABLE
                ? " (sort-based match only uses a single thread)" : ""));
-      printf("  - Match algorithm: %s\n",
-             p->kdtree ? "k-d tree" : "sort-based");
       printf("  - Input-1: %s; %zu rows\n",
              gal_fits_name_save_as_string(p->input1name, p->cp.hdu),
              p->cols1->size);
@@ -1278,17 +1286,15 @@ ui_free_report(struct matchparams *p, struct timeval *t1)
   /* Free the allocated arrays: */
   free(p->cp.hdu);
   free(p->aperture);
-  free(p->out1name);
-  free(p->out2name);
   free(p->cp.output);
   gal_data_free(p->ccol1);
   gal_data_free(p->ccol2);
-  gal_data_free(p->outcols);
   gal_list_data_free(p->cols1);
   gal_list_data_free(p->cols2);
   if(p->kdtree) free(p->kdtree);
   gal_list_str_free(p->acols, 0);
   gal_list_str_free(p->bcols, 0);
+  gal_list_str_free(p->outcols, 1);
   if(p->kdtreehdu) free(p->kdtreehdu);
   gal_list_str_free(p->stdinlines, 1);
 
