@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2024 Free Software Foundation, Inc.
+Copyright (C) 2016-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -163,13 +163,13 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "forcereadstd",
-      UI_KEY_FORCEREADSTD,
+      "novalinerror",
+      UI_KEY_NOVALINERRORS,
       0,
       0,
-      "Read STD even if no columns need it.",
+      "Ignore pixel values when estimating errors.",
       GAL_OPTIONS_GROUP_INPUT,
-      &p->forcereadstd,
+      &p->novalinerror,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
@@ -232,32 +232,6 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
-    {
-      "sfmagnsigma",
-      UI_KEY_SFMAGNSIGMA,
-      "FLT",
-      0,
-      "Surface brightness multiple of Sky STD.",
-      GAL_OPTIONS_GROUP_OUTPUT,
-      &p->sfmagnsigma,
-      GAL_TYPE_FLOAT32,
-      GAL_OPTIONS_RANGE_GT_0,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "sfmagarea",
-      UI_KEY_SFMAGAREA,
-      "FLT",
-      0,
-      "Surface brightness area (in arcseconds^2).",
-      GAL_OPTIONS_GROUP_OUTPUT,
-      &p->sfmagarea,
-      GAL_TYPE_FLOAT32,
-      GAL_OPTIONS_RANGE_GT_0,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
     {
       "inbetweenints",
       UI_KEY_INBETWEENINTS,
@@ -271,6 +245,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "meta-measures",
+      UI_KEY_METAMEASURES,
+      0,
+      0,
+      "Measures on/about indiv. measures (e.g., limits).",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->metameasures,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
 
 
@@ -1079,7 +1066,7 @@ struct argp_option program_options[] =
       UI_KEY_SUMERROR,
       0,
       0,
-      "Error (1-sigma) in measuring sum.",
+      "Standard deviation (error) in measuring sum.",
       UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
@@ -1130,6 +1117,20 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_column_codes_ll
     },
+    {
+      "mean-error",
+      UI_KEY_MEANERROR,
+      0,
+      0,
+      "Error of the mean in object/clump.",
+      UI_GROUP_COLUMNS_BRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
     {
       "std",
       UI_KEY_STD,
@@ -2158,6 +2159,92 @@ struct argp_option program_options[] =
 
 
 
+
+    {
+      0, 0, 0, 0,
+      "Metameasurements (e.g., limits):",
+      UI_GROUP_METAMEASURES
+    },
+    {
+      "sbl-sigma",
+      UI_KEY_SBLSIGMA,
+      "FLT",
+      0,
+      "Surf. bright. limit multiple of Sky STD.",
+      UI_GROUP_METAMEASURES,
+      &p->sblsigma,
+      GAL_TYPE_FLOAT32,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "sbl-area",
+      UI_KEY_SBLAREA,
+      "FLT",
+      0,
+      "Surf. bright. limit area (in arcseconds^2).",
+      UI_GROUP_METAMEASURES,
+      &p->sblarea,
+      GAL_TYPE_FLOAT32,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "nml-sigma",
+      UI_KEY_NMLSIGMA,
+      "FLT",
+      0,
+      "Noise-based mag. lim multip. of Sky STD.",
+      UI_GROUP_METAMEASURES,
+      &p->nmlsigma,
+      GAL_TYPE_FLOAT32,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "nml-area",
+      UI_KEY_NMLAREA,
+      "FLT",
+      0,
+      "Noise-based mag. lim area (in arcsec^2).",
+      UI_GROUP_METAMEASURES,
+      &p->nmlarea,
+      GAL_TYPE_FLOAT32,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "cnl-with-objects",
+      UI_KEY_CNLWITHOBJECTS,
+      0,
+      0,
+      "Use objects for confusion limit, not clumps.",
+      UI_GROUP_METAMEASURES,
+      &p->cnlwithobjects,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "cnl-check",
+      UI_KEY_CNLCHECK,
+      0,
+      0,
+      "Table as extra HDU in output with all matches.",
+      UI_GROUP_METAMEASURES,
+      &p->cnlcheck,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
     {0}
   };
 
