@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -49,15 +49,6 @@ struct mkcatalog_passparams
   gal_data_t        *vector;    /* Array of datasets for raw vectors.   */
 };
 
-void
-mkcatalog_outputs_keys_numeric(gal_fits_list_key_t **keylist, void *number,
-                               uint8_t type, char *nameliteral,
-                               char *commentliteral, char *unitliteral);
-
-void
-mkcatalog_outputs_keys_infiles(struct mkcatalogparams *p,
-                               gal_fits_list_key_t **keylist);
-
 void
 mkcatalog(struct mkcatalogparams *p);
 
