@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2024 Free Software Foundation, Inc.
+Copyright (C) 2016-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -43,6 +43,7 @@ enum program_args_groups
   UI_GROUP_COLUMNS_MORPHOLOGY,
   UI_GROUP_COLUMNS_ELLIPTICAL,
   UI_GROUP_COLUMNS_VECTOR,
+  UI_GROUP_METAMEASURES
 };
 
 
@@ -80,15 +81,14 @@ enum option_keys_enum
   UI_KEY_SKYHDU,
   UI_KEY_STDHDU,
   UI_KEY_WITHCLUMPS,
-  UI_KEY_FORCEREADSTD,
   UI_KEY_ZEROPOINT,
   UI_KEY_SIGMACLIP,
   UI_KEY_VARIANCE,
+  UI_KEY_NOVALINERRORS,
   UI_KEY_SUBTRACTSKY,
-  UI_KEY_SFMAGNSIGMA,
-  UI_KEY_SFMAGAREA,
   UI_KEY_SPECTRUM,
   UI_KEY_INBETWEENINTS,
+  UI_KEY_METAMEASURES,
   UI_KEY_UPMASKFILE,
   UI_KEY_UPMASKHDU,
   UI_KEY_UPNUM,
@@ -153,6 +153,7 @@ enum option_keys_enum
   UI_KEY_SUMNORIVER,
   UI_KEY_STD,
   UI_KEY_MEAN,
+  UI_KEY_MEANERROR,
   UI_KEY_MEDIAN,
   UI_KEY_MAXIMUM,
   UI_KEY_MAGNITUDEERROR,
@@ -211,6 +212,14 @@ enum option_keys_enum
   UI_KEY_AREAOTHERINSLICE,
   UI_KEY_SUMOTHERINSLICE,
   UI_KEY_SUMOTHERERRINSLICE,
+
+  /* Metameasures */
+  UI_KEY_SBLSIGMA,
+  UI_KEY_SBLAREA,
+  UI_KEY_NMLSIGMA,
+  UI_KEY_NMLAREA,
+  UI_KEY_CNLWITHOBJECTS,
+  UI_KEY_CNLCHECK,
 };
 
 
