@@ -5,7 +5,7 @@ MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2024 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -358,7 +358,7 @@ oneprofile_pix_by_pix(struct mkonthread *mkp)
   struct builtqueue *ibq=mkp->ibq;
   size_t ndim=ibq->image->ndim, *dsize=ibq->image->dsize;
 
-  uint8_t *byt;
+  uint8_t *byt, notfirst=0;
   gal_list_sizet_t *Q=NULL;
   int use_rand_points=1, ispeak=1;
   double tolerance=mkp->p->tolerance;
@@ -397,19 +397,21 @@ oneprofile_pix_by_pix(struct mkonthread *mkp)
     case PROFILE_GAUSSIAN:
       while(sQ)
         {
-          /* In case you want to see the status of the twosided ordered
-             queue, increasing and decreasing side by side, uncomment this
-             line. Note that there will be a lot of lines printed! */
-          /*print_tossll(lQ, sQ);*/
-
           /* Pop a pixel from the queue, convert its index into coordinates
              and use them to estimate the elliptical radius of the
-             pixel. If the pixel is outside the truncation radius, ignore
-             it. */
+             pixel. */
           p=gal_list_dosizet_pop_smallest(&lQ, &sQ, &circ_r);
           oneprofile_set_coord(mkp, p);
           oneprofile_r_el(mkp);
-          if(mkp->r > truncr) continue;
+
+          /* If the pixel is outside the truncation radius, ignore it. But
+             not if this is the first pixel: when the truncation radius can
+             be smaller than a pixel, see
+             https://savannah.gnu.org/bugs/index.php?66216 and for the
+             'notfirst' variable see
+             https://savannah.gnu.org/bugs/?66572.*/
+          if(notfirst && mkp->r > truncr) continue;
+          notfirst=1;
 
           /* Set the range for this pixel. */
           for(i=0;i<ndim;++i)
@@ -426,8 +428,8 @@ oneprofile_pix_by_pix(struct mkonthread *mkp)
 
           /* For a check:
           printf("coord: %g, %g\n", mkp->coord[0], mkp->coord[1]);
-          printf("r_rand: %g (rand: %g, center: %g)\n\n", mkp->r, array[p],
-                 approx);
+          printf("r_rand: %g (rand: %g, center: %g)\n\n", mkp->r,
+                 array[p], approx);
           */
 
           /* Save the peak flux if this is the first pixel: */
@@ -478,9 +480,15 @@ oneprofile_pix_by_pix(struct mkonthread *mkp)
           continue;
         }
 
-      /* Find the value for this pixel: */
+      /* Find the value for this pixel. */
       array[p]=profile(mkp);
 
+      /* In case the pixel value is non-zero and close to the floating
+         point limit, change the truncation radius so no more pixels go
+         this far. The non-zero check is because some profiles may actually
+         have a 0.0 value (for example the radial or custom profiles). */
+      if( array[p]!=0.0 && array[p]<10*FLT_MIN ) truncr=mkp->r;
+
       /* For a check:
       printf("r_center: %g\n", mkp->r);
       */
@@ -488,7 +496,7 @@ oneprofile_pix_by_pix(struct mkonthread *mkp)
       /* Save the peak flux if this is the first pixel: */
       if(ispeak) { mkp->peakflux=array[p]; ispeak=0; }
 
-      /* Go over the neighbours and add them to queue of elements
+      /* Go over the neighbors and add them to queue of elements
          to check. */
       GAL_DIMENSION_NEIGHBOR_OP(p, ndim, dsize, 1, dinc,
         {
