@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2024 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -202,9 +202,9 @@ threshold_write_sn_table(struct noisechiselparams *p, gal_data_t *insn,
     {
       /* Remove blank elements. */
       ind=gal_data_copy(inind);
+      gal_blank_remove(ind,1);
       sn=gal_data_copy(insn);
-      gal_blank_remove(ind);
-      gal_blank_remove(sn);
+      gal_blank_remove(sn,1);
     }
   else
     {
