@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2024 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -184,7 +184,8 @@ parse_opt(int key, char *arg, struct argp_state *state)
          'arg' will be an empty string! We don't want to account for such
          cases (and give a clear error that no input has been given). */
       if(p->inputname)
-        argp_error(state, "only one argument (input file) should be given");
+        argp_error(state, "only one argument (input image/cube) may be "
+                   "given; the extra argument is '%s'", arg);
       else
         if(arg[0]!='\0') p->inputname=arg;
       break;
@@ -446,9 +447,10 @@ ui_prepare_kernel(struct noisechiselparams *p)
 
           /* Make sure it has the same dimensions as the input. */
           if( p->kernel->ndim != p->input->ndim )
-            error(EXIT_FAILURE, 0, "%s (hdu %s): is %zuD, however, %s (%s) "
-                  "is a %zuD dataset", p->kernelname, p->khdu,
-                  p->kernel->ndim, p->inputname, p->cp.hdu, p->input->ndim);
+            error(EXIT_FAILURE, 0, "%s (hdu %s): is %zuD, however, %s "
+                  "(%s) is a %zuD dataset", p->kernelname, p->khdu,
+                  p->kernel->ndim, p->inputname, p->cp.hdu,
+                  p->input->ndim);
         }
       else
         p->kernel=NULL;
