@@ -11,7 +11,7 @@
 # Contributing authors:
 #     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 #     Samane Raji <samaneraji@gmail.com>
-# Copyright (C) 2021-2024 Free Software Foundation, Inc.
+# Copyright (C) 2021-2025 Free Software Foundation, Inc.
 #
 # Gnuastro is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -46,7 +46,7 @@ export LC_NUMERIC="en_US.UTF-8"
 
 # Default option values (can be changed with options on the command-line).
 hdu=""
-rhdu=1
+reghdu=1
 globalhdu=""
 
 # Minimum, weights, and zeropoint values
@@ -93,7 +93,6 @@ scriptname=@SCRIPT_NAME@
 
 
 
-# Output of `--usage' and `--help':
 print_usage() {
     cat <<EOF
 $scriptname: run with '--help' for list of options
@@ -128,7 +127,7 @@ experienced Gnuastro users and developers. For more information, please run:
 $scriptname options:
  Input:
   -h, --hdu=STR           HDU/extension for the input channels.
-  -t, --rhdu=STR          HDU/extension for the regions image.
+  -r, --reghdu=STR        HDU/extension for the regions image.
   -g, --globalhdu=STR/INT Use this HDU for all inputs, ignore '--hdu'.
   -w, --weight=FLT        Relative weight for each input channel.
   -m, --minimum=FLT       Minimum value for each input channel.
@@ -172,15 +171,10 @@ Report bugs to bug-gnuastro@gnu.org.
 EOF
 }
 
-
-
-
-
-# Output of `--version':
 print_version() {
     cat <<EOF
 $scriptname (GNU Astronomy Utilities) $version
-Copyright (C) 2021-2024, Free Software Foundation, Inc.
+Copyright (C) 2021-2025, Free Software Foundation, Inc.
 License GPLv3+: GNU General public license version 3 or later.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
@@ -189,7 +183,6 @@ Written/developed by Raul Infante-Sainz and Mohammad Akhlaghi.
 EOF
 }
 
-
 print_citation() {
     empty="" # needed for the ascii art!
     cat <<EOF
@@ -200,6 +193,44 @@ Citations and acknowledgement are vital for the continued work on Gnuastro.
 
 Please cite the following record(s) and add the acknowledgement statement below in your work to support us. Please note that different Gnuastro programs may have different corresponding papers. Hence, please check all the programs you used. Don't forget to also include the version as shown above for reproducibility.
 
+Main Gnuastro paper
+-------------------
+The paper below was the first published resource that introduced Gnuastro. Its focus is only three of the Gnuastro programs (NoiseChisel, Segment and MakeCatalog), but we have not yet had time write a dedicated paper for Gnuastro. Until a high-level paper that describes the whole of Gnuastro is published, this is the main paper that Gnuastro's citations will be counted against. Therefore, please cite this, even if you have not used those programs.
+
+  @ARTICLE{gnuastro,
+     author = {{Akhlaghi}, Mohammad and {Ichikawa}, Takashi},
+      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
+    journal = {ApJS},
+  archivePrefix = "arXiv",
+     eprint = {1505.01664},
+   primaryClass = "astro-ph.IM",
+   keywords = {galaxies: irregular, galaxies: photometry,
+               galaxies: structure, methods: data analysis,
+               techniques: image processing, techniques: photometric},
+       year = 2015,
+      month = sep,
+     volume = 220,
+        eid = {1},
+      pages = {1},
+        doi = {10.1088/0067-0049/220/1/1},
+     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
+    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
+  }
+
+
+Gnuastro book
+-------------
+If you want to cite any part of the book (in any of the programs), please use the BibTeX entry below:
+
+  @BOOK{gnuastrobook,
+     author = {{Akhlaghi}, Mohammad},
+      title = {GNU Astronomy Utilities (version $version)},
+       year = 2024,
+  publisher = {Free Software Foundation},
+        doi = {10.5281/zenodo.12738457}
+  }
+
+
 Paper introducing this script
 -----------------------------
   @ARTICLE{astscript-color-faint-gray,
@@ -221,24 +252,6 @@ Paper introducing this script
         adsnote = {Provided by the SAO/NASA Astrophysics Data System}
   }
 
-Paper introducing Gnuastro (currently main citation)
-----------------------------------------------------
-  @ARTICLE{gnuastro,
-     author = {{Akhlaghi}, M. and {Ichikawa}, T.},
-      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
-    journal = {ApJS},
-  archivePrefix = "arXiv",
-     eprint = {1505.01664},
-   primaryClass = "astro-ph.IM",
-       year = 2015,
-      month = sep,
-     volume = 220,
-        eid = {1},
-      pages = {1},
-        doi = {10.1088/0067-0049/220/1/1},
-     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
-    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
-  }
 
 Acknowledgement
 ---------------
@@ -318,9 +331,9 @@ do
         -h|--hdu)            aux="$2";                                  check_v "$1" "$aux"; hdu="$hdu $aux"; shift;shift;;
         -h=*|--hdu=*)        aux="${1#*=}";                             check_v "$1" "$aux"; hdu="$hdu $aux"; shift;;
         -h*)                 aux="$(echo "$1"  | sed -e's/-h//')";      check_v "$1" "$aux"; hdu="$hdu $aux"; shift;;
-        -t|--rhdu)           rhdu="$2";                                 check_v "$1" "$rhdu";  shift;shift;;
-        -t=*|--rhdu=*)       rhdu="${1#*=}";                            check_v "$1" "$rhdu";  shift;;
-        -t*)                 rhdu=$(echo "$1" | sed -e's/-t//');        check_v "$1" "$rhdu";  shift;;
+        -r|--reghdu)         reghdu="$2";                               check_v "$1" "$reghdu";  shift;shift;;
+        -r=*|--reghdu=*)     reghdu="${1#*=}";                          check_v "$1" "$reghdu";  shift;;
+        -r*)                 reghdu=$(echo "$1" | sed -e's/-r//');      check_v "$1" "$reghdu";  shift;;
         -w|--weight)         aux="$2";                                  check_v "$1" "$aux"; weight="$weight $aux"; shift;shift;;
         -w=*|--weight=*)     aux="${1#*=}";                             check_v "$1" "$aux"; weight="$weight $aux"; shift;;
         -w*)                 aux="$(echo "$1"  | sed -e's/-w//')";      check_v "$1" "$aux"; weight="$weight $aux"; shift;;
@@ -489,10 +502,10 @@ else
 fi
 
 
-# Weight. If the does not provide a value for weight, set it to 1.0. If
-# user provides a single value for --weight, use it for all channels.
-# Otherwise, check that the number of weights matches
-# with the number of inputs.
+# Weight. If the user does not provide a value for weight, set it to 1.0.
+# If the user provides a single value for --weight, use it for all
+# channels.  Otherwise, check that the number of weights matches with the
+# number of inputs.
 nweight=$(echo "$weight" | awk '{print NF}')
 if [ x$nweight = x0 ]; then
     rweight=1.0
@@ -565,7 +578,7 @@ if [ -d $tmpdir ]; then junk=1; else mkdir $tmpdir; fi
 #
 # If the user specify a given value below which the pixel values are
 # wanted to be zero, then put all of those pixels to zero. By default,
-# no clipping will be done (a symbolic link will be put to the input).
+# no clipping will be done.
 rclipped="$tmpdir/R_clipped.fits"
 gclipped="$tmpdir/G_clipped.fits"
 bclipped="$tmpdir/B_clipped.fits"
@@ -641,9 +654,9 @@ if [ x$rweight = x"1.0000" ] &&
    [ x$gweight = x"1.0000" ] &&
    [ x$bweight = x"1.0000" ] &&
    [ x"$rzero" = x ]; then
-    ln -sf $rclipped $rscaled
-    ln -sf $gclipped $gscaled
-    ln -sf $bclipped $bscaled
+    cp $rclipped $rscaled
+    cp $gclipped $gscaled
+    cp $bclipped $bscaled
 
 # If diferent weights but same zeropoints, then scale the images.
 elif [ x$rweight != x"1.0000" ] ||
@@ -888,7 +901,7 @@ else
         else
           i_back_kernel="$tmpdir/back_kernel.fits"
           astmkprof --kernel=gaussian,$graykernelfwhm,3 \
-                    --oversample=1 --output=$i_gray_kernel $quiet
+                    --oversample=1 --output=$i_back_kernel $quiet
           astconvolve $i_back --hdu=$khdu --kernel=$i_back_kernel \
                       --domain=spatial --output=$i_back_convolved $quiet
         fi
@@ -906,7 +919,7 @@ else
 
     else
         i_back_convolved=$kclipped
-        ln -sf $kclipped $i_colorgray_threshold
+        astfits $kclipped --copy=$khdu --output=$i_colorgray_threshold
     fi
 
 
@@ -955,7 +968,7 @@ else
                       i $colorval lt i $grayval gt and 1 uint8 x set-b \
                       i $colorval lt                   0 uint8 x set-g \
                       c b g 3 sum uint8 --output $total_mask
-        rhdu=1
+        reghdu=1
    else
        grayval="$regions"
        colorval="$regions"
@@ -990,8 +1003,8 @@ else
     #   function is specified. E.g., log, sqrt, asinh, etc.)
     grayscale=""
     i_gray_colormasked="$tmpdir/gray_colormasked.fits"
-    astarithmetic $i_back_convolved -h1          set-values \
-                  $total_mask -h$rhdu 2 uint8 eq set-mask \
+    astarithmetic $i_back_convolved -h$khdu set-values \
+                  $total_mask -h$reghdu 2 uint8 eq set-mask \
                   values mask nan where $grayscale set-masked \
                   masked minvalue set-oldmin \
                   masked maxvalue set-oldmax \
@@ -1014,7 +1027,7 @@ else
     # this, those pixels will be set to pure black color.
     i_gray_colormasked_zeroblack="$tmpdir/gray_colormasked_zeroblack.fits"
     astarithmetic $i_gray_colormasked -h1 set-i \
-                  $total_mask -h$rhdu 1 uint8 eq -h1 set-b \
+                  $total_mask -h$reghdu 1 uint8 eq -h1 set-b \
                   i b 0.0 where float32 \
                   --output=$i_gray_colormasked_zeroblack
 
