@@ -7,7 +7,7 @@
 # Contributing author(s):
 #     Samane Raji <samaneraji@protonmail.com>
 #     Raul Infante-Sainz <infantesainz@gmail.com>
-# Copyright (C) 2021-2024 Free Software Foundation, Inc.
+# Copyright (C) 2021-2025 Free Software Foundation, Inc.
 #
 # Gnuastro is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -43,6 +43,7 @@ mode=wcs
 radius=""
 command=""
 namecol=""
+fontsize=12
 out=ds9.reg
 color=green
 dontdelete=0
@@ -91,9 +92,10 @@ $scriptname options:
   -n, --namecol=STR       ID of each region (name or number of a column)
 
  Output:
-  -C, --color             Color for the regions (read by DS9).
-  -w, --width             Line thickness of the regions (in DS9).
-  -r, --radius            Radius of each region (arcseconds if in WCS mode).
+  -C, --color=STR         Color for the regions (read by DS9).
+  -w, --width=INT         Line thickness of the regions (in DS9).
+  -r, --radius=FLT        Radius of each region (arcseconds if in WCS mode).
+  -s, --fontsize=INT      Font size for name.
   -D, --dontdelete        Don't delete output if it exists.
   -o, --output=STR        Name of output file.
       --command="STR"     DS9 command to run after making region.
@@ -120,7 +122,7 @@ EOF
 print_version() {
     cat <<EOF
 $scriptname (GNU Astronomy Utilities) $version
-Copyright (C) 2015-2024 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 License GPLv3+: GNU General public license version 3 or later.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
@@ -212,6 +214,9 @@ do
         -r|--radius)        radius="$2";                           check_v "$1" "$radius";  shift;shift;;
         -r=*|--radius=*)    radius="${1#*=}";                      check_v "$1" "$radius";  shift;;
         -r*)                radius=$(echo "$1"  | sed -e's/-r//'); check_v "$1" "$radius";  shift;;
+        -s|--fontsize)      fontsize="$2";                           check_v "$1" "$fontsize";  shift;shift;;
+        -s=*|--fontsize=*)  fontsize="${1#*=}";                      check_v "$1" "$fontsize";  shift;;
+        -s*)                fontsize=$(echo "$1"  | sed -e's/-s//'); check_v "$1" "$fontsize";  shift;;
         -D|--dontdelete)    dontdelete=1; shift;;
         -D*|--dontdelete=*) on_off_option_error --dontdelete -D;;
         -o|--output)        out="$2";                              check_v "$1" "$out";     shift;shift;;
@@ -312,6 +317,9 @@ fi
 # Set the units of the radius.
 if [ x"$mode" = xwcs ]; then unit="\""; else unit=""; fi
 
+# Set the font size.
+font="font=\"helvetica $fontsize normal roman\""
+
 
 
 
@@ -327,7 +335,7 @@ if [ x"$namecol" != x ]; then
     printf "# Region name (or label) column: $namecol\n" >> $out
 fi
 printf "global color=%s width=%d\n" $color $width >> $out
-if [ "$mode" = wcs ]; then  printf "fk5\n" >> $out
+if [ "$mode" = wcs ]; then  printf "icrs\n" >> $out
 else                        printf "image\n" >> $out;   fi
 
 
@@ -359,13 +367,13 @@ else
         cat /dev/stdin \
             | asttable --column=$col --column=$namecol \
             | while read a b c; do \
-                  printf "circle(%f,%f,%f%s) # text={%g}\n" \
+                  printf "circle(%f,%f,%f%s) # $font text={%g}\n" \
                          $a $b $radius "$unit" $c >> $out; \
               done
     else
         asttable $input --hdu=$hdu --column=$col --column=$namecol \
             | while read a b c; do \
-                  printf "circle(%f,%f,%f%s) # text={%g}\n" \
+                  printf "circle(%f,%f,%f%s) # $font text={%g}\n" \
                          $a $b $radius "$unit" $c >> $out; \
               done
     fi
