@@ -5,7 +5,7 @@
 # Original author:
 #     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
-# Copyright (C) 2023-2024 Free Software Foundation, Inc.
+# Copyright (C) 2023-2025 Free Software Foundation, Inc.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ numthreads=0
 version=@VERSION@
 hook_warp_after=""
 hook_warp_before=""
-stack_operator="sum"
+coadd_operator="sum"
 scriptname=@SCRIPT_NAME@
 ctype="RA---TAN,DEC--TAN"
 output=pointing-simulate.fits
@@ -61,8 +61,8 @@ Usage: $scriptname positions-cat.fits --center=1.23,4.56 \
 Given a set of dithering positions ('positions-cat.fits' in the example
 above), and an image ('image.fits' which can be from any part of the sky,
 only its distortion and orientation are important), build the exposure map
-of the output stack after applying that dither (where each pixel contains
-the number of exposures that were used in it). The field of the final stack
+of the output coadd after applying that dither (where each pixel contains
+the number of exposures that were used in it). The field of the final coad
 can be set with the '--center' and '--width' options.
 
 $scriptname options:
@@ -81,12 +81,12 @@ $scriptname options:
                           Input: '$WARPED'. Output: '$TARGET'.
 
  Output:
-  -o, --output=STR        Name of finally stacked image.
+  -o, --output=STR        Name of finally coadded image.
   -C, --center=FLT,FLT    Center of output stack (in RA,Dec).
-  -w, --width=FLT,FLT     Width of output stack (in WCS).
+  -w, --width=FLT,FLT     Width of output coadd (in WCS).
       --ctype=STR,STR     Projection of output (CTYPEi in WCS).
       --widthinpix        Interpret '--width' values in pixels.
-      --stack-operator="STR" Arithmetic operator to use for stacking.
+      --coadd-operator="STR" Arithmetic operator to use for coadding.
   -t, --tmpdir=STR        Directory to keep temporary files.
   -k, --keeptmp           Keep temporal/auxiliar files.
 
@@ -109,7 +109,7 @@ EOF
 print_version() {
      cat <<EOF
 $scriptname (GNU Astronomy Utilities) $version
-Copyright (C) 2023-2024 Free Software Foundation, Inc.
+Copyright (C) 2023-2025 Free Software Foundation, Inc.
 License GPLv3+: GNU General public license version 3 or later.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
@@ -128,6 +128,44 @@ Citations and acknowledgement are vital for the continued work on Gnuastro.
 
 Please cite the following record(s) and add the acknowledgement statement below in your work to support us. Please note that different Gnuastro programs may have different corresponding papers. Hence, please check all the programs you used. Don't forget to also include the version as shown above for reproducibility.
 
+Main Gnuastro paper
+-------------------
+The paper below was the first published resource that introduced Gnuastro. Its focus is only three of the Gnuastro programs (NoiseChisel, Segment and MakeCatalog), but we have not yet had time write a dedicated paper for Gnuastro. Until a high-level paper that describes the whole of Gnuastro is published, this is the main paper that Gnuastro's citations will be counted against. Therefore, please cite this, even if you have not used those programs.
+
+  @ARTICLE{gnuastro,
+     author = {{Akhlaghi}, Mohammad and {Ichikawa}, Takashi},
+      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
+    journal = {ApJS},
+  archivePrefix = "arXiv",
+     eprint = {1505.01664},
+   primaryClass = "astro-ph.IM",
+   keywords = {galaxies: irregular, galaxies: photometry,
+               galaxies: structure, methods: data analysis,
+               techniques: image processing, techniques: photometric},
+       year = 2015,
+      month = sep,
+     volume = 220,
+        eid = {1},
+      pages = {1},
+        doi = {10.1088/0067-0049/220/1/1},
+     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
+    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
+  }
+
+
+Gnuastro book
+-------------
+If you want to cite any part of the book (in any of the programs), please use the BibTeX entry below:
+
+  @BOOK{gnuastrobook,
+     author = {{Akhlaghi}, Mohammad},
+      title = {GNU Astronomy Utilities (version $version)},
+       year = 2024,
+  publisher = {Free Software Foundation},
+        doi = {10.5281/zenodo.12738457}
+  }
+
+
 Paper describing this script
 ----------------------------
   @ARTICLE{astscript-pointing-simulate,
@@ -148,24 +186,6 @@ archivePrefix = {arXiv},
         adsnote = {Provided by the SAO/NASA Astrophysics Data System}
   }
 
-Paper introducing Gnuastro (currently main citation)
-----------------------------------------------------
-  @ARTICLE{gnuastro,
-     author = {{Akhlaghi}, M. and {Ichikawa}, T.},
-      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
-    journal = {ApJS},
-  archivePrefix = "arXiv",
-     eprint = {1505.01664},
-   primaryClass = "astro-ph.IM",
-       year = 2015,
-      month = sep,
-     volume = 220,
-        eid = {1},
-      pages = {1},
-        doi = {10.1088/0067-0049/220/1/1},
-     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
-    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
-  }
 
 Acknowledgement
 ---------------
@@ -276,8 +296,8 @@ do
     --ctype=*)       ctype="${1#*=}";                             check_v "$1" "$ctype";  shift;;
     --widthinpix)    widthinpix=1;                                                        shift;;
     --widthinpix=*)  on_off_option_error --quiet -q;;
-    --stack-operator) stack_operator="$2";                        check_v "$1" "$stack_operator";  shift;shift;;
-    --stack-operator=*) stack_operator="${1#*=}";                 check_v "$1" "$stack_operator";  shift;;
+    --coadd-operator) coadd_operator="$2";                        check_v "$1" "$coadd_operator";  shift;shift;;
+    --coadd-operator=*) coadd_operator="${1#*=}";                 check_v "$1" "$coadd_operator";  shift;;
     -k|--keeptmp)           keeptmp=1; shift;;
     -k*|--keeptmp=*)        on_off_option_error --keeptmp -k;;
     -t|--tmpdir)            tmpdir="$2";                                  check_v "$1" "$tmpdir";  shift;shift;;
@@ -327,13 +347,13 @@ if [ x"$img" = x ]; then
     echo "$scriptname: no reference image given to '--img'"; exit 1
 fi
 if [ x"$width" = x ]; then
-    echo "$scriptname: no stack width given to '--width'"; exit 1
+    echo "$scriptname: no coadd width given to '--width'"; exit 1
 fi
 if [ x"$ctype" = x ]; then
     echo "$scriptname: no projection given to '--ctype'"; exit 1
 fi
 if [ x"$center" = x ]; then
-    echo "$scriptname: no stack center given to '--center'"; exit 1
+    echo "$scriptname: no coadd center given to '--center'"; exit 1
 else
     ncenter=$(echo $center | awk 'BEGIN{FS=","}\
                                   {for(i=1;i<=NF;++i) c+=$i!=""}\
@@ -396,7 +416,7 @@ echo "center = $center" >> $config
 echo "output = $output" >> $config
 echo "imghdu = $imghdu" >> $config
 echo "scriptname = $scriptname" >> $config
-echo "stack-operator = $stack_operator" >> $config
+echo "coadd-operator = $coadd_operator" >> $config
 echo "dithers = $(seq $ndither | tr '\n' ' ')" >> $config
 echo "hook-warp-before=$hook_warp_before" >> $config
 echo "hook-warp-after=$hook_warp_after" >> $config
