@@ -16,7 +16,7 @@
 #     Raul Infante-Sainz <infantesainz@gmail.com>
 #     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 #     Carlos Morales-Socorro <cmorsoc@gmail.com>
-# Copyright (C) 2020-2024 Free Software Foundation, Inc.
+# Copyright (C) 2020-2025 Free Software Foundation, Inc.
 #
 # Gnuastro is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -104,7 +104,7 @@ $scriptname options:
   -p, --parallaxanderrorcolumn=STR,STR The name of the parallax column.
   -m, --magnituderange=FLT,FLT The range of magnitude.
   -Q, --minaxisratio=FLT  Minimum axis ratio to be accepted (default to 0.9).
-  -M, --mindistdeg=FLT    Minimum distance to more bright neighbour stars.
+  -M, --mindistdeg=FLT    Minimum distance to more bright neighbor stars.
                           to be accepted, in degrees.
   -c, --catalog=STR       Catalog of stars containing:
                           ra, dec, magnitude, parrallax, parrallax_error.
@@ -143,7 +143,7 @@ EOF
 print_version() {
      cat <<EOF
 $scriptname (GNU Astronomy Utilities) $version
-Copyright (C) 2020-2024 Free Software Foundation, Inc.
+Copyright (C) 2020-2025 Free Software Foundation, Inc.
 License GPLv3+: GNU General public license version 3 or later.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
@@ -167,9 +167,50 @@ Citations and acknowledgement are vital for the continued work on Gnuastro.
 
 Please cite the following record(s) and add the acknowledgement statement below in your work to support us. Please note that different Gnuastro programs may have different corresponding papers. Hence, please check all the programs you used. Don't forget to also include the version as shown above for reproducibility.
 
-Paper describing the creation of an extended PSF
-------------------------------------------------
-@ARTICLE{gnuastro-psf,
+Main Gnuastro paper
+-------------------
+The paper below was the first published resource that introduced Gnuastro. Its focus is only three of the Gnuastro programs (NoiseChisel, Segment and MakeCatalog), but we have not yet had time write a dedicated paper for Gnuastro. Until a high-level paper that describes the whole of Gnuastro is published, this is the main paper that Gnuastro's citations will be counted against. Therefore, please cite this, even if you have not used those programs.
+
+  @ARTICLE{gnuastro,
+     author = {{Akhlaghi}, Mohammad and {Ichikawa}, Takashi},
+      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
+    journal = {ApJS},
+  archivePrefix = "arXiv",
+     eprint = {1505.01664},
+   primaryClass = "astro-ph.IM",
+   keywords = {galaxies: irregular, galaxies: photometry,
+               galaxies: structure, methods: data analysis,
+               techniques: image processing, techniques: photometric},
+       year = 2015,
+      month = sep,
+     volume = 220,
+        eid = {1},
+      pages = {1},
+        doi = {10.1088/0067-0049/220/1/1},
+     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
+    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
+  }
+
+
+Gnuastro book
+-------------
+If you want to cite any part of the book (in any of the programs), please use the BibTeX entry below:
+
+  @BOOK{gnuastrobook,
+     author = {{Akhlaghi}, Mohammad},
+      title = {GNU Astronomy Utilities (version $version)},
+       year = 2024,
+  publisher = {Free Software Foundation},
+        doi = {10.5281/zenodo.12738457}
+  }
+
+
+Papers describing the creation of an extended PSF
+-------------------------------------------------
+
+Gnuastro's PSF scripts have been written based on the experience and methods that were found in the first paper below (when these scripts did not exist yet) and more formally introduced as part of the second paper below. So if you use any of these scripts, please cite the following two papers.
+
+@ARTICLE{gnuastro-psf-1,
        author = {{Infante-Sainz}, Ra{\'u}l and {Trujillo}, Ignacio and {Rom{\'a}n}, Javier},
         title = "{The Sloan Digital Sky Survey extended point spread functions}",
       journal = {MNRAS},
@@ -186,28 +227,23 @@ archivePrefix = {arXiv},
       adsnote = {Provided by the SAO/NASA Astrophysics Data System}
 }
 
-First paper introducing Gnuastro
---------------------------------
-  @ARTICLE{gnuastro,
-     author = {{Akhlaghi}, M. and {Ichikawa}, T.},
-      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
-    journal = {ApJS},
-  archivePrefix = "arXiv",
-     eprint = {1505.01664},
-   primaryClass = "astro-ph.IM",
-       year = 2015,
-      month = sep,
-     volume = 220,
-        eid = {1},
-      pages = {1},
-        doi = {10.1088/0067-0049/220/1/1},
-     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
-    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
-  }
+@ARTICLE{gnuastro-psf-2,
+       author = {{Eskandarlou}, Sepideh and {Akhlaghi}, Mohammad and {Knapen}, Johan H. and {L{\'o}pez-Sanjuan}, Carlos and {Infante-Sainz}, Ra{\'u}l and {Dom{\'\i}nguez S{\'a}nchez}, Helena and {Sharbaf}, Zahra and {V{\'a}zquez Rami{\'o}}, H{\'e}ctor and {Fern{\'a}ndez Ontiveros}, Juan Antonio and {I{\~n}iguez Garc{\'\i}a}, C{\'e}sar and {Civera Lorenzo}, Tamara and {Jos{\'e} Muniesa Gallardo}, David and {Coelho}, Paula R.~T. and {Ederoclite}, Alessandro and {Varela}, Jesus and {Jim{\'e}nez-Esteban}, Fran and {Cenarro}, A. Javier and {Mar{\'\i}n-Franch}, Antonio and {Dupke}, Renato A. and {Moles}, Mariano and {Hern{\'a}ndez-Monteagudo}, Carlos and {Rahna P.}, T. and {Crist{\'o}bal-Hornillos}, David and {Alcaniz}, Jailson and {Sodr{\'e}}, Jr., Laerte and {Angulo}, Raul E.},
+        title = "{J-PLUS: Turning Off the Bright Stars}",
+      journal = {arXiv e-prints},
+     keywords = {Instrumentation and Methods for Astrophysics, Astrophysics of Galaxies},
+         year = 2025,
+        month = oct,
+          eid = {arXiv:2510.12940},
+        pages = {arXiv:2510.12940},
+          doi = {10.1051/0004-6361/202556547},
+archivePrefix = {arXiv},
+       eprint = {2510.12940},
+ primaryClass = {astro-ph.IM},
+       adsurl = {https://ui.adsabs.harvard.edu/abs/2025arXiv251012940E},
+      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
+}
 
-Acknowledgement
----------------
-This work was partly done using GNU Astronomy Utilities (Gnuastro, ascl.net/1801.009) version $version. Work on Gnuastro has been funded by the Japanese Ministry of Education, Culture, Sports, Science, and Technology (MEXT) scholarship and its Grant-in-Aid for Scientific Research (21244012, 24253003), the European Research Council (ERC) advanced grant 339659-MUSICOS, the Spanish Ministry of Economy and Competitiveness (MINECO, grant number AYA2016-76219-P) and the NextGenerationEU grant through the Recovery and Resilience Facility project ICTS-MRR-2021-03-CEFCA.
                                                ,
                                               {|'--.
                                              {{\    \ $empty
@@ -720,7 +756,7 @@ echo "# Column 3: $field [mag,f64] Magnitude"       >> $moreneighbor
 asttable $circular -c$racolumn,$deccolumn,$field \
          | while read r d mag; do
 
-   # Make a file for number of neighbourhood.
+   # Make a file for number of neighborhood.
    numberneighbor=$tmpdir/"$r_$d"th-star-neighbor.fits
 
    # Find number of neighborhod for each star.
