@@ -16,7 +16,7 @@
 #   Mohammad Akhlaghi <mohammad@akhlaghi.org>
 #   Carlos Morales-Socorro <cmorsoc@gmail.com>
 #   Sepideh Eskandarlou <sepideh.eskandarlou@gmail.com>
-# Copyright (C) 2021-2024 Free Software Foundation, Inc.
+# Copyright (C) 2021-2025 Free Software Foundation, Inc.
 #
 # Gnuastro is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -37,7 +37,10 @@ set -e
 
 # 'LC_NUMERIC' is responsible for formatting numbers printed by the OS.  It
 # prevents floating points like '23,45' instead of '23.45'.
-export LC_NUMERIC=C
+sys_lang=$LANG
+export LANG=C
+sys_lcnumeric=$LC_NUMERIC
+export LC_NUMERIC="en_US.UTF-8"
 
 
 
@@ -45,6 +48,7 @@ export LC_NUMERIC=C
 
 # Default option values (can be changed with options on the command-line).
 hdu=1
+mode=""
 quiet=""
 center=""
 keeptmp=0
@@ -141,7 +145,7 @@ EOF
 print_version() {
     cat <<EOF
 $scriptname (GNU Astronomy Utilities) $version
-Copyright (C) 2020-2024 Free Software Foundation, Inc.
+Copyright (C) 2020-2025 Free Software Foundation, Inc.
 License GPLv3+: GNU General public license version 3 or later.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
@@ -165,9 +169,50 @@ Citations and acknowledgement are vital for the continued work on Gnuastro.
 
 Please cite the following record(s) and add the acknowledgement statement below in your work to support us. Please note that different Gnuastro programs may have different corresponding papers. Hence, please check all the programs you used. Don't forget to also include the version as shown above for reproducibility.
 
-Paper describing the creation of an extended PSF
-------------------------------------------------
-@ARTICLE{gnuastro-psf,
+Main Gnuastro paper
+-------------------
+The paper below was the first published resource that introduced Gnuastro. Its focus is only three of the Gnuastro programs (NoiseChisel, Segment and MakeCatalog), but we have not yet had time write a dedicated paper for Gnuastro. Until a high-level paper that describes the whole of Gnuastro is published, this is the main paper that Gnuastro's citations will be counted against. Therefore, please cite this, even if you have not used those programs.
+
+  @ARTICLE{gnuastro,
+     author = {{Akhlaghi}, Mohammad and {Ichikawa}, Takashi},
+      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
+    journal = {ApJS},
+  archivePrefix = "arXiv",
+     eprint = {1505.01664},
+   primaryClass = "astro-ph.IM",
+   keywords = {galaxies: irregular, galaxies: photometry,
+               galaxies: structure, methods: data analysis,
+               techniques: image processing, techniques: photometric},
+       year = 2015,
+      month = sep,
+     volume = 220,
+        eid = {1},
+      pages = {1},
+        doi = {10.1088/0067-0049/220/1/1},
+     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
+    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
+  }
+
+
+Gnuastro book
+-------------
+If you want to cite any part of the book (in any of the programs), please use the BibTeX entry below:
+
+  @BOOK{gnuastrobook,
+     author = {{Akhlaghi}, Mohammad},
+      title = {GNU Astronomy Utilities (version $version)},
+       year = 2024,
+  publisher = {Free Software Foundation},
+        doi = {10.5281/zenodo.12738457}
+  }
+
+
+Papers describing the creation of an extended PSF
+-------------------------------------------------
+
+Gnuastro's PSF scripts have been written based on the experience and methods that were found in the first paper below (when these scripts did not exist yet) and more formally introduced as part of the second paper below. So if you use any of these scripts, please cite the following two papers.
+
+@ARTICLE{gnuastro-psf-1,
        author = {{Infante-Sainz}, Ra{\'u}l and {Trujillo}, Ignacio and {Rom{\'a}n}, Javier},
         title = "{The Sloan Digital Sky Survey extended point spread functions}",
       journal = {MNRAS},
@@ -184,28 +229,23 @@ archivePrefix = {arXiv},
       adsnote = {Provided by the SAO/NASA Astrophysics Data System}
 }
 
-First paper introducing Gnuastro
---------------------------------
-  @ARTICLE{gnuastro,
-     author = {{Akhlaghi}, M. and {Ichikawa}, T.},
-      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
-    journal = {ApJS},
-  archivePrefix = "arXiv",
-     eprint = {1505.01664},
-   primaryClass = "astro-ph.IM",
-       year = 2015,
-      month = sep,
-     volume = 220,
-        eid = {1},
-      pages = {1},
-        doi = {10.1088/0067-0049/220/1/1},
-     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
-    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
-  }
+@ARTICLE{gnuastro-psf-2,
+       author = {{Eskandarlou}, Sepideh and {Akhlaghi}, Mohammad and {Knapen}, Johan H. and {L{\'o}pez-Sanjuan}, Carlos and {Infante-Sainz}, Ra{\'u}l and {Dom{\'\i}nguez S{\'a}nchez}, Helena and {Sharbaf}, Zahra and {V{\'a}zquez Rami{\'o}}, H{\'e}ctor and {Fern{\'a}ndez Ontiveros}, Juan Antonio and {I{\~n}iguez Garc{\'\i}a}, C{\'e}sar and {Civera Lorenzo}, Tamara and {Jos{\'e} Muniesa Gallardo}, David and {Coelho}, Paula R.~T. and {Ederoclite}, Alessandro and {Varela}, Jesus and {Jim{\'e}nez-Esteban}, Fran and {Cenarro}, A. Javier and {Mar{\'\i}n-Franch}, Antonio and {Dupke}, Renato A. and {Moles}, Mariano and {Hern{\'a}ndez-Monteagudo}, Carlos and {Rahna P.}, T. and {Crist{\'o}bal-Hornillos}, David and {Alcaniz}, Jailson and {Sodr{\'e}}, Jr., Laerte and {Angulo}, Raul E.},
+        title = "{J-PLUS: Turning Off the Bright Stars}",
+      journal = {arXiv e-prints},
+     keywords = {Instrumentation and Methods for Astrophysics, Astrophysics of Galaxies},
+         year = 2025,
+        month = oct,
+          eid = {arXiv:2510.12940},
+        pages = {arXiv:2510.12940},
+          doi = {10.1051/0004-6361/202556547},
+archivePrefix = {arXiv},
+       eprint = {2510.12940},
+ primaryClass = {astro-ph.IM},
+       adsurl = {https://ui.adsabs.harvard.edu/abs/2025arXiv251012940E},
+      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
+}
 
-Acknowledgement
----------------
-This work was partly done using GNU Astronomy Utilities (Gnuastro, ascl.net/1801.009) version $version. Work on Gnuastro has been funded by the Japanese Ministry of Education, Culture, Sports, Science, and Technology (MEXT) scholarship and its Grant-in-Aid for Scientific Research (21244012, 24253003), the European Research Council (ERC) advanced grant 339659-MUSICOS, the Spanish Ministry of Economy and Competitiveness (MINECO, grant number AYA2016-76219-P) and the NextGenerationEU grant through the Recovery and Resilience Facility project ICTS-MRR-2021-03-CEFCA.
                                                ,
                                               {|'--.
                                              {{\    \ $empty
@@ -481,7 +521,13 @@ fi
 # Make sure the value to '--mode' is either 'wcs' or 'img'. Note: '-o'
 # means "or" and is preferred to '[ ] || [ ]' because only a single
 # invocation of 'test' is done. Run 'man test' for more.
-if [ "$mode" = wcs     -o      $mode = "img" ]; then
+if [ x"$mode"  = x ]; then
+    cat <<EOF
+$scriptname: '--mode' (or '-O') is mandatory. It takes one of the following two values: 'img' (for pixel coordinates) or 'wcs' (for celestial coordinates)
+EOF
+    exit 1
+fi
+if [ "$mode" = wcs -o "$mode" = img ]; then
     junk=1
 else
     cat <<EOF
@@ -555,18 +601,18 @@ fi
 # Crop the object around its center with the given stamp size width. It may
 # happen that the given coordinate is fully outside of the image (within
 # the requested stamp-width). In this case Crop won't generate any output,
-# so we are checking the existance of the '$cropped' file. If not created
-# we will create a fully NaN-valued image that can be ignored in stacks.
+# in this case we will create a fully NaN-valued image that can be ignored
+# in stacks within large pipelines that should not stop in this situations.
 #
 # We are setting '--checkcenter=0' in Crop so it doesn't check if the
 # central pixel is covered or not (we may be interested in the outer parts
 # of the PSF of a star that is centered outside of the image).
 cropped=$tmpdir/cropped.fits
-astcrop $inputs --hdu=$hdu --mode=img --checkcenter=0 \
-        --width=$xwidthinpix,$ywidthinpix \
-        --center=$xcenter,$ycenter \
-        --output=$cropped $quiet
-if ! [ -f $cropped ]; then
+if ! astcrop $inputs --hdu=$hdu \
+             --mode=img \
+             --center=$xcenter,$ycenter \
+             --width=$xwidthinpix,$ywidthinpix \
+             --checkcenter=0 --output=$cropped $quiet; then
 
     # 'makenew' will generate a fully 0-valued image. Adding (with '+') any
     # number by NaN will be NaN. Therefore the output of the Arithmetic
@@ -580,7 +626,13 @@ if ! [ -f $cropped ]; then
     # Let the user know what happened.
     if [ x"$quiet" = x ]; then all_nan_warning; fi
 
-    # Return successfully and don't continue
+    # If the user does not specify to keep the temporal files with the option
+    # `--keeptmp', then remove the whole directory.
+    if [ $keeptmp = 0 ]; then
+        rm -r $tmpdir
+    fi
+
+    # Return successfully and continue
     exit 0
 fi
 
@@ -917,6 +969,7 @@ if [ x"$quiet" = x ] && [ $normvalue = nan ]; then all_nan_warning; fi
 
 
 
+
 # Remove temporary files
 # ----------------------
 #
@@ -925,3 +978,12 @@ if [ x"$quiet" = x ] && [ $normvalue = nan ]; then all_nan_warning; fi
 if [ $keeptmp = 0 ]; then
     rm -r $tmpdir
 fi
+
+
+
+
+
+# The script has finished, reset the original language to the system's
+# default language.
+export LANG=$sys_lang
+export LC_NUMERIC=$sys_lcnumeric
