@@ -10,7 +10,7 @@
 #   Zahra Sharbaf <zahra.sharbaf2@gmail.com>
 #   Carlos Morales-Socorro <cmorsoc@gmail.com>
 #   Sepideh Eskandarlou <sepideh.eskandarlou@gmail.com>
-# Copyright (C) 2020-2024 Free Software Foundation, Inc.
+# Copyright (C) 2020-2025 Free Software Foundation, Inc.
 #
 # Gnuastro is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -57,10 +57,10 @@ zeropoint=""
 sigmaclip=""
 measuretmp=""
 oversample=""
-undersample=""
 positionangle=0
 zeroisnotblank=""
 version=@VERSION@
+undersample=1.000000
 scriptname=@SCRIPT_NAME@
 
 
@@ -112,12 +112,15 @@ $scriptname options:
   -c, --center=FLT,FLT     Coordinate of the center along 2 axes.
   -R, --rmax=FLT           Maximum radius for the radial profile (in pixels).
   -Q, --axis-ratio=FLT     Axis ratio for ellipse profiles (A/B).
-  -a, --azimuth=FLT,FLT    Azimuthal angles range (from the major axis).
+  -a, --azimuth=FLT,FLT    Azimuthal range(s) (from the major axis).
   -p, --position-angle=FLT Position angle for ellipse profiles.
   -g, --polar              Polar plot (azimuth angle vs. radius).
   -s, --sigmaclip=FLT,FLT  Sigma-clip multiple and tolerance.
   -z, --zeropoint=FLT      Zeropoint magnitude of input dataset.
   -Z, --zeroisnotblank     0.0 in float or double images are not blank.
+  -i, --instd=FLT/STR      Sky standard deviation per pixel, as a single
+                           number or as the filename (given to MakeCatalog).
+  -d, --stdhdu=STR         HDU/extension of the sky standard deviation image.
 
  Output:
   -o, --output             Output table with the radial profile.
@@ -127,9 +130,6 @@ $scriptname options:
   -P, --precision=INT      Number of digits after decimal point for radius.
   -v, --oversample=INT     Oversample for higher resolution radial profile.
   -u, --undersample=INT    Undersample for lower resolution radial profile.
-  -i, --instd=FLT/STR      Sky standard deviation per pixel, as a single
-                           number or as the filename.
-  -d, --stdhdu=STR         HDU/extension of the sky standard deviation image.
 
  Operating mode:
   -?, --help               Print this help list.
@@ -154,7 +154,7 @@ EOF
 print_version() {
     cat <<EOF
 $scriptname (GNU Astronomy Utilities) $version
-Copyright (C) 2020-2024 Free Software Foundation, Inc.
+Copyright (C) 2020-2025 Free Software Foundation, Inc.
 License GPLv3+: GNU General public license version 3 or later.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
@@ -164,6 +164,9 @@ EOF
 }
 
 
+
+
+
 print_citation() {
     empty="" # needed for the ascii art!
     cat <<EOF
@@ -174,8 +177,46 @@ Citations and acknowledgement are vital for the continued work on Gnuastro.
 
 Please cite the following record(s) and add the acknowledgement statement below in your work to support us. Please note that different Gnuastro programs may have different corresponding papers. Hence, please check all the programs you used. Don't forget to also include the version as shown above for reproducibility.
 
-Paper introducing this script
------------------------------
+Main Gnuastro paper
+-------------------
+The paper below was the first published resource that introduced Gnuastro. Its focus is only three of the Gnuastro programs (NoiseChisel, Segment and MakeCatalog), but we have not yet had time write a dedicated paper for Gnuastro. Until a high-level paper that describes the whole of Gnuastro is published, this is the main paper that Gnuastro's citations will be counted against. Therefore, please cite this, even if you have not used those programs.
+
+  @ARTICLE{gnuastro,
+     author = {{Akhlaghi}, Mohammad and {Ichikawa}, Takashi},
+      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
+    journal = {ApJS},
+  archivePrefix = "arXiv",
+     eprint = {1505.01664},
+   primaryClass = "astro-ph.IM",
+   keywords = {galaxies: irregular, galaxies: photometry,
+               galaxies: structure, methods: data analysis,
+               techniques: image processing, techniques: photometric},
+       year = 2015,
+      month = sep,
+     volume = 220,
+        eid = {1},
+      pages = {1},
+        doi = {10.1088/0067-0049/220/1/1},
+     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
+    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
+  }
+
+
+Gnuastro book
+-------------
+If you want to cite any part of the book (in any of the programs), please use the BibTeX entry below:
+
+  @BOOK{gnuastrobook,
+     author = {{Akhlaghi}, Mohammad},
+      title = {GNU Astronomy Utilities (version $version)},
+       year = 2024,
+  publisher = {Free Software Foundation},
+        doi = {10.5281/zenodo.12738457}
+  }
+
+
+Paper introducing this script in general
+----------------------------------------
   @ARTICLE{astscript-radial-profile,
          author = {{Infante-Sainz}, Ra{\'u}l and {Akhlaghi}, Mohammad and {Eskandarlou}, Sepideh},
           title = "{Gnuastro: Measuring Radial Profiles from Images}",
@@ -195,9 +236,10 @@ Paper introducing this script
         adsnote = {Provided by the SAO/NASA Astrophysics Data System}
   }
 
+
 Paper introducing the --polar option
 ------------------------------------
-  @ARTICLE{2024RNAAS...8..168E,
+  @ARTICLE{gnuastro-polarplot,
          author = {{Eskandarlou}, Sepideh and {Akhlaghi}, Mohammad},
           title = "{Gnuastro: Generating Polar Plots in Astronomical Images}",
         journal = {Research Notes of the American Astronomical Society},
@@ -216,26 +258,6 @@ Paper introducing the --polar option
         adsnote = {Provided by the SAO/NASA Astrophysics Data System}
   }
 
-Paper introducing Gnuastro (currently main citation)
-----------------------------------------------------
-  @ARTICLE{gnuastro,
-     author = {{Akhlaghi}, M. and {Ichikawa}, T.},
-      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
-    journal = {ApJS},
-  archivePrefix = "arXiv",
-     eprint = {1505.01664},
-   primaryClass = "astro-ph.IM",
-       year = 2015,
-      month = sep,
-     volume = 220,
-        eid = {1},
-      pages = {1},
-        doi = {10.1088/0067-0049/220/1/1},
-     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
-    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
-  }
-
-
 
 Acknowledgement
 ---------------
@@ -326,9 +348,9 @@ do
         -Q|--axis-ratio)         axisratio="$2";                               check_v "$1" "$axisratio";  shift;shift;;
         -Q=*|--axis-ratio=*)     axisratio="${1#*=}";                          check_v "$1" "$axisratio";  shift;;
         -Q*)                     axisratio=$(echo "$1"  | sed -e's/-Q//');     check_v "$1" "$axisratio";  shift;;
-        -a|--azimuth)            azimuth="$2";                                 check_v "$1" "$azimuth";  shift;shift;;
-        -a=*|--azimuth=*)        azimuth="${1#*=}";                            check_v "$1" "$azimuth";  shift;;
-        -a*)                     azimuth=$(echo "$1"  | sed -e's/-a//');       check_v "$1" "$azimuth";  shift;;
+        -a|--azimuth)            aztmp="$2";                                   check_v "$1" "$aztmp";  shift;shift;;
+        -a=*|--azimuth=*)        aztmp="${1#*=}";                              check_v "$1" "$aztmp";  shift;;
+        -a*)                     aztmp=$(echo "$1"  | sed -e's/-a//');         check_v "$1" "$aztmp";  shift;;
         -p|--position-angle)     positionangle="$2";                           check_v "$1" "$positionangle";  shift;shift;;
         -p=*|--position-angle=*) positionangle="${1#*=}";                      check_v "$1" "$positionangle";  shift;;
         -p*)                     positionangle=$(echo "$1"  | sed -e's/-p//'); check_v "$1" "$positionangle";  shift;;
@@ -389,14 +411,33 @@ do
         *) if [ x"$inputs" = x ]; then inputs="$1"; else inputs="$inputs $1"; fi; shift;;
     esac
 
-    # If a measurment was given, add it to possibly existing previous
-    # measurements into a comma-separate list.
+    # If a measurment or azimuth were given (options that can be called
+    # more than once), add their value to previous values.
     if [ x"$measuretmp" != x ]; then
         if [ x"$measure" = x ]; then measure=$measuretmp;
         else                         measure="$measure,$measuretmp";
         fi
         measuretmp=""
     fi
+    if [ x"$aztmp" != x ]; then
+
+        # Make sure two values are indeed given to this option.
+        naz=$(echo $aztmp \
+                  | awk 'BEGIN{FS=","} \
+                         {for(i=1;i<=NF;++i) c+=$i!=""} \
+                         END{print c}')
+        if [ x$naz != x2 ]; then
+            printf "$scriptname: '--azimuth' (or '-a') only takes "
+            printf "two numbers, but $naz number(s) were given "
+            printf "in '$aztmp'\n"; exit 1
+        fi
+
+        # Add the give values to any potentially pre-existing values.
+        if [ x"$azimuth" = x ]; then azimuth=$aztmp;
+        else                         azimuth="$azimuth:$aztmp";
+        fi
+        aztmp=""
+    fi
 done
 
 
@@ -437,17 +478,6 @@ else
     exit 1
 fi
 
-# If a '--azimuth' has been given, make sure it only has two numbers.
-if [ x"$azimuth" != x ]; then
-    nazimuth=$(echo $azimuth | awk 'BEGIN{FS=","} \
-                                    {for(i=1;i<=NF;++i) c+=$i!=""} \
-                                    END{print c}')
-    if [ x$nazimuth != x2 ]; then
-        echo "$scriptname: '--azimuth' (or '-a') only takes two values, but $nazimuth were given"
-        exit 1
-    fi
-fi
-
 # The precision should be an integer, smaller than 6.
 pcheck=$(echo $precision \
              | awk 'int($1)==$1 && $1>=0 && $1<=6 {print 1}')
@@ -612,16 +642,31 @@ fi
 
 # Correct the central position (to cropped image)
 # -----------------------------------------------
-dxy=$(astfits $crop -h0 \
-          | grep ICF1PIX \
-          | sed -e"s/'/ /g" -e's/\:/ /g' -e's/,/ /' \
-          | awk '{print $3-1, $5-1}')
-xcenter=$(echo "$xcenter $cropwidth $dxy" \
-              | awk '{ if($1>int($2/2)) print $1-$3; \
-                       else             print int($2/2)+$1-int($1) }')
-ycenter=$(echo "$ycenter $cropwidth $dxy" \
-              | awk '{ if($1>int($2/2)) print $1-$4; \
-                       else             print int($2/2)+$1-int($1) }')
+#
+# After 'astcrop' above, the image is going to have an odd width and the
+# central pixel of the crop contains our desired central coordinate (which
+# can be a floating point). However, the FITS standard sets the center of a
+# pixel to be an integer, not its edge (as we would inherently assume!). So
+# for example the bottom-left corner of the bottom-left pixel in the image
+# has a coordinate of (0.5,0.5).
+#
+# When correcting the original center to the center of the crop this needs
+# to be taken into account: below, we show the original center with 'c',
+# and its non-integer part with 'f'. We will also show the cropped image's
+# central pixel (an integer) with 'cci'.  When 'f<=0.5', we can simply add
+# 'f' to 'cci'. But when 'f>0.5', we need to subtract one from 'cci', then
+# add the fractional value. In both cases the coordinate will fall in the
+# same pixel, this is just because of the FITS standard's definition!
+correct_center() {
+    astfits $crop -h1 --keyvalue=NAXIS$1 --quiet \
+              | awk '{c='$2'; ic=int(c); \
+                      f=c-ic; cci=int($1/2)+1; \
+                      if(f>0.5) cc=(cci-1)+f; \
+                      else      cc=cci+f; \
+                      print cc}'
+}
+xcenter=$(correct_center 1 $xcenter)
+ycenter=$(correct_center 2 $ycenter)
 
 
 
@@ -733,7 +778,7 @@ rm $radialaperturesholed
 # Create an azimuthal apertures image if the user specifies a range of
 # angles over which compute the radial profile. The azimuthal radial
 # profile is combined with the radial apertures generated above, to only
-# consider the desired portion of the image. If the azimuth option is not
+# consider the desired portions of the image. If the azimuth option is not
 # called, then the aperture image is a symbolic link to the radial aperture
 # image constructed above.
 aperturesrawbase=apertures-raw.fits
@@ -749,58 +794,78 @@ if [ x"$azimuth" != x ]; then
         echo $new_angle
     }
 
-    # Get the initial and final azimuth angles
-    raw_azimuth_ini=$(echo "$azimuth" | awk 'BEGIN{FS=","} {print $1}')
-    raw_azimuth_fin=$(echo "$azimuth" | awk 'BEGIN{FS=","} {print $2}')
-
-    # Transform the azimuth angles to [0, 360) range
-    azimuth_ini=$(transform_angle $raw_azimuth_ini)
-    azimuth_fin=$(transform_angle $raw_azimuth_fin)
-
-    # Generate the azimuthal apertures
-    azimuthaperturesbase=azimuth-raw.fits
-    azimuthapertures=$tmpdir/$azimuthaperturesbase
+    # Generate the azimuthal apertures over the full profile.
+    azraw=$tmpdir/azimuth-raw.fits
+    azone=$tmpdir/azimuth-one.fits
     echo "1 $xcenter $ycenter 9 $rmax 1 $positionangle $axisratio 1 1" \
-         | astmkprof --background=$values --backhdu=1 --mforflatpix \
-                     --mode=img --clearcanvas --type=float32 $quiet \
-                     --circumwidth=1 --replace --output=$azimuthapertures
+        | astmkprof --background=$values --backhdu=1 --mforflatpix \
+                    --mode=img --clearcanvas --type=float32 $quiet \
+                    --circumwidth=1 --replace --output=$azraw
 
-    # From the azimuthal aperture image, consider only that portion that
-    # are in between (or outside) the two specified angles. Set the rest of
-    # the pixels to zero values (on the radial apertures image).
-    #
-    # There are two ways that the angle range can be defined:
-    #
-    #   - The first is smaller than the second (for example
-    #     '--azimuth=10,20'). In this case, we will use an 'and' between
-    #     the regions of each angle so the user gets the region between the
-    #     two angles.
-    #
-    #   - The first is greater than the second (for example
-    #     '--azimuth=355,5') In this case, we assume that the user wants an
-    #     azimuthal range outside the two angles. In the example above, its
-    #     the 10 degrees around the azimuthal angle 0. For this case, we
-    #     should use 'or' between the two regions.
-    #
-    #   - The central pixel contains all azimuth angles, it should
-    #     therefore be included in azimthal ranges. By default MakeProfiles
-    #     is forced to give it a single azimuth angle value, so without
-    #     explicitly adding the central pixel to all azimuthal profiles, it
-    #     the radius of 1 will be ignored in some azimuthal ranges and kept
-    #     in others. But this is only because it is smaller than our
-    #     sampling rate, in practice, the central pixel contains all
-    #     azimuth angles. So when defining the 'arc', we are also adding
-    #     'radial 1 eq or' to add that central pixel.
-    condition=$(echo $azimuth_ini $azimuth_fin \
-                     | awk ' {if ($1<$2) print "and"; \
-                              else       print "or"}')
-    astarithmetic --output=$aperturesraw $quiet \
-                  $radialapertures  -h1 uint32 set-radial \
-                  $azimuthapertures -h1 uint16 set-azimuth \
-                  azimuth $azimuth_ini uint16 ge \
-                  azimuth $azimuth_fin uint16 le $condition \
-                  radial 1 eq or set-arc \
-                  radial arc 1 uint8 ne 0 uint8 where
+    # Loop over all the pairs. But first, make sure that the final file
+    # (with all azimuthal ranges) is not present (from a previous run for
+    # example).
+    rm -f $aperturesraw
+    for p in $(echo "$azimuth" | tr ':' ' '); do
+
+        # Get the initial and final azimuth angles
+        raw_azimuth_ini=$(echo "$p" | awk 'BEGIN{FS=","} {print $1}')
+        raw_azimuth_fin=$(echo "$p" | awk 'BEGIN{FS=","} {print $2}')
+
+        # Transform the azimuth angles to [0, 360) range
+        azimuth_ini=$(transform_angle $raw_azimuth_ini)
+        azimuth_fin=$(transform_angle $raw_azimuth_fin)
+
+        # From the azimuthal aperture image, consider only that portion
+        # that are in between (or outside) the two specified angles. Set
+        # the rest of the pixels to zero values (on the radial apertures
+        # image).
+        #
+        # There are two ways that the angle range can be defined:
+        #
+        #   - The first is smaller than the second (for example
+        #     '--azimuth=10,20'). In this case, we will use an 'and'
+        #     between the regions of each angle so the user gets the region
+        #     between the two angles.
+        #
+        #   - The first is greater than the second (for example
+        #     '--azimuth=355,5') In this case, we assume that the user
+        #     wants an azimuthal range outside the two angles. In the
+        #     example above, its the 10 degrees around the azimuthal angle
+        #     0. For this case, we should use 'or' between the two regions.
+        #
+        #   - The central pixel contains all azimuth angles, it should
+        #     therefore be included in azimthal ranges. By default
+        #     MakeProfiles is forced to give it a single azimuth angle
+        #     value, so without explicitly adding the central pixel to all
+        #     azimuthal profiles, it the radius of 1 will be ignored in
+        #     some azimuthal ranges and kept in others. But this is only
+        #     because it is smaller than our sampling rate, in practice,
+        #     the central pixel contains all azimuth angles. So when
+        #     defining the 'arc', we are also adding 'radial 1 eq or' to
+        #     add that central pixel.
+        condition=$(echo $azimuth_ini $azimuth_fin \
+                         | awk ' {if ($1<$2) print "and"; \
+                                  else       print "or"}')
+        astarithmetic --output=$azone $quiet \
+                      $radialapertures  -h1 uint16 set-radial \
+                      $azraw            -h1 uint16 set-azimuth \
+                      azimuth $azimuth_ini  uint16 ge \
+                      azimuth $azimuth_fin  uint16 le $condition \
+                      radial 1 eq or set-arc \
+                      radial arc 1 uint8 ne 0 uint8 where
+
+        # Put the output of this azimuthal range into the intermediate
+        # file.
+        if [ -f $aperturesraw ]; then
+            aperturesrawtmp=$tmpdir/azimuth-intermediate-tmp.fits
+            astarithmetic $aperturesraw set-i $azone set-o \
+                          i i 0 eq o where -g1 -o$aperturesrawtmp
+            mv $aperturesrawtmp $aperturesraw
+        else
+            mv $azone $aperturesraw
+        fi
+    done
 else
     cd $tmpdir; ln -fs $radialaperturesbase $aperturesrawbase; cd $curdir
 fi
@@ -822,19 +887,10 @@ else                           finalzp="--zeropoint=$zeropoint";
 fi
 if [ x$polar = x0 ]; then polaropt=""
 else
-    # We are using 'raw_azimuth_fin' because it can be set to zero when the
-    # value was 360. Note also that we should convert to an integer at the
-    # start (before 'set-i'), otherwise, in floating point, the 'gt'
-    # condition is going to miss the outer edge.
-    condition=$(echo $azimuth_ini $azimuth_fin \
-                     | awk ' {if ($1<$2) print "or"; \
-                              else       print "and"}')
-    intazimuthapertures=$tmpdir/azimuth-raw-int.fits
-    astarithmetic $azimuthapertures uint16 set-i i \
-                  i $azimuth_ini     uint16 lt \
-                  i $raw_azimuth_fin uint16 gt $condition \
-                  0 where uint32 --output=$intazimuthapertures $quiet
-    polaropt="--clumpsfile=$intazimuthapertures --clumpshdu=1 --clumpscat"
+    azclumps=$tmpdir/azimuth-clumps.fits
+    astarithmetic $azraw $aperturesraw 0 eq 0 where -g1 \
+                  uint16 --output=$azclumps
+    polaropt="--clumpsfile=$azclumps --clumpshdu=1 --clumpscat"
 fi
 
 
@@ -848,21 +904,28 @@ fi
 # number of pixels on each aperture (at each radius) will be larger. Most
 # of times this option is good to average over a larger number of pixels
 # and increase the signal-to-noise ratio of the measurement.
+#
+# Undersampling by 1 means doing nothing. However, it may happen that the
+# user provide this value as a result of automatic pipelines where this
+# value is considered. Here, a check to ensure that undersample is
+# different from 1.000000 is performed. This ensures the non-trivial
+# undersampling.
 aperturesbase=apertures.fits
 apertures=$tmpdir/$aperturesbase
-if [ x"$undersample" != x ]; then
+undersample_precision=$(printf "%.6f\n" $undersample)
+if [ x"$undersample_precision" != x"1.000000" ]; then
 
     # Divide by the undersampling factor (multiplied by the precision
     # factor: multiple of 10).
-    astarithmetic $aperturesraw $undersample $precfactor int16 x / \
-                  $quiet --output $apertures.fits
+    astarithmetic $aperturesraw $undersample $precfactor x / \
+                  $quiet int16 --output=$apertures.fits
 
     # The integer division above will re-create the 0-valued hole in the
     # center! So we need to fill it like above.
     astarithmetic $apertures.fits set-i \
                   i 0 ne 1 fill-holes set-good \
                   i good i 1 + where \
-                  $quiet --output $apertures
+                  $quiet --output=$apertures
 else
     cd $tmpdir; ln -fs $aperturesrawbase $aperturesbase; cd $curdir
 fi
@@ -917,7 +980,6 @@ astmkcatalog $apertures --hdu=1 --valuesfile=$values --valueshdu=1 \
 
 
 
-
 # Final radii of each row
 # -----------------------
 #
@@ -1018,6 +1080,17 @@ if [ x$polar = x1 ]; then
     maxazim=$(aststatistics $polarcat -h1 -cAZIMUTH --maximum)
     minazim=$(aststatistics $polarcat -h1 -cAZIMUTH --minimum)
 
+    # If the maximum value is 360, make sure we have more than one row with
+    # an azimuth of 360. If there is only one element at an azimuth of 360,
+    # it was just the central pixel and the largest azimuthal value was not
+    # 360.
+    if [ $maxazim = 360 ]; then
+        if [ $(asttable $polarcat --equal=AZIMUTH,360 | wc -l) = 1 ]; then
+            maxazim=$(asttable $polarcat -cAZIMUTH --range=AZIMUTH,0,360 \
+                               -O | aststatistics --maximum)
+        fi
+    fi
+
     # In the end, many azimuth angles close to radius zero are not going to
     # be filled at all! We need to set those to NaN within the radial
     # profile script (because there was no data there and we want to
@@ -1081,7 +1154,7 @@ if [ x$polar = x1 ]; then
 
     # If the output is a FITS file, then add a new HDU. Otherwise (for
     # example output is plain-text), we need to make a new file.
-    if astfits $outputraw &> /dev/null; then
+    if astfits $outputraw > /dev/null 2>&1; then
         exthdu=2
         polarfile=$outputraw
     else
