@@ -16,7 +16,7 @@
 # Contributing authors:
 #     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 #     Raul Infante-Sainz <infantesainz@gmail.com>
-# Copyright (C) 2022-2024 Free Software Foundation, Inc.
+# Copyright (C) 2022-2025 Free Software Foundation, Inc.
 #
 # Gnuastro is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -63,7 +63,7 @@ refcatra="RA"
 refimgshdu=""
 starcatra="RA"
 refcatdec="DEC"
-matchradius=0.2
+matchradius=0.5
 starcatdec="DEC"
 magnituderange=""
 version=@VERSION@
@@ -118,7 +118,7 @@ $scriptname options:
   -r, --refcatra=STR           Reference catalog Right Ascension (R.A.) column.
   -d, --refcatdec=STR          Reference catalog Declination (Dec) column.
   -m, --refcatmag=STR          Reference catalog Magnitude column.
-  -s, --matchradius=FLT        Radius (arcsec) to match stars and ref. catalog.
+  -s, --matchradius=FLT        Radius (in arcsec) to match with reference.
 
  Reference image mode
   -R, --refimgs=STR[,STR]      Reference image name(s) with known zeropoint.
@@ -149,7 +149,7 @@ EOF
 print_version() {
      cat <<EOF
 $scriptname (GNU Astronomy Utilities) $version
-Copyright (C) 2022-2024 Free Software Foundation, Inc.
+Copyright (C) 2022-2025 Free Software Foundation, Inc.
 License GPLv3+: GNU General public license version 3 or later.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
@@ -158,6 +158,10 @@ Written/developed by Sepideh Eskandarlou et al.
 EOF
 }
 
+
+
+
+
 # Output of `--cite':
 print_citation() {
     empty="" # needed for the ascii art!
@@ -169,6 +173,44 @@ Citations and acknowledgement are vital for the continued work on Gnuastro.
 
 Please cite the following record(s) and add the acknowledgement statement below in your work to support us. Please note that different Gnuastro programs may have different corresponding papers. Hence, please check all the programs you used. Don't forget to also include the version as shown above for reproducibility.
 
+Main Gnuastro paper
+-------------------
+The paper below was the first published resource that introduced Gnuastro. Its focus is only three of the Gnuastro programs (NoiseChisel, Segment and MakeCatalog), but we have not yet had time write a dedicated paper for Gnuastro. Until a high-level paper that describes the whole of Gnuastro is published, this is the main paper that Gnuastro's citations will be counted against. Therefore, please cite this, even if you have not used those programs.
+
+  @ARTICLE{gnuastro,
+     author = {{Akhlaghi}, Mohammad and {Ichikawa}, Takashi},
+      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
+    journal = {ApJS},
+  archivePrefix = "arXiv",
+     eprint = {1505.01664},
+   primaryClass = "astro-ph.IM",
+   keywords = {galaxies: irregular, galaxies: photometry,
+               galaxies: structure, methods: data analysis,
+               techniques: image processing, techniques: photometric},
+       year = 2015,
+      month = sep,
+     volume = 220,
+        eid = {1},
+      pages = {1},
+        doi = {10.1088/0067-0049/220/1/1},
+     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
+    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
+  }
+
+
+Gnuastro book
+-------------
+If you want to cite any part of the book (in any of the programs), please use the BibTeX entry below:
+
+  @BOOK{gnuastrobook,
+     author = {{Akhlaghi}, Mohammad},
+      title = {GNU Astronomy Utilities (version $version)},
+       year = 2024,
+  publisher = {Free Software Foundation},
+        doi = {10.5281/zenodo.12738457}
+  }
+
+
 Paper introducing this script
 -----------------------------
   @ARTICLE{astscrip-zeropoint,
@@ -190,28 +232,9 @@ Paper introducing this script
         adsnote = {Provided by the SAO/NASA Astrophysics Data System}
   }
 
-Paper introducing Gnuastro (currently main citation)
-----------------------------------------------------
-  @ARTICLE{gnuastro,
-     author = {{Akhlaghi}, M. and {Ichikawa}, T.},
-      title = "{Noise-based Detection and Segmentation of Nebulous Objects}",
-    journal = {ApJS},
-  archivePrefix = "arXiv",
-     eprint = {1505.01664},
-   primaryClass = "astro-ph.IM",
-       year = 2015,
-      month = sep,
-     volume = 220,
-        eid = {1},
-      pages = {1},
-        doi = {10.1088/0067-0049/220/1/1},
-     adsurl = {https://ui.adsabs.harvard.edu/abs/2015ApJS..220....1A},
-    adsnote = {Provided by the SAO/NASA Astrophysics Data System}
-  }
 
 Acknowledgement
 ---------------
-
 This work was partly done using GNU Astronomy Utilities (Gnuastro, ascl.net/1801.009) version $version. Work on Gnuastro has been funded by the Japanese Ministry of Education, Culture, Sports, Science, and Technology (MEXT) scholarship and its Grant-in-Aid for Scientific Research (21244012, 24253003), the European Research Council (ERC) advanced grant 339659-MUSICOS, the Spanish Ministry of Economy and Competitiveness (MINECO, grant number AYA2016-76219-P) and the NextGenerationEU grant through the Recovery and Resilience Facility project ICTS-MRR-2021-03-CEFCA.
                                                ,
                                               {|'--.
@@ -292,6 +315,10 @@ do
         -s|--starcat)            starcat="$2";                                 check_v "$1" "$starcat";  shift;shift;;
         -s=*|--starcat=*)        starcat="${1#*=}";                            check_v "$1" "$starcat";  shift;;
         -s*)                     starcat=$(echo "$1" | sed -e's/-c//');        check_v "$1" "$starcat";  shift;;
+        --starcatra)             starcatra="$2";                               check_v "$1" "$starcatra";  shift;shift;;
+        --starcatra=*)           starcatra="${1#*=}";                          check_v "$1" "$starcatra";  shift;;
+        --starcatdec)            starcatdec="$2";                              check_v "$1" "$starcatdec"; shift;shift;;
+        --starcatdec=*)          starcatdec="${1#*=}";                         check_v "$1" "$starcatdec"; shift;;
         --starcathdu)            starcathdu="$2";                              check_v "$1" "$starcathdu";  shift;shift;;
         --starcathdu=*)          starcathdu="${1#*=}";                         check_v "$1" "$starcathdu";  shift;;
         -c|--refcat)             refcat="$2";                                  check_v "$1" "$refcat";  shift;shift;;
@@ -566,6 +593,7 @@ echo "output = $output" >> $config
 echo "tmpdir = $tmpdir" >> $config
 echo "starcat = $starcat" >> $config
 echo "starcatra = $starcatra" >> $config
+echo "scriptname = $scriptname" >> $config
 echo "starcatdec = $starcatdec" >> $config
 echo "starcathdu = $starcathdu" >> $config
 echo "matchradius = $matchradius" >> $config
@@ -655,16 +683,16 @@ fi
 # Call the Makefile
 # -----------------
 #
-# Here, Makefile is invoked.
+# Here, Make is invoked for the actual operation.
 if [ x$mksrc = x ]; then
     mksrc=$installdir/zeropoint.mk
 fi
+rm -f $output
 make -f $mksrc tmpdir=$tmpdir --jobs=$numthreads
 
 
 
 
-
 # Remove temporary directory
 # --------------------------
 #
