@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -44,19 +44,6 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
-    {
-      "refcol",
-      UI_KEY_REFCOL,
-      "STR",
-      0,
-      "Reference column name or number.",
-      GAL_OPTIONS_GROUP_INPUT,
-      &p->refcol,
-      GAL_TYPE_STRING,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
     {
       "greaterequal",
       UI_KEY_GREATEREQUAL,
@@ -209,6 +196,20 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_add_to_single_value
     },
+    {
+      "mad",
+      UI_KEY_MAD,
+      0,
+      0,
+      "Median absolute deviation.",
+      UI_GROUP_SINGLE_VALUE,
+      &p->singlevalue,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value
+    },
     {
       "median",
       UI_KEY_MEDIAN,
@@ -377,6 +378,104 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_add_to_single_value
     },
+    {
+      "sigclip-mad",
+      UI_KEY_SIGCLIPMAD,
+      0,
+      0,
+      "Sigma-clipped Median Absolute Deviation.",
+      UI_GROUP_SINGLE_VALUE,
+      &p->singlevalue,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value
+    },
+    {
+      "madclip-number",
+      UI_KEY_MADCLIPNUMBER,
+      0,
+      0,
+      "Number of elements after MAD-clipping.",
+      UI_GROUP_SINGLE_VALUE,
+      &p->singlevalue,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value
+    },
+    {
+      "madclip-median",
+      UI_KEY_MADCLIPMEDIAN,
+      0,
+      0,
+      "MAD-clipped median.",
+      UI_GROUP_SINGLE_VALUE,
+      &p->singlevalue,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value
+    },
+    {
+      "madclip-mean",
+      UI_KEY_MADCLIPMEAN,
+      0,
+      0,
+      "MAD-clipped mean.",
+      UI_GROUP_SINGLE_VALUE,
+      &p->singlevalue,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value
+    },
+    {
+      "madclip-std",
+      UI_KEY_MADCLIPSTD,
+      0,
+      0,
+      "MAD-clipped standard deviation.",
+      UI_GROUP_SINGLE_VALUE,
+      &p->singlevalue,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value
+    },
+    {
+      "madclip-mad",
+      UI_KEY_MADCLIPMAD,
+      0,
+      0,
+      "MAD-clipped Median Absolute Deviation.",
+      UI_GROUP_SINGLE_VALUE,
+      &p->singlevalue,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value
+    },
+    {
+      "concentration",
+      UI_KEY_CONCENTRATION,
+      "FLT[,...]",
+      0,
+      "Uniform dist: 1.0; higher: concentrated.",
+      UI_GROUP_SINGLE_VALUE,
+      &p->singlevalue,
+      GAL_TYPE_FLOAT32,
+      GAL_OPTIONS_RANGE_GT_0_LT_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value
+    },
 
 
 
@@ -504,6 +603,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "madclip",
+      UI_KEY_MADCLIP,
+      0,
+      0,
+      "Overall MAD-clipping (see '--mclipparams')",
+      UI_GROUP_PARTICULAR_STAT,
+      &p->madclip,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "contour",
       UI_KEY_CONTOUR,
@@ -581,6 +693,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "outliernumngb",
+      UI_KEY_OUTLIERNUMNGB,
+      "INT",
+      0,
+      "Num neighboring tiles to look for outlier.",
+      UI_GROUP_SKY,
+      &p->outliernumngb,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "outliersclip",
       UI_KEY_OUTLIERSCLIP,
@@ -622,6 +747,20 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       gal_options_read_sigma_clip
     },
+    {
+      "mclipparams",
+      UI_KEY_MCLIPPARAMS,
+      "FLT,FLT",
+      0,
+      "MAD clip: Multiple, and tolerance/number.",
+      UI_GROUP_SKY,
+      p->mclipparams,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_read_sigma_clip
+    },
     {
       "smoothwidth",
       UI_KEY_SMOOTHWIDTH,
@@ -648,6 +787,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "checkskynointerp",
+      UI_KEY_CHECKSKYNOINTERP,
+      0,
+      0,
+      "Similar to --checksky, stops at interpolation.",
+      UI_GROUP_SKY,
+      &p->checkskynointerp,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "ignoreblankintiles",
       UI_KEY_IGNOREBLANKINTILES,
@@ -815,6 +967,108 @@ struct argp_option program_options[] =
     },
 
 
+    {
+      0, 0, 0, 0,
+      "Fitting (1 and 2 dimensional)",
+      UI_GROUP_FIT
+    },
+    {
+      "fit",
+      UI_KEY_FIT,
+      "STR",
+      0,
+      "Type of fitting: 'linear', 'linear-weighted', "
+      "'linear-no-constant', 'linear-no-constant-weighted', "
+      "'polynomial', 'polynomial-weighted', "
+      "'polynomial-robust'.",
+      UI_GROUP_FIT,
+      &p->fitname,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitweight",
+      UI_KEY_FITWEIGHT,
+      "STR",
+      0,
+      "Input weight: 'std', 'var' or 'inv-variance'.",
+      UI_GROUP_FIT,
+      &p->fitweight,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitmaxpower",
+      UI_KEY_FITMAXPOWER,
+      "INT",
+      0,
+      "Maximum power in polynomial to fit.",
+      UI_GROUP_FIT,
+      &p->fitmaxpower,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitrobust",
+      UI_KEY_FITROBUST,
+      "STR",
+      0,
+      "The robust function name to use.",
+      UI_GROUP_FIT,
+      &p->fitrobustname,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitestimate",
+      UI_KEY_FITESTIMATE,
+      "STR/FLT",
+      0,
+      "Estimate fitted func. (number, file or 'self').",
+      UI_GROUP_FIT,
+      &p->fitestimate,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitestimatehdu",
+      UI_KEY_FITESTIMATEHDU,
+      "STR/INT",
+      0,
+      "HDU containing the --fitestimate values.",
+      UI_GROUP_FIT,
+      &p->fitestimatehdu,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fitestimatecol",
+      UI_KEY_FITESTIMATECOL,
+      "STR/INT",
+      0,
+      "Column containing the --fitestimate values.",
+      UI_GROUP_FIT,
+      &p->fitestimatecol,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
+
     {0}
   };
 
