@@ -1,4 +1,4 @@
-/* Copyright (C) 1991-1992, 1997, 1999, 2003, 2006, 2008-2022 Free Software
+/* Copyright (C) 1991-1992, 1997, 1999, 2003, 2006, 2008-2025 Free Software
    Foundation, Inc.
 
    This file is free software: you can redistribute it and/or modify
@@ -14,7 +14,7 @@
    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
-#if ! defined USE_LONG_DOUBLE
+#if ! (defined USE_FLOAT || defined USE_LONG_DOUBLE)
 # include <config.h>
 #endif
 
@@ -23,7 +23,7 @@
 
 #include <ctype.h>      /* isspace() */
 #include <errno.h>
-#include <float.h>      /* {DBL,LDBL}_{MIN,MAX} */
+#include <float.h>      /* {FLT,DBL,LDBL}_{MIN,MAX} */
 #include <limits.h>     /* LONG_{MIN,MAX} */
 #include <locale.h>     /* localeconv() */
 #include <math.h>       /* NAN */
@@ -37,7 +37,26 @@
 
 #undef MIN
 #undef MAX
-#ifdef USE_LONG_DOUBLE
+#if defined USE_FLOAT
+# define STRTOD strtof
+# define LDEXP ldexpf
+# if STRTOF_HAS_UNDERFLOW_BUG
+   /* strtof would not set errno=ERANGE upon flush-to-zero underflow.  */
+#  define HAVE_UNDERLYING_STRTOD 0
+# else
+#  define HAVE_UNDERLYING_STRTOD HAVE_STRTOF
+# endif
+# define HAS_GRADUAL_UNDERFLOW_PROBLEM STRTOF_HAS_GRADUAL_UNDERFLOW_PROBLEM
+# define DOUBLE float
+# define MIN FLT_MIN
+# define MAX FLT_MAX
+# define L_(literal) literal##f
+# if HAVE_LDEXPF_IN_LIBC
+#  define USE_LDEXP 1
+# else
+#  define USE_LDEXP 0
+# endif
+#elif defined USE_LONG_DOUBLE
 # define STRTOD strtold
 # define LDEXP ldexpl
 # if defined __hpux && defined __hppa
@@ -45,29 +64,51 @@
       not a 'long double'.  */
 #  define HAVE_UNDERLYING_STRTOD 0
 # elif STRTOLD_HAS_UNDERFLOW_BUG
-   /* strtold would not set errno=ERANGE upon underflow.  */
+   /* strtold would not set errno=ERANGE upon flush-to-zero underflow.  */
+#  define HAVE_UNDERLYING_STRTOD 0
+# elif defined __MINGW32__ && __MINGW64_VERSION_MAJOR < 10
+   /* strtold is broken in mingw versions before 10.0:
+      - Up to mingw 5.0.x, it leaks memory at every invocation.
+      - Up to mingw 9.0.x, it allocates an unbounded amount of stack.
+      See <https://github.com/mingw-w64/mingw-w64/commit/450309b97b2e839ea02887dfaf0f1d10fb5d40cc>
+      and <https://github.com/mingw-w64/mingw-w64/commit/73806c0709b7e6c0f6587f11a955743670e85470>.  */
+#  define HAVE_UNDERLYING_STRTOD 0
+# elif defined __HAIKU__
+   /* Haiku's strtold maps denormalized numbers to zero.
+      <https://dev.haiku-os.org/ticket/19040>  */
 #  define HAVE_UNDERLYING_STRTOD 0
 # else
 #  define HAVE_UNDERLYING_STRTOD HAVE_STRTOLD
 # endif
+# define HAS_GRADUAL_UNDERFLOW_PROBLEM STRTOLD_HAS_GRADUAL_UNDERFLOW_PROBLEM
 # define DOUBLE long double
 # define MIN LDBL_MIN
 # define MAX LDBL_MAX
 # define L_(literal) literal##L
+# if HAVE_LDEXPL_IN_LIBC
+#  define USE_LDEXP 1
+# else
+#  define USE_LDEXP 0
+# endif
 #else
 # define STRTOD strtod
 # define LDEXP ldexp
-# define HAVE_UNDERLYING_STRTOD 1
+# if STRTOD_HAS_UNDERFLOW_BUG
+   /* strtod would not set errno=ERANGE upon flush-to-zero underflow.  */
+#  define HAVE_UNDERLYING_STRTOD 0
+# else
+#  define HAVE_UNDERLYING_STRTOD 1
+# endif
+# define HAS_GRADUAL_UNDERFLOW_PROBLEM STRTOD_HAS_GRADUAL_UNDERFLOW_PROBLEM
 # define DOUBLE double
 # define MIN DBL_MIN
 # define MAX DBL_MAX
 # define L_(literal) literal
-#endif
-
-#if (defined USE_LONG_DOUBLE ? HAVE_LDEXPM_IN_LIBC : HAVE_LDEXP_IN_LIBC)
-# define USE_LDEXP 1
-#else
-# define USE_LDEXP 0
+# if HAVE_LDEXP_IN_LIBC
+#  define USE_LDEXP 1
+# else
+#  define USE_LDEXP 0
+# endif
 #endif
 
 /* Return true if C is a space in the current locale, avoiding
@@ -134,11 +175,20 @@ scale_radix_exp (DOUBLE x, int radix, long int exponent)
         {
           if (e < 0)
             {
-              while (e++ != 0)
+              for (;;)
                 {
-                  r /= radix;
-                  if (r == 0 && x != 0)
+                  if (e++ == 0)
                     {
+                      if (r < MIN && r > -MIN)
+                        /* Gradual underflow, resulting in a denormalized
+                           number.  */
+                        errno = ERANGE;
+                      break;
+                    }
+                  r /= radix;
+                  if (r == 0)
+                    {
+                      /* Flush-to-zero underflow.  */
                       errno = ERANGE;
                       break;
                     }
@@ -233,9 +283,8 @@ parse_number (const char *nptr,
 
   /* Then, convert the digit sequence to a number.  */
   {
-    const char *dp;
     num = 0;
-    for (dp = digits_start; dp < digits_end; dp++)
+    for (const char *dp = digits_start; dp < digits_end; dp++)
       if (dp != radixchar_ptr)
         {
           int digit;
@@ -299,7 +348,7 @@ parse_number (const char *nptr,
 static DOUBLE
 minus_zero (void)
 {
-#if defined __hpux || defined __sgi || defined __ICC
+#if defined __hpux || defined __ICC
   return -MIN * MIN;
 #else
   return -0.0;
@@ -311,15 +360,29 @@ minus_zero (void)
 DOUBLE
 STRTOD (const char *nptr, char **endptr)
 #if HAVE_UNDERLYING_STRTOD
-# ifdef USE_LONG_DOUBLE
+# if defined USE_FLOAT
+#  undef strtof
+# elif defined USE_LONG_DOUBLE
 #  undef strtold
 # else
 #  undef strtod
 # endif
+# if HAS_GRADUAL_UNDERFLOW_PROBLEM
+#  define SET_ERRNO_UPON_GRADUAL_UNDERFLOW(RESULT) \
+    do                                                          \
+      {                                                         \
+        if ((RESULT) != 0 && (RESULT) < MIN && (RESULT) > -MIN) \
+          errno = ERANGE;                                       \
+      }                                                         \
+    while (0)
+# else
+#  define SET_ERRNO_UPON_GRADUAL_UNDERFLOW(RESULT) (void)0
+# endif
 #else
 # undef STRTOD
 # define STRTOD(NPTR,ENDPTR) \
    parse_number (NPTR, 10, 10, 1, radixchar, 'e', ENDPTR)
+# define SET_ERRNO_UPON_GRADUAL_UNDERFLOW(RESULT) (void)0
 #endif
 /* From here on, STRTOD refers to the underlying implementation.  It needs
    to handle only finite unsigned decimal numbers with non-null ENDPTR.  */
@@ -347,6 +410,7 @@ STRTOD (const char *nptr, char **endptr)
     ++s;
 
   num = STRTOD (s, &endbuf);
+  SET_ERRNO_UPON_GRADUAL_UNDERFLOW (num);
   end = endbuf;
 
   if (c_isdigit (s[*s == radixchar]))
@@ -389,6 +453,7 @@ STRTOD (const char *nptr, char **endptr)
                     {
                       dup[p - s] = '\0';
                       num = STRTOD (dup, &endbuf);
+                      SET_ERRNO_UPON_GRADUAL_UNDERFLOW (num);
                       saved_errno = errno;
                       free (dup);
                       errno = saved_errno;
@@ -419,12 +484,17 @@ STRTOD (const char *nptr, char **endptr)
                 {
                   dup[e - s] = '\0';
                   num = STRTOD (dup, &endbuf);
+                  SET_ERRNO_UPON_GRADUAL_UNDERFLOW (num);
                   saved_errno = errno;
                   free (dup);
                   errno = saved_errno;
                 }
               end = e;
             }
+          /* If "1e50" was converted to Inf (overflow), errno needs to be
+             set.  */
+          else if (isinf (num))
+            errno = ERANGE;
         }
 
       s = end;
