@@ -1,5 +1,5 @@
 /* Test of gl_locale_name function and its variants.
-   Copyright (C) 2007-2023 Free Software Foundation, Inc.
+   Copyright (C) 2007-2025 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 
 #include "macros.h"
 
-#if HAVE_WORKING_NEWLOCALE && HAVE_WORKING_USELOCALE && !HAVE_FAKE_LOCALES
+#if HAVE_WORKING_USELOCALE && !HAVE_FAKE_LOCALES
 # define HAVE_GOOD_USELOCALE 1
 #endif
 
@@ -36,10 +36,30 @@
 #endif
 
 /* Suppress GCC false positive.  */
-#if __GNUC__ >= 12
+#if _GL_GNUC_PREREQ (12, 0)
 # pragma GCC diagnostic ignored "-Wanalyzer-use-of-uninitialized-value"
 #endif
 
+/* The name that setlocale(,NULL) returns for the "C" locale.  */
+#ifdef __HAIKU__
+# define C_CANONICALIZED "POSIX"
+#else
+# define C_CANONICALIZED "C"
+#endif
+
+static int
+is_default (const char *name)
+{
+  return strcmp (name, gl_locale_name_default ()) == 0
+         || (strcmp (name, C_CANONICALIZED) == 0
+             && strcmp (gl_locale_name_default (), "C") == 0)
+#if MUSL_LIBC
+         || (strcmp (name, "C.UTF-8") == 0
+             && strcmp (gl_locale_name_default (), "C") == 0)
+#endif
+         ;
+}
+
 #if HAVE_GOOD_USELOCALE
 
 static struct { int cat; int mask; const char *string; } const categories[] =
@@ -110,10 +130,8 @@ test_locale_name (void)
   unsetenv ("LC_TELEPHONE");
   ret = setlocale (LC_ALL, "");
   ASSERT (ret != NULL);
-  ASSERT (strcmp (gl_locale_name (LC_MESSAGES, "LC_MESSAGES"),
-                  gl_locale_name_default ()) == 0);
-  ASSERT (strcmp (gl_locale_name (LC_NUMERIC, "LC_NUMERIC"),
-                  gl_locale_name_default ()) == 0);
+  ASSERT (is_default (gl_locale_name (LC_MESSAGES, "LC_MESSAGES")));
+  ASSERT (is_default (gl_locale_name (LC_NUMERIC, "LC_NUMERIC")));
 
   /* Check that an empty environment variable is treated like an unset
      environment variable.  */
@@ -123,32 +141,28 @@ test_locale_name (void)
   unsetenv ("LC_MESSAGES");
   unsetenv ("LANG");
   setlocale (LC_ALL, "");
-  ASSERT (strcmp (gl_locale_name (LC_MESSAGES, "LC_MESSAGES"),
-                  gl_locale_name_default ()) == 0);
+  ASSERT (is_default (gl_locale_name (LC_MESSAGES, "LC_MESSAGES")));
 
   unsetenv ("LC_ALL");
   setenv ("LC_CTYPE", "", 1);
   unsetenv ("LC_MESSAGES");
   unsetenv ("LANG");
   setlocale (LC_ALL, "");
-  ASSERT (strcmp (gl_locale_name (LC_MESSAGES, "LC_MESSAGES"),
-                  gl_locale_name_default ()) == 0);
+  ASSERT (is_default (gl_locale_name (LC_MESSAGES, "LC_MESSAGES")));
 
   unsetenv ("LC_ALL");
   unsetenv ("LC_CTYPE");
   setenv ("LC_MESSAGES", "", 1);
   unsetenv ("LANG");
   setlocale (LC_ALL, "");
-  ASSERT (strcmp (gl_locale_name (LC_MESSAGES, "LC_MESSAGES"),
-                  gl_locale_name_default ()) == 0);
+  ASSERT (is_default (gl_locale_name (LC_MESSAGES, "LC_MESSAGES")));
 
   unsetenv ("LC_ALL");
   unsetenv ("LC_CTYPE");
   unsetenv ("LC_MESSAGES");
   setenv ("LANG", "", 1);
   setlocale (LC_ALL, "");
-  ASSERT (strcmp (gl_locale_name (LC_MESSAGES, "LC_MESSAGES"),
-                  gl_locale_name_default ()) == 0);
+  ASSERT (is_default (gl_locale_name (LC_MESSAGES, "LC_MESSAGES")));
 
   /* Check that LC_ALL overrides the others, and LANG is overridden by the
      others.  */
@@ -158,21 +172,24 @@ test_locale_name (void)
   unsetenv ("LC_MESSAGES");
   unsetenv ("LANG");
   setlocale (LC_ALL, "");
-  ASSERT (strcmp (gl_locale_name (LC_MESSAGES, "LC_MESSAGES"), "C") == 0);
+  ASSERT (strcmp (gl_locale_name (LC_MESSAGES, "LC_MESSAGES"),
+                  C_CANONICALIZED) == 0);
 
   unsetenv ("LC_ALL");
   setenv ("LC_CTYPE", "C", 1);
   setenv ("LC_MESSAGES", "C", 1);
   unsetenv ("LANG");
   setlocale (LC_ALL, "");
-  ASSERT (strcmp (gl_locale_name (LC_MESSAGES, "LC_MESSAGES"), "C") == 0);
+  ASSERT (strcmp (gl_locale_name (LC_MESSAGES, "LC_MESSAGES"),
+                  C_CANONICALIZED) == 0);
 
   unsetenv ("LC_ALL");
   unsetenv ("LC_CTYPE");
   unsetenv ("LC_MESSAGES");
   setenv ("LANG", "C", 1);
   setlocale (LC_ALL, "");
-  ASSERT (strcmp (gl_locale_name (LC_MESSAGES, "LC_MESSAGES"), "C") == 0);
+  ASSERT (strcmp (gl_locale_name (LC_MESSAGES, "LC_MESSAGES"),
+                  C_CANONICALIZED) == 0);
 
   /* Check mixed situations.  */
 
@@ -204,7 +221,7 @@ test_locale_name (void)
   if (setlocale (LC_ALL, "") != NULL)
     {
       name = gl_locale_name (LC_CTYPE, "LC_CTYPE");
-      ASSERT (strcmp (name, gl_locale_name_default ()) == 0);
+      ASSERT (is_default (name));
       name = gl_locale_name (LC_MESSAGES, "LC_MESSAGES");
       ASSERT (strcmp (name, "fr_FR.UTF-8") == 0);
     }
@@ -227,38 +244,32 @@ test_locale_name (void)
 
   /* Check that gl_locale_name distinguishes different categories of the
      thread locale, and that the name is the right one for each.  */
-  {
-    unsigned int i;
-
-    for (i = 0; i < SIZEOF (categories); i++)
-      {
-        int category_mask = categories[i].mask;
-        locale_t loc = newlocale (LC_ALL_MASK, "fr_FR.UTF-8", NULL);
-        if (loc != NULL)
-          {
-            locale_t locale = newlocale (category_mask, "de_DE.UTF-8", loc);
-            if (locale == NULL)
-              freelocale (loc);
-            else
-              {
-                unsigned int j;
-
-                uselocale (locale);
-                for (j = 0; j < SIZEOF (categories); j++)
-                  {
-                    const char *name_j =
-                      gl_locale_name (categories[j].cat, categories[j].string);
-                    if (j == i)
-                      ASSERT (strcmp (name_j, "de_DE.UTF-8") == 0);
-                    else
-                      ASSERT (strcmp (name_j, "fr_FR.UTF-8") == 0);
-                  }
-                uselocale (LC_GLOBAL_LOCALE);
-                freelocale (locale);
-              }
-          }
-      }
-  }
+  for (unsigned int i = 0; i < SIZEOF (categories); i++)
+    {
+      int category_mask = categories[i].mask;
+      locale_t loc = newlocale (LC_ALL_MASK, "fr_FR.UTF-8", NULL);
+      if (loc != NULL)
+        {
+          locale_t locale = newlocale (category_mask, "de_DE.UTF-8", loc);
+          if (locale == NULL)
+            freelocale (loc);
+          else
+            {
+              uselocale (locale);
+              for (unsigned int j = 0; j < SIZEOF (categories); j++)
+                {
+                  const char *name_j =
+                    gl_locale_name (categories[j].cat, categories[j].string);
+                  if (j == i)
+                    ASSERT (strcmp (name_j, "de_DE.UTF-8") == 0);
+                  else
+                    ASSERT (strcmp (name_j, "fr_FR.UTF-8") == 0);
+                }
+              uselocale (LC_GLOBAL_LOCALE);
+              freelocale (locale);
+            }
+        }
+    }
 #endif
 }
 
@@ -295,39 +306,33 @@ test_locale_name_thread (void)
 
   /* Check that gl_locale_name_thread distinguishes different categories of the
      thread locale, and that the name is the right one for each.  */
-  {
-    unsigned int i;
-
-    for (i = 0; i < SIZEOF (categories); i++)
-      {
-        int category_mask = categories[i].mask;
-        locale_t loc = newlocale (LC_ALL_MASK, "fr_FR.UTF-8", NULL);
-        if (loc != NULL)
-          {
-            locale_t locale = newlocale (category_mask, "de_DE.UTF-8", loc);
-            if (locale == NULL)
-              freelocale (loc);
-            else
-              {
-                unsigned int j;
-
-                uselocale (locale);
-                for (j = 0; j < SIZEOF (categories); j++)
-                  {
-                    const char *name_j =
-                      gl_locale_name_thread (categories[j].cat,
-                                             categories[j].string);
-                    if (j == i)
-                      ASSERT (strcmp (name_j, "de_DE.UTF-8") == 0);
-                    else
-                      ASSERT (strcmp (name_j, "fr_FR.UTF-8") == 0);
-                  }
-                uselocale (LC_GLOBAL_LOCALE);
-                freelocale (locale);
-              }
-          }
-      }
-  }
+  for (unsigned int i = 0; i < SIZEOF (categories); i++)
+    {
+      int category_mask = categories[i].mask;
+      locale_t loc = newlocale (LC_ALL_MASK, "fr_FR.UTF-8", NULL);
+      if (loc != NULL)
+        {
+          locale_t locale = newlocale (category_mask, "de_DE.UTF-8", loc);
+          if (locale == NULL)
+            freelocale (loc);
+          else
+            {
+              uselocale (locale);
+              for (unsigned int j = 0; j < SIZEOF (categories); j++)
+                {
+                  const char *name_j =
+                    gl_locale_name_thread (categories[j].cat,
+                                           categories[j].string);
+                  if (j == i)
+                    ASSERT (strcmp (name_j, "de_DE.UTF-8") == 0);
+                  else
+                    ASSERT (strcmp (name_j, "fr_FR.UTF-8") == 0);
+                }
+              uselocale (LC_GLOBAL_LOCALE);
+              freelocale (locale);
+            }
+        }
+    }
 
   /* Check that gl_locale_name_thread returns a string that is allocated with
      indefinite extent.  */
@@ -446,18 +451,15 @@ test_locale_name_thread (void)
     /* Array of remembered results of gl_locale_name_thread, stored in safe
        memory.  */
     char *saved_names[SIZEOF (choices)][SIZEOF (categories)];
-    unsigned int j;
 
-    for (j = 0; j < SIZEOF (choices); j++)
+    for (unsigned int j = 0; j < SIZEOF (choices); j++)
       {
         locale_t locale = newlocale (LC_ALL_MASK, choices[j], NULL);
         available[j] = (locale != NULL);
         if (locale != NULL)
           {
-            unsigned int i;
-
             uselocale (locale);
-            for (i = 0; i < SIZEOF (categories); i++)
+            for (unsigned int i = 0; i < SIZEOF (categories); i++)
               {
                 unsaved_names[j][i] = gl_locale_name_thread (categories[i].cat, categories[i].string);
                 saved_names[j][i] = strdup (unsaved_names[j][i]);
@@ -467,27 +469,24 @@ test_locale_name_thread (void)
           }
       }
     /* Verify the unsaved_names are still valid.  */
-    for (j = 0; j < SIZEOF (choices); j++)
+    for (unsigned int j = 0; j < SIZEOF (choices); j++)
       if (available[j])
         {
-          unsigned int i;
-
-          for (i = 0; i < SIZEOF (categories); i++)
+          for (unsigned int i = 0; i < SIZEOF (categories); i++)
             ASSERT (strcmp (unsaved_names[j][i], saved_names[j][i]) == 0);
         }
     /* Allocate many locales, without freeing them.  This is an attempt at
        overwriting as much of the previously allocated memory as possible.  */
-    for (j = SIZEOF (choices); j > 0; )
+    for (unsigned int j = SIZEOF (choices); j > 0; )
       {
         j--;
         if (available[j])
           {
             locale_t locale = newlocale (LC_ALL_MASK, choices[j], NULL);
-            unsigned int i;
 
             ASSERT (locale != NULL);
             uselocale (locale);
-            for (i = 0; i < SIZEOF (categories); i++)
+            for (unsigned int i = 0; i < SIZEOF (categories); i++)
               {
                 const char *name = gl_locale_name_thread (categories[i].cat, categories[i].string);
                 ASSERT (strcmp (unsaved_names[j][i], name) == 0);
@@ -497,12 +496,10 @@ test_locale_name_thread (void)
           }
       }
     /* Verify the unsaved_names are still valid.  */
-    for (j = 0; j < SIZEOF (choices); j++)
+    for (unsigned int j = 0; j < SIZEOF (choices); j++)
       if (available[j])
         {
-          unsigned int i;
-
-          for (i = 0; i < SIZEOF (categories); i++)
+          for (unsigned int i = 0; i < SIZEOF (categories); i++)
             {
               ASSERT (strcmp (unsaved_names[j][i], saved_names[j][i]) == 0);
               free (saved_names[j][i]);
@@ -552,9 +549,9 @@ test_locale_name_posix (void)
   ret = setlocale (LC_ALL, "");
   ASSERT (ret != NULL);
   name = gl_locale_name_posix (LC_MESSAGES, "LC_MESSAGES");
-  ASSERT (name == NULL || strcmp (name, gl_locale_name_default ()) == 0);
+  ASSERT (name == NULL || is_default (name));
   name = gl_locale_name_posix (LC_NUMERIC, "LC_NUMERIC");
-  ASSERT (name == NULL || strcmp (name, gl_locale_name_default ()) == 0);
+  ASSERT (name == NULL || is_default (name));
 
   /* Check that an empty environment variable is treated like an unset
      environment variable.  */
@@ -565,7 +562,7 @@ test_locale_name_posix (void)
   unsetenv ("LANG");
   setlocale (LC_ALL, "");
   name = gl_locale_name_posix (LC_MESSAGES, "LC_MESSAGES");
-  ASSERT (name == NULL || strcmp (name, gl_locale_name_default ()) == 0);
+  ASSERT (name == NULL || is_default (name));
 
   unsetenv ("LC_ALL");
   setenv ("LC_CTYPE", "", 1);
@@ -573,7 +570,7 @@ test_locale_name_posix (void)
   unsetenv ("LANG");
   setlocale (LC_ALL, "");
   name = gl_locale_name_posix (LC_MESSAGES, "LC_MESSAGES");
-  ASSERT (name == NULL || strcmp (name, gl_locale_name_default ()) == 0);
+  ASSERT (name == NULL || is_default (name));
 
   unsetenv ("LC_ALL");
   unsetenv ("LC_CTYPE");
@@ -581,7 +578,7 @@ test_locale_name_posix (void)
   unsetenv ("LANG");
   setlocale (LC_ALL, "");
   name = gl_locale_name_posix (LC_MESSAGES, "LC_MESSAGES");
-  ASSERT (name == NULL || strcmp (name, gl_locale_name_default ()) == 0);
+  ASSERT (name == NULL || is_default (name));
 
   unsetenv ("LC_ALL");
   unsetenv ("LC_CTYPE");
@@ -589,7 +586,7 @@ test_locale_name_posix (void)
   setenv ("LANG", "", 1);
   setlocale (LC_ALL, "");
   name = gl_locale_name_posix (LC_MESSAGES, "LC_MESSAGES");
-  ASSERT (name == NULL || strcmp (name, gl_locale_name_default ()) == 0);
+  ASSERT (name == NULL || is_default (name));
 
   /* Check that LC_ALL overrides the others, and LANG is overridden by the
      others.  */
@@ -600,7 +597,7 @@ test_locale_name_posix (void)
   unsetenv ("LANG");
   setlocale (LC_ALL, "");
   name = gl_locale_name_posix (LC_MESSAGES, "LC_MESSAGES");
-  ASSERT (strcmp (name, "C") == 0);
+  ASSERT (strcmp (name, C_CANONICALIZED) == 0);
 
   unsetenv ("LC_ALL");
   setenv ("LC_CTYPE", "C", 1);
@@ -608,7 +605,7 @@ test_locale_name_posix (void)
   unsetenv ("LANG");
   setlocale (LC_ALL, "");
   name = gl_locale_name_posix (LC_MESSAGES, "LC_MESSAGES");
-  ASSERT (strcmp (name, "C") == 0);
+  ASSERT (strcmp (name, C_CANONICALIZED) == 0);
 
   unsetenv ("LC_ALL");
   unsetenv ("LC_CTYPE");
@@ -616,7 +613,7 @@ test_locale_name_posix (void)
   setenv ("LANG", "C", 1);
   setlocale (LC_ALL, "");
   name = gl_locale_name_posix (LC_MESSAGES, "LC_MESSAGES");
-  ASSERT (strcmp (name, "C") == 0);
+  ASSERT (strcmp (name, C_CANONICALIZED) == 0);
 
   /* Check mixed situations.  */
 
@@ -643,7 +640,7 @@ test_locale_name_posix (void)
   if (setlocale (LC_ALL, "") != NULL)
     {
       name = gl_locale_name_posix (LC_CTYPE, "LC_CTYPE");
-      ASSERT (name == NULL || strcmp (name, gl_locale_name_default ()) == 0);
+      ASSERT (name == NULL || is_default (name));
       name = gl_locale_name_posix (LC_MESSAGES, "LC_MESSAGES");
       ASSERT (strcmp (name, "fr_FR.UTF-8") == 0);
     }
@@ -661,7 +658,7 @@ test_locale_name_posix (void)
         setlocale (LC_ALL, "");
         uselocale (locale);
         name = gl_locale_name_posix (LC_MESSAGES, "LC_MESSAGES");
-        ASSERT (strcmp (name, "C") == 0);
+        ASSERT (strcmp (name, C_CANONICALIZED) == 0);
         uselocale (LC_GLOBAL_LOCALE);
         freelocale (locale);
       }
@@ -821,5 +818,5 @@ main ()
   test_locale_name_environ ();
   test_locale_name_default ();
 
-  return 0;
+  return test_exit_status;
 }
