@@ -1,5 +1,5 @@
 /* Test of perror() function.
-   Copyright (C) 2011-2023 Free Software Foundation, Inc.
+   Copyright (C) 2011-2025 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include <unistd.h>
 
 /* Tell GCC not to warn about myerr being leaked.  */
-#if __GNUC__ >= 13
+#if _GL_GNUC_PREREQ (13, 0)
 # pragma GCC diagnostic ignored "-Wanalyzer-fd-leak"
 #endif
 
@@ -95,8 +95,7 @@ main (void)
   /* Test that perror uses the same message as strerror.  */
   {
     int errs[] = { EACCES, 0, -3, };
-    int i;
-    for (i = 0; i < SIZEOF (errs); i++)
+    for (int i = 0; i < SIZEOF (errs); i++)
       {
         char buf[256];
         const char *err = strerror (errs[i]);
@@ -126,7 +125,7 @@ main (void)
        https://sourceware.org/ml/newlib/2011/msg00228.html */
     ASSERT (errno > 0);
     /* Commented out until glibc behaves:
-       https://sourceware.org/bugzilla/show_bug.cgi?id=12792 */
+       https://sourceware.org/PR12792 */
     ASSERT (ferror (stderr));
 #endif
   }
@@ -134,5 +133,5 @@ main (void)
   ASSERT (fclose (stderr) == 0);
   ASSERT (remove (BASE ".tmp") == 0);
 
-  return 0;
+  return test_exit_status;
 }
