@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2024 Free Software Foundation, Inc.
+Copyright (C) 2016-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -81,8 +81,8 @@ gal_data_alloc(void *array, uint8_t type, size_t ndim, size_t *dsize,
           __func__, sizeof *out);
 
   /* Initialize the allocated array. */
-  gal_data_initialize(out, array, type, ndim, dsize, wcs, clear, minmapsize,
-                      quietmmap, name, unit, comment);
+  gal_data_initialize(out, array, type, ndim, dsize, wcs, clear,
+                      minmapsize, quietmmap, name, unit, comment);
 
   /* Return the final structure. */
   return out;
@@ -245,17 +245,17 @@ gal_data_alloc_empty(size_t ndim, size_t minmapsize, int quietmmap)
 
 
 /* Free the allocated contents of a data structure, not the structure
-   itsself. The reason that this function is separate from 'gal_data_free'
+   itself. The reason that this function is separate from 'gal_data_free'
    is that the data structure might be allocated as an array (statically
    like 'gal_data_t da[20]', or dynamically like 'gal_data_t *da;
    da=malloc(20*sizeof *da);'). In both cases, a loop will be necessary to
    delete the allocated contents of each element of the data structure
-   array, but not the structure its self. After that loop, if the array of
+   array, but not the structure itself. After that loop, if the array of
    data structures was statically allocated, you don't have to do
    anything. If it was dynamically allocated, we just have to run
    'free(da)'.
 
-   Since we aren't freeing the 'gal_data_t' its-self, after the allocated
+   Since we aren't freeing the 'gal_data_t' itself, after the allocated
    space for each pointer is freed, the pointer is set to NULL for safety
    (to avoid possible re-calls).
 */
@@ -302,7 +302,7 @@ gal_data_free_contents(gal_data_t *data)
 
 
 /* Free the contents of the data structure and the data structure
-   itsself. */
+   itself. */
 void
 gal_data_free(gal_data_t *data)
 {
@@ -388,7 +388,7 @@ gal_data_array_free(gal_data_t *dataarr, size_t size, int free_array)
 {
   size_t i;
 
-  /* If its NULL, don't do anything. */
+  /* If it is NULL, don't do anything. */
   if(dataarr==NULL) return;
 
   /* First free all the contents. */
@@ -666,13 +666,13 @@ data_copy_to_string(gal_data_t *from, gal_data_t *to)
     case GAL_TYPE_STRLL:
     case GAL_TYPE_COMPLEX32:
     case GAL_TYPE_COMPLEX64:
-      error(EXIT_FAILURE, 0, "%s: copying to %s type not currently supported",
-            __func__, gal_type_name(from->type, 1));
+      error(EXIT_FAILURE, 0, "%s: copying to %s type not currently "
+            "supported", __func__, gal_type_name(from->type, 1));
       break;
 
     default:
-      error(EXIT_FAILURE, 0, "%s: type %d not recognized for 'from->type'",
-            __func__, from->type);
+      error(EXIT_FAILURE, 0, "%s: type %d not recognized for "
+            "'from->type'", __func__, from->type);
     }
 }
 
@@ -799,7 +799,8 @@ gal_data_copy_to_new_type(gal_data_t *in, uint8_t newtype)
                      0, in->minmapsize, in->quietmmap, in->name,
                      in->unit, in->comment);
 
-  /* Fill in the output array: */
+  /* Fill in the output array. */
+
   gal_data_copy_to_allocated(in, out);
 
   /* Return the created array */
@@ -868,13 +869,13 @@ gal_data_copy_to_allocated(gal_data_t *in, gal_data_t *out)
      output is not smaller than the input. Note that the type is irrelevant
      because we will be doing type conversion if they differ.*/
   if( out->size < in->size  )
-    error(EXIT_FAILURE, 0, "%s: the output dataset must be equal or larger "
-          "than the input. the sizes are %zu and %zu respectively", __func__,
-          out->size, in->size);
+    error(EXIT_FAILURE, 0, "%s: the output dataset must be equal or "
+          "larger than the input. the sizes are %zu and %zu respectively",
+          __func__, out->size, in->size);
   if( out->ndim != in->ndim )
-    error(EXIT_FAILURE, 0, "%s: the output dataset must have the same number "
-          "of dimensions, the dimensions are %zu and %zu respectively",
-          __func__, out->ndim, in->ndim);
+    error(EXIT_FAILURE, 0, "%s: the output dataset must have the same "
+          "number of dimensions, the dimensions are %zu and %zu "
+          "respectively", __func__, out->ndim, in->ndim);
 
   /* Free possibly allocated meta-data strings. */
   if(out->name)    free(out->name);
@@ -891,8 +892,10 @@ gal_data_copy_to_allocated(gal_data_t *in, gal_data_t *out)
   gal_checkset_allocate_copy(in->unit,    &out->unit);
   gal_checkset_allocate_copy(in->comment, &out->comment);
 
-  /* Do the copying. */
-  if(in->array)
+  /* Do the copying (only when 'array!=NULL' and size==0). Note that when
+     working in-place, it can happen that array is not NULL, but size is
+     zero. */
+  if(in->array && in->size>0)
     switch(out->type)
       {
       case GAL_TYPE_UINT8:   COPY_OT_SET( uint8_t  );      break;
@@ -946,6 +949,55 @@ gal_data_copy_to_allocated(gal_data_t *in, gal_data_t *out)
 
 
 
+/* Append the array of the second input dataset to the first one. */
+void
+gal_data_append_second_array_to_first_free(gal_data_t *a, gal_data_t *b)
+{
+  void *new;
+  size_t i, len;
+  char **strarr;
+
+  /* In case 'b' doesn't have any elements, return (nothing to do!). */
+  if(b && b->size==0) return;
+
+  /* Make sure the two arrays have the same type. */
+  if(a->type!=b->type)
+    error(EXIT_FAILURE, 0, "%s: the two input datasets must have "
+          "the same type. The first has a type of '%s' and the "
+          "second a type of '%s'", __func__,
+          gal_type_name(a->type, 1), gal_type_name(b->type, 1));
+
+  /* Make sure both arrays are one-dimensional (the only supported format
+     here). */
+  if(a->ndim!=1 || b->ndim!=1)
+    error(EXIT_FAILURE, 0, "%s: the two input datasets should only have "
+          "a single dimension. The first and second inputs respectively "
+          "have %zu and %zu dimensions", __func__, a->ndim, b->ndim);
+
+  /* Allocate the new array and copy the contents of the two inputs into
+     it. */
+  len=a->size+b->size;
+  new=gal_pointer_allocate(a->type, len, 0, __func__, "new");
+  memcpy(new, a->array, a->size*gal_type_sizeof(a->type));
+  memcpy(gal_pointer_increment(new, a->size, a->type),
+         b->array, b->size*gal_type_sizeof(b->type));
+
+  /* In case the second input had a string type, set all the contents to
+     'NULL' (so the actual strings are not freed), then free it. */
+  if(b->type==GAL_TYPE_STRING)
+    { strarr=b->array; for(i=0;i<b->size;++i) strarr[i]=NULL; }
+  gal_data_free(b);
+
+  /* Free the first input's array, and replace it with the new one. */
+  a->size=a->dsize[0]=len;
+  free(a->array);
+  a->array=new;
+}
+
+
+
+
+
 /* Just a wrapper around 'gal_type_from_string_auto', to return a
    'gal_data_t' dataset hosting the allocated number. */
 gal_data_t *
