@@ -5,7 +5,8 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2022-2024 Free Software Foundation, Inc.
+     Giacomo Lorenzetti <glorenzetti@cefca.es>
+Copyright (C) 2022-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -63,6 +64,8 @@ gal_fit_name_to_id(char *name)
     return GAL_FIT_POLYNOMIAL;
   else if( !strcmp(name, "polynomial-robust") )
     return GAL_FIT_POLYNOMIAL_ROBUST;
+  else if( !strcmp(name, "polynomial-tikhonov") )
+    return GAL_FIT_POLYNOMIAL_TIKHONOV;
   else return GAL_FIT_INVALID;
 
   /* If control reaches here, there was a bug! */
@@ -88,6 +91,7 @@ gal_fit_name_from_id(uint8_t fitid)
     case GAL_FIT_POLYNOMIAL:          return "polynomial";
     case GAL_FIT_POLYNOMIAL_WEIGHTED: return "polynomial-weighted";
     case GAL_FIT_POLYNOMIAL_ROBUST:   return "polynomial-robust";
+    case GAL_FIT_POLYNOMIAL_TIKHONOV: return "polynomial-tikhonov";
     case GAL_FIT_LINEAR_NO_CONSTANT_WEIGHTED:
       return "linear-no-constant-weighted";
     default: return NULL;
@@ -155,6 +159,28 @@ gal_fit_name_robust_from_id(uint8_t robustid)
 
 
 
+static char *
+fit_name_matrix_from_id(uint8_t matrixid)
+{
+  switch(matrixid)
+    {
+    case GAL_FIT_MATRIX_POLYNOMIAL_1D:     return "polynomial-1d";
+    case GAL_FIT_MATRIX_POLYNOMIAL_2D:     return "polynomial-2d";
+    case GAL_FIT_MATRIX_POLYNOMIAL_2D_TPV: return "polynomial-2d-tpv";
+    default:                               return NULL;
+    }
+
+  /* If control reaches here, there was a bug! */
+  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to "
+        "find a fix it. Control should not have reached this point",
+        __func__, PACKAGE_BUGREPORT);
+  return NULL;
+}
+
+
+
+
+
 
 
 
@@ -174,7 +200,7 @@ gal_fit_name_robust_from_id(uint8_t robustid)
 /****************            Common to all             ****************/
 /**********************************************************************/
 static gal_data_t *
-fit_1d_sanity_check(gal_data_t *in, gal_data_t *ref, const char *func)
+fit_sanity_check_col(gal_data_t *in, gal_data_t *ref, const char *func)
 {
   gal_data_t *out;
 
@@ -225,7 +251,7 @@ fit_1d_sanity_check(gal_data_t *in, gal_data_t *ref, const char *func)
 /****************              Linear fit              ****************/
 /**********************************************************************/
 static gal_data_t *
-fit_1d_linear_base(gal_data_t *xin, gal_data_t *yin,
+fit_linear_1d_base(gal_data_t *xin, gal_data_t *yin,
                    gal_data_t *ywht, int fitid)
 {
   double *o, nparam=NAN;
@@ -233,9 +259,12 @@ fit_1d_linear_base(gal_data_t *xin, gal_data_t *yin,
   gal_data_t *x=NULL, *y=NULL, *w=NULL, *out;
 
   /* Basic sanity checks. */
-  x=fit_1d_sanity_check(xin, xin, __func__);
-  y=fit_1d_sanity_check(yin, xin, __func__);
-  if(ywht) w=fit_1d_sanity_check(ywht, xin, __func__);
+  if(xin==NULL || xin->size==0 || yin==NULL || yin->size==0)
+    error(EXIT_FAILURE, 0, "%s: the inputs are either NULL or "
+          "do not contain any data (have a length of zero)", __func__);
+  x=fit_sanity_check_col(xin, xin, __func__);
+  y=fit_sanity_check_col(yin, xin, __func__);
+  if(ywht) w=fit_sanity_check_col(ywht, xin, __func__);
 
   /* Allocate the output dataset. */
   osize = ( fitid==GAL_FIT_LINEAR || fitid==GAL_FIT_LINEAR_WEIGHTED
@@ -249,7 +278,7 @@ fit_1d_linear_base(gal_data_t *xin, gal_data_t *yin,
     double *xa=x->array, *ya=y->array;
     for(i=0;i<x->size;++i)
       printf("%-15f %-15f\n", xa[i], ya[i]);
-  } */
+  } //*/
 
   /* Do the fitting. */
   o=out->array;
@@ -289,7 +318,7 @@ fit_1d_linear_base(gal_data_t *xin, gal_data_t *yin,
     printf("c0: %f\nc1: %f\n"
            "cov00: %f\ncov01: %f\ncov11: %f\nsumsq: %f\n",
            o[0], o[1], o[2], o[3], o[4], o[5]);
-  } */
+  } //*/
 
   /* Calculate the reduced chi^2: As mentioned in [1], in case we have the
      chi^2, then it is simply the chi^2 divided by the degrees of
@@ -314,9 +343,9 @@ fit_1d_linear_base(gal_data_t *xin, gal_data_t *yin,
 
 
 gal_data_t *
-gal_fit_1d_linear(gal_data_t *xin, gal_data_t *yin, gal_data_t *ywht)
+gal_fit_linear_1d(gal_data_t *xin, gal_data_t *yin, gal_data_t *ywht)
 {
-  return fit_1d_linear_base(xin, yin, ywht,
+  return fit_linear_1d_base(xin, yin, ywht,
                             ( ywht
                               ? GAL_FIT_LINEAR_WEIGHTED
                               : GAL_FIT_LINEAR));
@@ -327,10 +356,10 @@ gal_fit_1d_linear(gal_data_t *xin, gal_data_t *yin, gal_data_t *ywht)
 
 
 gal_data_t *
-gal_fit_1d_linear_no_constant(gal_data_t *xin, gal_data_t *yin,
+gal_fit_linear_no_constant_1d(gal_data_t *xin, gal_data_t *yin,
                               gal_data_t *ywht)
 {
-  return fit_1d_linear_base(xin, yin, ywht,
+  return fit_linear_1d_base(xin, yin, ywht,
                             ( ywht
                               ? GAL_FIT_LINEAR_NO_CONSTANT_WEIGHTED
                               : GAL_FIT_LINEAR_NO_CONSTANT) );
@@ -340,7 +369,7 @@ gal_fit_1d_linear_no_constant(gal_data_t *xin, gal_data_t *yin,
 
 
 
-gal_data_t *
+static gal_data_t *
 fit_1d_estimate_prepare(gal_data_t *xin, gal_data_t *fit, gal_data_t **xd,
                         const char *func)
 {
@@ -384,7 +413,7 @@ fit_1d_estimate_prepare(gal_data_t *xin, gal_data_t *fit, gal_data_t **xd,
 
 
 gal_data_t *
-gal_fit_1d_linear_estimate(gal_data_t *fit, gal_data_t *xin)
+gal_fit_linear_estimate_1d(gal_data_t *fit, gal_data_t *xin)
 {
   size_t i;
   gal_data_t *out=NULL, *xd;
@@ -415,7 +444,7 @@ gal_fit_1d_linear_estimate(gal_data_t *fit, gal_data_t *xin)
     default:                    /* Un-recognized situation! */
       error(EXIT_FAILURE, 0, "%s: the 'fit' argument should "
             "either have 6 or 3 elements (be an output of "
-            "'gal_fit_1d_linear' or 'gal_fit_1d_linear_no_constant'"
+            "'gal_fit_linear_1d' or 'gal_fit_linear_1d_no_constant'"
             "respectively), but it has %zu elements", __func__,
             fit->size);
     }
@@ -429,22 +458,61 @@ gal_fit_1d_linear_estimate(gal_data_t *fit, gal_data_t *xin)
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+/**********************************************************************/
+/****************           Polynomial fits            ****************/
+/**********************************************************************/
+static size_t
+fit_polynomial_nconst(uint8_t maxpower, uint8_t matrixid)
+{
+  size_t nconst=GAL_BLANK_SIZE_T;
+
+  switch(matrixid)
+    {
+    case GAL_FIT_MATRIX_POLYNOMIAL_1D:
+      nconst=maxpower+1;
+      break;
+    case GAL_FIT_MATRIX_POLYNOMIAL_2D:
+      nconst=(maxpower+1)*(maxpower+2)/2;
+      break;
+    case GAL_FIT_MATRIX_POLYNOMIAL_2D_TPV:
+      /* TPV has odd radial terms. */
+      nconst=(maxpower+1)*(maxpower+2)/2 + (maxpower/2 + 1);
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
+            "'%s' to fix the problem. The value '%u' is not "
+            "recognized for 'matrixid'", __func__, PACKAGE_BUGREPORT,
+            matrixid);
+    }
+
+  return nconst;
+}
+
+
+
+
+
 static void
-fit_1d_polynomial_prepare(gal_data_t *xin,  gal_data_t *yin,
-                          gal_data_t *ywht, int nconst,
-                          gsl_matrix **x,   gsl_vector **c,
-                          gsl_matrix **cov, gsl_vector *y,
-                          gsl_vector *w)
+fit_polynomial_1d_matrix_fill(gal_data_t *xin, int nconst, gsl_matrix **x)
 {
   size_t i, j;
   double *xo, *xi;
 
-  /* Use GSL's own matrix allocation functions for the structures that need
-     allocation and we can't use the same allocated space of the inputs. */
-  *c   = gsl_vector_alloc(nconst);
-  *cov = gsl_matrix_alloc(nconst, nconst);
-  *x   = gsl_matrix_alloc(xin->size, nconst);
-
   /* Fill in the X matrix. */
   xi=xin->array;
   xo=(*x)->data;
@@ -454,7 +522,7 @@ fit_1d_polynomial_prepare(gal_data_t *xin,  gal_data_t *yin,
          a value of 1.0. */
       xo[ i*nconst ] = 1.0f;
 
-      /* Column i is the multiplication of column i-1 with the input
+      /* Column j is the multiplication of column j-1 with the input
          horizontal value. This will make it a polynomial. */
       for(j=1;j<nconst;++j)
         xo[ i*nconst + j ] = xo[ i*nconst + j-1 ] * xi[i];
@@ -468,7 +536,119 @@ fit_1d_polynomial_prepare(gal_data_t *xin,  gal_data_t *yin,
       printf("%.3f ", xo[ checki*maxpower + j ]);
     printf("\n");
     exit(0);
-  } */
+  } //*/
+}
+
+
+
+
+
+static void
+fit_polynomial_2d_matrix_fill(gal_data_t *xin, int nconst,
+                              gsl_matrix **x, size_t maxpower, uint8_t tpv)
+{
+  size_t i, j, k, deg;
+  double r=NAN, r_pow=NAN;
+  gal_data_t *xin1=xin, *xin2=xin->next;
+  double *xo, *xi1, *xi2, *xi1_pow, *xi2_pow;
+
+  /* Allocate the necessary arrays. */
+  xi1_pow=gal_pointer_allocate(GAL_TYPE_FLOAT64, maxpower+1, 1,
+                               __func__, "xi1_pow");
+  xi2_pow=gal_pointer_allocate(GAL_TYPE_FLOAT64, maxpower+1, 1,
+                               __func__, "xi2_pow");
+
+  /* Fill in the matrix. */
+  xo=(*x)->data;
+  xi1=xin1->array;
+  xi2=xin2->array;
+  for(i=0;i<xin1->size;++i)
+    {
+      /* Restart from first column */
+      k=0;
+
+      /* Compute the radius */
+      if(tpv) r_pow=r=sqrt( xi1[i]*xi1[i] + xi2[i]*xi2[i] );
+
+      /* Column k is a combination of powers of the input values.
+         This will make it a polynomial. */
+      for(deg=0; deg<=maxpower; deg++)
+        {
+          /* Store the precomputed powers of xi1 and xi2 */
+          xi1_pow[deg] = deg>0 ? xi1[i]*xi1_pow[deg-1] : 1.0;
+          xi2_pow[deg] = deg>0 ? xi2[i]*xi2_pow[deg-1] : 1.0;
+
+          /* Use the previously computed powers to fill the matrix */
+          for(j=deg+1; j-->0;)
+            {
+              /* For a check on the powers of the dimensions.
+              if(i==0) printf("%s: %zu, %zu\n", __func__, j, deg-j);
+              //*/
+
+              xo[ i*nconst + k++ ] = xi1_pow[j] * xi2_pow[deg-j];
+            }
+
+          /* If a tpv polynomial is requested, add odd powers of
+             the radial term. See
+             https://fits.gsfc.nasa.gov/registry/tpvwcs/tpv.html */
+          if(tpv && deg%2)
+            {
+              xo[ i*nconst + k++ ] = r_pow;
+              r_pow*=(r*r);
+            }
+        }
+    }
+
+  /* For a check.
+  {
+    size_t checki=2;
+    printf("Row %zu: ", checki);
+    for(j=0;j<nconst;++j)
+      printf("%.3f ", xo[ checki*nconst + j ]);
+    printf("\n");
+    exit(0);
+  } //*/
+
+  /* Clean up. */
+  free(xi1_pow);
+  free(xi2_pow);
+}
+
+
+
+
+
+static void
+fit_polynomial_prepare(gal_data_t *xin,  gal_data_t *yin,
+                       gal_data_t *ywht, int nconst,
+                       gsl_matrix **x,   gsl_vector **c,
+                       gsl_matrix **cov, gsl_vector *y,
+                       gsl_vector *w,    size_t maxpower,
+                       uint8_t matrixid)
+{
+  /* Use GSL's own matrix allocation functions for the structures that need
+     allocation and we can't use the same allocated space of the inputs. */
+  *c   = gsl_vector_alloc(nconst);
+  *cov = gsl_matrix_alloc(nconst, nconst);
+  *x   = gsl_matrix_alloc(xin->size, nconst);
+
+  /* Fill the design matrix */
+  switch(matrixid)
+    {
+    case GAL_FIT_MATRIX_POLYNOMIAL_1D:
+      fit_polynomial_1d_matrix_fill(xin, nconst, x);
+      break;
+    case GAL_FIT_MATRIX_POLYNOMIAL_2D:
+      fit_polynomial_2d_matrix_fill(xin, nconst, x, maxpower, 0);
+      break;
+    case GAL_FIT_MATRIX_POLYNOMIAL_2D_TPV:
+      fit_polynomial_2d_matrix_fill(xin, nconst, x, maxpower, 1);
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to "
+            "fix the problem. '%d' isn't recognized as a design matrix ID",
+            __func__, PACKAGE_BUGREPORT, matrixid);
+    }
 
   /* Set the pointers of the 'y' and 'w' GSL vectors. */
   y->data=yin->array;
@@ -479,22 +659,107 @@ fit_1d_polynomial_prepare(gal_data_t *xin,  gal_data_t *yin,
 
 
 
-gal_data_t *
-gal_fit_1d_polynomial_base(gal_data_t *xin, gal_data_t *yin,
-                           gal_data_t *ywht, size_t maxpower,
-                           uint8_t robustid, double *redchisq)
+static void
+fit_polynomial_sanity_check(gal_data_t *xin, gal_data_t *yin,
+                            gal_data_t *ywht, uint8_t matrixid,
+                            gal_data_t **xdata, gal_data_t **ydata,
+                            gal_data_t **wdata, uint8_t robustid,
+                            double tikhonovlambda)
+{
+  /* Check the dimensionality. */
+  if(xin->next)
+    {
+      if(xin->next->next)
+        error(EXIT_FAILURE, 0, "%s: only 1d and 2d polynomials are "
+              "currently supported, but more than 2 columns have "
+              "been given as the first independenat variable ('x'). "
+              "This might happen if the fitting columns directly "
+              "came from a reading of a table and the columns were "
+              "not separated ('xin->next' or xin->next->next were "
+              "are not set to NULL before calling 'gal_fit_polynomial')",
+              __func__);
+      else if(xin->next==yin)
+        error(EXIT_FAILURE, 0, "%s: the second independent variable "
+              "points to the measurement variable. Set 'x->next=NULL' "
+              "before calling 'gal_fit_polynomial'", __func__);
+      else if(matrixid < GAL_FIT_MATRIX_NUMBER_1D)
+        error(EXIT_FAILURE, 0, "%s: 'matrixid=%s' describes a 1d design "
+              "matrix, but 'xin' is a list of more than one datasets",
+              __func__, fit_name_matrix_from_id(matrixid));
+    }
+  else if(matrixid > GAL_FIT_MATRIX_NUMBER_1D)
+    error(EXIT_FAILURE, 0, "%s: 'matrixid=%s' describes a 2d design "
+          "matrix, but 'xin' is a list of just one dataset",
+          __func__, fit_name_matrix_from_id(matrixid));
+
+  /* Tikhonov regularized regression, as discussed in GSL's manual:
+     https://www.gnu.org/s/gsl/doc/html/lls.html#regularized-regression */
+  if(!isnan(tikhonovlambda) && ywht)
+    error(EXIT_FAILURE, 0, "%s: tikhonov regularized fitting with "
+          "weights is still not implemented. Please use "
+          "'gal_fit_polynomial()' or set 'ywht=NULL'", __func__);
+
+  /* Only one type of fitting is acceptable. */
+  if(!isnan(tikhonovlambda) && robustid!=GAL_FIT_ROBUST_INVALID)
+    error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
+          "'%s' to fix the problem. the 'robustid' value '%d' "
+          "should be null when 'gal_fit_polynomial_tikhonov' "
+          "is invoked, i.e. when 'lambda' is set (lambda=%lf)",
+          __func__, PACKAGE_BUGREPORT, robustid, tikhonovlambda);
+
+
+  /* Make sure the types and lengths of each column are correct. */
+  *xdata =        fit_sanity_check_col(xin,  xin, __func__);
+  *ydata =        fit_sanity_check_col(yin,  xin, __func__);
+  *wdata = ywht ? fit_sanity_check_col(ywht, xin, __func__) : NULL;
+  (*xdata)->next = ( xin->next
+                   ? fit_sanity_check_col(xin->next, xin, __func__)
+                   : NULL );
+}
+
+
+
+
+
+static double
+fit_polynomial_base_robust(gsl_matrix *x, gsl_vector *y, gsl_vector *c,
+                           gsl_matrix *cov,
+                           const gsl_multifit_robust_type *rtype)
+{
+  double out=NAN;
+  gsl_multifit_robust_workspace *work_r;
+
+  /* Initialize the worker and do the fit (depending on if a weight
+     image was provided). */
+  work_r=gsl_multifit_robust_alloc(rtype, x->size1, x->size2);
+  gsl_multifit_robust(x, y, c, cov, work_r);
+
+  /* Get the residual sum of squares, free the worker and return. */
+  out=gsl_multifit_robust_statistics(work_r).sse;
+  gsl_multifit_robust_free(work_r);
+  return out;
+}
+
+
+
+
+
+static gal_data_t *
+fit_polynomial_base(gal_data_t *xin, gal_data_t *yin,
+                    gal_data_t *ywht, size_t maxpower,
+                    uint8_t robustid, double *redchisq,
+                    uint8_t matrixid, double tikhonovlambda)
 {
   /* Low-level variable. */
-  size_t nconst=maxpower+1;
+  size_t nconst = fit_polynomial_nconst(maxpower, matrixid);
 
   /* Other variables */
   gsl_vector *c=NULL;
+  double rnorm_t, snorm_t;
   double chisq=NAN, sse=NAN;
   gsl_matrix *x=NULL, *cov=NULL;
   size_t covsize[2]={nconst, nconst};
   gsl_multifit_linear_workspace *work_n;
-  gsl_multifit_robust_workspace *work_r;
-  const gsl_multifit_robust_type *rtype=NULL;
   gal_data_t *xdata, *ydata, *wdata, *tmp, *out=NULL;
 
   /* For the 'y' and 'w' GSL vectors, we don't actually need to allocate
@@ -505,54 +770,65 @@ gal_fit_1d_polynomial_base(gal_data_t *xin, gal_data_t *yin,
   gsl_vector wvec={yin->size, 1, NULL, NULL, 0}; /* 'ywht' may be NULL!  */
   gsl_vector *y=&yvec, *w=&wvec; /* These have to be after the two above.*/
 
-  /* Basic sanity checks. */
-  xdata =        fit_1d_sanity_check(xin,  xin, __func__);
-  ydata =        fit_1d_sanity_check(yin,  xin, __func__);
-  wdata = ywht ? fit_1d_sanity_check(ywht, xin, __func__) : NULL;
+  /* Basic check of the inputs. */
+  if(xin==NULL || xin->size==0 || yin==NULL || yin->size==0)
+    error(EXIT_FAILURE, 0, "%s: the inputs are either NULL or "
+          "do not contain any data (have a length of zero)", __func__);
 
-  /* Fill all the GSL structures. */
-  fit_1d_polynomial_prepare(xdata, ydata, wdata, nconst,
-                            &x, &c, &cov, y, w);
+  /* Fill all the GSL structures after a sanity check of th einput
+     columns. */
+  fit_polynomial_sanity_check(xin, yin, ywht, matrixid, &xdata,
+                              &ydata, &wdata, robustid, tikhonovlambda);
+  fit_polynomial_prepare(xdata, ydata, wdata, nconst,
+                         &x, &c, &cov, y, w, maxpower, matrixid);
 
-  /* Do the fit (depending on if it is robust or not. */
-  if(robustid==GAL_FIT_ROBUST_INVALID)
+  /* Do the fit depending on the input arguments. */
+  switch(robustid)
     {
-      work_n = gsl_multifit_linear_alloc(xin->size, nconst);
-      if(ywht) gsl_multifit_wlinear(x, w, y, c, cov, &chisq, work_n);
-      else     gsl_multifit_linear( x,    y, c, cov, &sse,   work_n);
-      gsl_multifit_linear_free(work_n);
-    }
-  else
-    {
-      /* Select the robust function type. */
-      switch(robustid)
+    case GAL_FIT_ROBUST_BISQUARE:
+      sse=fit_polynomial_base_robust(x, y, c, cov,
+                                     gsl_multifit_robust_bisquare);
+      break;
+    case GAL_FIT_ROBUST_CAUCHY:
+      sse=fit_polynomial_base_robust(x, y, c, cov,
+                                     gsl_multifit_robust_cauchy);
+      break;
+    case GAL_FIT_ROBUST_FAIR:
+      sse=fit_polynomial_base_robust(x, y, c, cov,
+                                     gsl_multifit_robust_fair);
+      break;
+    case GAL_FIT_ROBUST_HUBER:
+      sse=fit_polynomial_base_robust(x, y, c, cov,
+                                     gsl_multifit_robust_huber);
+      break;
+    case GAL_FIT_ROBUST_OLS:
+      sse=fit_polynomial_base_robust(x, y, c, cov,
+                                     gsl_multifit_robust_ols);
+      break;
+    case GAL_FIT_ROBUST_WELSCH:
+      sse=fit_polynomial_base_robust(x, y, c, cov,
+                                     gsl_multifit_robust_welsch);
+      break;
+    case GAL_FIT_ROBUST_INVALID:
+      if( !isnan(tikhonovlambda) ) /* Tikonov regularization. */
         {
-        case GAL_FIT_ROBUST_BISQUARE: rtype=gsl_multifit_robust_bisquare;
-          break;
-        case GAL_FIT_ROBUST_CAUCHY:   rtype=gsl_multifit_robust_cauchy;
-          break;
-        case GAL_FIT_ROBUST_FAIR:     rtype=gsl_multifit_robust_fair;
-          break;
-        case GAL_FIT_ROBUST_HUBER:    rtype=gsl_multifit_robust_huber;
-          break;
-        case GAL_FIT_ROBUST_OLS:      rtype=gsl_multifit_robust_ols;
-          break;
-        case GAL_FIT_ROBUST_WELSCH:   rtype=gsl_multifit_robust_welsch;
-          break;
-        default:
-          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
-                "'%s' to fix the problem. the 'robustid' value '%d' "
-                "isn't recognize", __func__, PACKAGE_BUGREPORT, robustid);
+          work_n = gsl_multifit_linear_alloc(xin->size, nconst);
+          gsl_multifit_linear_svd(x, work_n);
+          gsl_multifit_linear_solve(tikhonovlambda, x, y, c,
+                                    &rnorm_t, &snorm_t, work_n);
         }
-
-      /* Initialize the worker and do the fit (depending on if a weight
-         image was provided). */
-      work_r=gsl_multifit_robust_alloc(rtype, x->size1, x->size2);
-      gsl_multifit_robust(x, y, c, cov, work_r);
-
-      /* Get the residual sum of squares and free the worker. */
-      sse=gsl_multifit_robust_statistics(work_r).sse;
-      gsl_multifit_robust_free(work_r);
+      else /* Linear fit. */
+        {
+          work_n = gsl_multifit_linear_alloc(xin->size, nconst);
+          if(ywht) gsl_multifit_wlinear(x, w, y, c, cov, &chisq, work_n);
+          else     gsl_multifit_linear( x,    y, c, cov, &sse,   work_n);
+          gsl_multifit_linear_free(work_n);
+        }
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to "
+            "fix the problem. the 'robustid' value '%d' isn't recognize",
+            __func__, PACKAGE_BUGREPORT, robustid);
     }
 
   /* For a check:
@@ -560,7 +836,7 @@ gal_fit_1d_polynomial_base(gal_data_t *xin, gal_data_t *yin,
     size_t i;
     double *ca=c->data;
     for(i=0;i<=nconst;++i) { printf("%f ", ca[i]); } printf("\n");
-  } */
+  } //*/
 
   /* Allocate the output dataset containing the fit results as first
      'gal_data_t'. */
@@ -582,12 +858,14 @@ gal_fit_1d_polynomial_base(gal_data_t *xin, gal_data_t *yin,
   *redchisq = (isnan(chisq) ? sse : chisq) / (xdata->size-nconst);
 
   /* Clean up and return. */
-  gsl_matrix_free(x);
-  gsl_vector_free(c);
-  gsl_matrix_free(cov);
+  if(xdata->next && xdata->next!=xin->next) /* Must be before 'xdata'.*/
+    { gal_data_free(xdata->next); xdata->next=NULL; }
+  if(ywht && wdata!=ywht) gal_data_free(wdata);
   if(xdata!=xin) gal_data_free(xdata);
   if(ydata!=yin) gal_data_free(ydata);
-  if(ywht && wdata!=ywht) gal_data_free(wdata);
+  gsl_matrix_free(cov);
+  gsl_matrix_free(x);
+  gsl_vector_free(c);
   return out;
 }
 
@@ -596,13 +874,13 @@ gal_fit_1d_polynomial_base(gal_data_t *xin, gal_data_t *yin,
 
 
 gal_data_t *
-gal_fit_1d_polynomial(gal_data_t *xin, gal_data_t *yin,
-                      gal_data_t *ywht, size_t maxpower,
-                      double *redchisq)
+gal_fit_polynomial(gal_data_t *xin, gal_data_t *yin,
+                   gal_data_t *ywht, size_t maxpower,
+                   double *redchisq, uint8_t matrixid)
 {
-  return gal_fit_1d_polynomial_base(xin, yin, ywht, maxpower,
-                                    GAL_FIT_ROBUST_INVALID,
-                                    redchisq);
+  return fit_polynomial_base(xin, yin, ywht, maxpower,
+                             GAL_FIT_ROBUST_INVALID,
+                             redchisq, matrixid, NAN);
 }
 
 
@@ -610,14 +888,28 @@ gal_fit_1d_polynomial(gal_data_t *xin, gal_data_t *yin,
 
 
 gal_data_t *
-gal_fit_1d_polynomial_robust(gal_data_t *xin, gal_data_t *yin,
-                             size_t maxpower, uint8_t robustid,
-                             double *redchisq)
+gal_fit_polynomial_robust(gal_data_t *xin, gal_data_t *yin,
+                          size_t maxpower, uint8_t robustid,
+                          double *redchisq, uint8_t matrixid)
 {
   /* Robust fitting doesn't use weights (the functions are effectively the
      weight). */
-  return gal_fit_1d_polynomial_base(xin, yin, NULL, maxpower,
-                                    robustid, redchisq);
+  return fit_polynomial_base(xin, yin, NULL, maxpower, robustid,
+                             redchisq, matrixid, NAN);
+}
+
+
+
+
+
+gal_data_t *
+gal_fit_polynomial_tikhonov(gal_data_t *xin, gal_data_t *yin,
+                            size_t maxpower, double *redchisq,
+                            uint8_t matrixid, double tikhonovlambda)
+{
+  return fit_polynomial_base(xin, yin, NULL, maxpower,
+                             GAL_FIT_ROBUST_INVALID,
+                             redchisq, matrixid, tikhonovlambda);
 }
 
 
@@ -626,7 +918,7 @@ gal_fit_1d_polynomial_robust(gal_data_t *xin, gal_data_t *yin,
 
 /* Estimate values from a polynomial fit. */
 gal_data_t *
-gal_fit_1d_polynomial_estimate(gal_data_t *fit, gal_data_t *xin)
+gal_fit_polynomial_estimate_1d(gal_data_t *fit, gal_data_t *xin)
 {
   size_t i, j;
   size_t nconst=fit->size;
@@ -665,3 +957,13 @@ gal_fit_1d_polynomial_estimate(gal_data_t *fit, gal_data_t *xin)
   free(xvec.data);
   return out;
 }
+
+
+
+
+
+gal_data_t *
+gal_fit_polynomial_estimate_2d(gal_data_t *fit, gal_data_t *xin)
+{
+  printf("%s: GOOD\n", __func__); exit(0);
+}
