@@ -6,7 +6,7 @@ Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
      Pedram Ashofteh Ardakani <pedramardakani@pm.me>
-Copyright (C) 2015-2024 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -110,6 +110,11 @@ gal_checkset_noprefix_isequal(char *string, char *prefix,
 int
 gal_checkset_exec(char *executable_abs_address, gal_list_str_t *args);
 
+/**************************************************************/
+/**********         Common dataset formats         ************/
+/**************************************************************/
+gal_data_t *
+gal_checkset_labels_to_int32(gal_data_t *input, const char *funcname);
 
 /**************************************************************/
 /********** Set file names and check if they exist ************/
