@@ -4,7 +4,7 @@ Function to parse options and configuration file values.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2024 Free Software Foundation, Inc.
+Copyright (C) 2017-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -347,6 +347,11 @@ gal_options_parse_name_and_strings(struct argp_option *option, char *arg,
                                    char *filename, size_t lineno,
                                    void *junk);
 
+void *
+gal_options_parse_name_and_strings_append(struct argp_option *option,
+                                          char *arg, char *filename,
+                                          size_t lineno, void *junk);
+
 void *
 gal_options_parse_name_and_float64s(struct argp_option *option, char *arg,
                                     char *filename, size_t lineno,
