@@ -6,7 +6,7 @@ Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
      Faezeh Bidjarchian <fbidjarchian@gmail.com>
-Copyright (C) 2017-2024 Free Software Foundation, Inc.
+Copyright (C) 2017-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -149,6 +149,8 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_DEGREE_TO_DEC,/* declination to decimal. */
   GAL_ARITHMETIC_OP_COUNTS_TO_MAG,/* Counts to magnitude. */
   GAL_ARITHMETIC_OP_MAG_TO_COUNTS,/* Magnitude to counts. */
+  GAL_ARITHMETIC_MAG_TO_LUMINOSITY, /* Apparent magnitude to luminosity. */
+  GAL_ARITHMETIC_LUMINOSITY_TO_MAG, /* Luminosity to apparent magnitude. */
   GAL_ARITHMETIC_OP_MAG_TO_SB,    /* Magnitude to Surface Brightness. */
   GAL_ARITHMETIC_OP_SB_TO_MAG,    /* Surface Brightness to Magnitude. */
   GAL_ARITHMETIC_OP_COUNTS_TO_SB, /* Counts to Surface Brightness. */
@@ -157,8 +159,12 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_JY_TO_COUNTS, /* Janskys to Counts (AB zeropoint). */
   GAL_ARITHMETIC_OP_MAG_TO_JY,    /* Magnitude to Janskys. */
   GAL_ARITHMETIC_OP_JY_TO_MAG,    /* Janskys to Magnitude. */
+  GAL_ARITHMETIC_OP_JY_TO_WAVELENGTH_FLUX_DENSITY, /* Janskys to f_\lambda. */
+  GAL_ARITHMETIC_OP_WAVELENGTH_FLUX_DENSITY_TO_JY, /* f_\lambda to Janskys. */
+  GAL_ARITHMETIC_OP_ZEROPOINT_CHANGE, /* Change the zero point. */
   GAL_ARITHMETIC_OP_COUNTS_TO_NANOMAGGY,/* Counts to SDSS nanomaggies. */
   GAL_ARITHMETIC_OP_NANOMAGGY_TO_COUNTS,/* SDSS nanomaggies to counts. */
+  GAL_ARITHMETIC_OP_SBLIM_DIFF,   /* Calculate surface bright. on new image. */
   GAL_ARITHMETIC_OP_AU_TO_PC,     /* Astronomical units (AU) to Parsecs (PC). */
   GAL_ARITHMETIC_OP_PC_TO_AU,     /* Parsecs (PC) to Astronomical units (AU). */
   GAL_ARITHMETIC_OP_LY_TO_PC,     /* Astronomical units (AU) to Parsecs (PC). */
@@ -176,6 +182,10 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_UNIQUE,       /* Only return unique elements.          */
   GAL_ARITHMETIC_OP_NOBLANK,      /* Only keep non-blank elements.         */
 
+  GAL_ARITHMETIC_OP_LABEL_AREA,   /* Fill labeles with area of its values. */
+  GAL_ARITHMETIC_OP_LABEL_MINIMUM,/* Fill labeles with min of its values.  */
+  GAL_ARITHMETIC_OP_LABEL_MAXIMUM,/* Fill labeles with max of its values.  */
+
   GAL_ARITHMETIC_OP_MIN,          /* Minimum per pixel of multiple arrays. */
   GAL_ARITHMETIC_OP_MAX,          /* Maximum per pixel of multiple arrays. */
   GAL_ARITHMETIC_OP_NUMBER,       /* Non-blank number of pixels in arrays. */
@@ -185,29 +195,21 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_MAD,          /* MAD per pixel of multiple arrays.     */
   GAL_ARITHMETIC_OP_MEDIAN,       /* Median per pixel of multiple arrays.  */
   GAL_ARITHMETIC_OP_QUANTILE,     /* Quantile per pixel of multiple arrays.*/
-  GAL_ARITHMETIC_OP_SIGCLIP_NUMBER,/* Sigma-clipped number of mult. arrays.*/
   GAL_ARITHMETIC_OP_SIGCLIP_MEAN, /* Sigma-clipped mean of multiple arrays.*/
   GAL_ARITHMETIC_OP_SIGCLIP_MEDIAN,/* Sigma-clipped median of mult. arrays.*/
   GAL_ARITHMETIC_OP_SIGCLIP_STD,  /* Sigma-clipped STD of multiple arrays. */
   GAL_ARITHMETIC_OP_SIGCLIP_MAD,  /* Sigma-clipped STD of multiple arrays. */
-  GAL_ARITHMETIC_OP_SIGCLIP_FILL_NUMBER,  /* MAD-clipped num. of arrays.   */
-  GAL_ARITHMETIC_OP_SIGCLIP_FILL_MEAN,   /* MAD-clipped mean of arrays.    */
-  GAL_ARITHMETIC_OP_SIGCLIP_FILL_MEDIAN,  /* MAD-clipped median of arrays. */
-  GAL_ARITHMETIC_OP_SIGCLIP_FILL_STD,     /* MAD-clipped STD of arrays.    */
-  GAL_ARITHMETIC_OP_SIGCLIP_FILL_MAD,     /* MAD-clipped STD of arrays.    */
-  GAL_ARITHMETIC_OP_MADCLIP_NUMBER,/* MAD-clipped number of mult. arrays.  */
+  GAL_ARITHMETIC_OP_SIGCLIP_MASKFILLED, /* Mask clipped elem. of each in.  */
+  GAL_ARITHMETIC_OP_SIGCLIP_ALL,   /* Sigma-clip all statistics.           */
   GAL_ARITHMETIC_OP_MADCLIP_MEAN,  /* MAD-clipped mean of multiple arrays. */
   GAL_ARITHMETIC_OP_MADCLIP_MEDIAN,/* MAD-clipped median of mult. arrays.  */
   GAL_ARITHMETIC_OP_MADCLIP_STD,   /* MAD-clipped STD of multiple arrays.  */
   GAL_ARITHMETIC_OP_MADCLIP_MAD,   /* MAD-clipped STD of multiple arrays.  */
-  GAL_ARITHMETIC_OP_MADCLIP_FILL_NUMBER,/* MAD-clipped num. of arrays.   */
-  GAL_ARITHMETIC_OP_MADCLIP_FILL_MEAN, /* MAD-clipped mean of arrays.    */
-  GAL_ARITHMETIC_OP_MADCLIP_FILL_MEDIAN,/* MAD-clipped median of arrays. */
-  GAL_ARITHMETIC_OP_MADCLIP_FILL_STD,   /* MAD-clipped STD of arrays.    */
-  GAL_ARITHMETIC_OP_MADCLIP_FILL_MAD,   /* MAD-clipped STD of arrays.    */
+  GAL_ARITHMETIC_OP_MADCLIP_MASKFILLED, /* Mask clipped elem. of each in.  */
+  GAL_ARITHMETIC_OP_MADCLIP_ALL,   /* Sigma-clip all statistics.           */
 
   GAL_ARITHMETIC_OP_MKNOISE_SIGMA,/* Fixed-sigma noise to every element.   */
-  GAL_ARITHMETIC_OP_MKNOISE_SIGMA_FROM_MEAN, /* Sigma comes from background. */
+  GAL_ARITHMETIC_OP_MKNOISE_SIGMA_FROM_MEAN, /* Sigma calculated from mean.*/
   GAL_ARITHMETIC_OP_MKNOISE_POISSON,/* Poission noise on every element.    */
   GAL_ARITHMETIC_OP_MKNOISE_UNIFORM,/* Uniform noise on every element.     */
   GAL_ARITHMETIC_OP_RANDOM_FROM_HIST,/* Randoms from a histogram (uniform).*/
@@ -281,6 +283,9 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_SUPERGALACTIC_TO_ECJ2000,
   GAL_ARITHMETIC_OP_SUPERGALACTIC_TO_GALACTIC,
 
+  /* Memory operations. */
+  GAL_ARITHMETIC_OP_FREE,
+
   /* Counter for number of operators. */
   GAL_ARITHMETIC_OP_LAST_CODE,    /* Last code of the library operands.    */
 };
