@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2024 Free Software Foundation, Inc.
+Copyright (C) 2017-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -141,7 +141,7 @@ void
 gal_blank_flag_apply(gal_data_t *input, gal_data_t *flag);
 
 void
-gal_blank_remove(gal_data_t *data);
+gal_blank_remove(gal_data_t *data, int free_if_all_blank);
 
 void
 gal_blank_remove_realloc(gal_data_t *input);
