@@ -1,12 +1,13 @@
 /*********************************************************************
-kdtree -- Create kd-tree and nearest neighbour searches.
+kdtree -- Create kd-tree and nearest neighbor searches.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
      Sachin Kumar Singh <sachinkumarsingh092@gmail.com>
 Contributing author(s):
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
-Copyright (C) 2020-2024 Free Software Foundation, Inc.
+     Barış Güngör <barisgungor1010@gmail.com>
+Copyright (C) 2020-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -48,12 +49,19 @@ __BEGIN_C_DECLS  /* From C++ preparations */
 
 
 
+
+
 gal_data_t *
 gal_kdtree_create(gal_data_t *coords_raw, size_t *root);
 
 size_t
-gal_kdtree_nearest_neighbour(gal_data_t *coords_raw, gal_data_t *kdtree,
-                             size_t root, double *point, double *least_dist);
+gal_kdtree_nearest_neighbor(gal_data_t *coords_raw, gal_data_t *kdtree,
+                            size_t root, double *point,
+                            double *least_dist, uint8_t nosamenode);
+
+gal_list_sizetf64_t *
+gal_kdtree_range(gal_data_t *coords_raw, gal_data_t *kdtree,
+                 size_t root, double *point, double radius);
 
 
 
