@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2024 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -242,6 +242,55 @@ gal_list_f64_free(gal_list_f64_t *list);
 
 
 
+/****************************************************************
+ *****************     size_t & double       ********************
+ ****************************************************************/
+typedef struct gal_list_sizetf64_t
+{
+  size_t i;
+  double v;
+  struct gal_list_sizetf64_t *next;
+} gal_list_sizetf64_t;
+
+void
+gal_list_sizetf64_add(gal_list_sizetf64_t **list, size_t i, double v);
+
+void
+gal_list_sizetf64_pop(gal_list_sizetf64_t **list, size_t *i, double *v);
+
+void
+gal_list_sizetf64_free(gal_list_sizetf64_t **list);
+
+
+
+
+
+/****************************************************************
+ *****************     size_t & double       ********************
+ ****************************************************************/
+typedef struct gal_list_sizetsizetf64_t
+{
+  size_t i;
+  size_t j;
+  double v;
+  struct gal_list_sizetsizetf64_t *next;
+} gal_list_sizetsizetf64_t;
+
+void
+gal_list_sizetsizetf64_add(gal_list_sizetsizetf64_t **list, size_t i,
+                           size_t j, double v);
+
+void
+gal_list_sizetsizetf64_pop(gal_list_sizetsizetf64_t **list, size_t *i,
+                           size_t *j, double *v);
+
+void
+gal_list_sizetsizetf64_free(gal_list_sizetsizetf64_t **list);
+
+
+
+
+
 /****************************************************************
  *****************          void *           ********************
  ****************************************************************/
