@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2023 Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -105,10 +105,13 @@ gal_txt_image_read(char *filename, gal_list_str_t *lines, size_t minmapsize,
 gal_list_str_t *
 gal_txt_stdin_read(long timeout_microsec);
 
+gal_list_str_t *
+gal_txt_read_to_list(char *filename);
+
 void
-gal_txt_write(gal_data_t *input, struct gal_fits_list_key_t **keylist,
+gal_txt_write(gal_data_t *input, struct gal_fits_list_key_t *keylist,
               gal_list_str_t *comment, char *filename,
-              uint8_t colinfoinstdout, int tab0_img1);
+              uint8_t colinfoinstdout, int tab0_img1, int freekeys);
 
 
 
