@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -26,8 +26,6 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <limits.h>
 #include <stdint.h>
 
-#include <gsl/gsl_complex.h>
-
 /* When we are within Gnuastro's building process, 'IN_GNUASTRO_BUILD' is
    defined. In the build process, installation information (in particular
    'GAL_CONFIG_SIZEOF_SIZE_T' that we need below) is kept in
@@ -71,15 +69,21 @@ enum gal_types
 {
   GAL_TYPE_INVALID,         /* Invalid (=0 by C standard).             */
 
-  GAL_TYPE_BIT,             /* 1 bit                                   */
-  GAL_TYPE_UINT8,           /* 8-bit  unsigned integer.                */
+  /* Integer types: the ordering here is used to find the output type of
+     binary operations in 'gal_type_out'. Therefore, as in the automatic C
+     type conversion, the unsigned types should be placed after (so their
+     type is preferred over a similar-width integer that is signed). */
   GAL_TYPE_INT8,            /* 8-bit  signed   integer.                */
-  GAL_TYPE_UINT16,          /* 16-bit unsigned integer.                */
+  GAL_TYPE_UINT8,           /* 8-bit  unsigned integer.                */
   GAL_TYPE_INT16,           /* 16-bit signed   integer.                */
-  GAL_TYPE_UINT32,          /* 32-bit unsigned integer.                */
+  GAL_TYPE_UINT16,          /* 16-bit unsigned integer.                */
   GAL_TYPE_INT32,           /* 32-bit signed   integer.                */
-  GAL_TYPE_UINT64,          /* 64-bit unsigned integer.                */
+  GAL_TYPE_UINT32,          /* 32-bit unsigned integer.                */
   GAL_TYPE_INT64,           /* 64-bit signed   integer.                */
+  GAL_TYPE_UINT64,          /* 64-bit unsigned integer.                */
+
+  /* Other types. */
+  GAL_TYPE_BIT,             /* 1 bit                                   */
   GAL_TYPE_FLOAT32,         /* 32-bit single precision floating point. */
   GAL_TYPE_FLOAT64,         /* 64-bit double precision floating point. */
   GAL_TYPE_COMPLEX32,       /* Complex 32-bit floating point.          */
