@@ -6,7 +6,7 @@ Original author:
      Kartik Ohri <kartikohri13@gmail.com>
 Contributing author(s):
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
-Copyright (C) 2020-2024 Free Software Foundation, Inc.
+Copyright (C) 2020-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -79,6 +79,14 @@ gal_units_counts_to_mag(double counts, double zeropoint);
 double
 gal_units_mag_to_counts(double mag, double zeropoint);
 
+double
+gal_units_mag_to_luminosity(double mag, double mag_absolute_sun,
+                            double distance_modulus);
+
+double
+gal_units_luminosity_to_mag(double luminosity, double mag_absolute_sun,
+                            double distance_modulus);
+
 double
 gal_units_mag_to_sb(double mag, double area_arcsec2);
 
@@ -115,9 +123,15 @@ gal_units_jy_to_mag(double jy);
 double
 gal_units_jy_to_wavelength_flux_density(double jy, double wavelength);
 
+double
+gal_units_wavelength_flux_density_to_jy(double wfd, double angstrom);
+
 double
 gal_units_mag_to_jy(double mag);
 
+double
+gal_units_sblim_diff(double r_frac, double t_frac);
+
 double
 gal_units_au_to_pc(double au);
 
