@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2024 Free Software Foundation, Inc.
+Copyright (C) 2016-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -32,8 +32,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/git.h>
 #include <gnuastro/txt.h>
 #include <gnuastro/blank.h>
+#include <gnuastro/qsort.h>
 #include <gnuastro/table.h>
 #include <gnuastro/pointer.h>
+#include <gnuastro/permutation.h>
 
 #include <gnuastro-internal/timing.h>
 #include <gnuastro-internal/checkset.h>
@@ -624,7 +626,7 @@ gal_table_write(gal_data_t *cols, struct gal_fits_list_key_t *keylist,
 void
 gal_table_write_log(gal_data_t *logll, char *program_string,
                     time_t *rawtime, gal_list_str_t *comments,
-                    char *filename, int quiet)
+                    char *filename, int quiet, int format)
 {
   char *msg;
 
@@ -632,8 +634,8 @@ gal_table_write_log(gal_data_t *logll, char *program_string,
   gal_table_comments_add_intro(&comments, program_string, rawtime);
 
   /* Write the log file to disk. */
-  gal_table_write(logll, NULL, comments, GAL_TABLE_FORMAT_TXT,
-                  filename, "LOG", 0, 0);
+  gal_table_write(logll, NULL, comments, format, filename, "LOG",
+                  0, 0);
 
   /* In verbose mode, print the information. */
   if(!quiet)
@@ -778,3 +780,133 @@ gal_table_cols_to_vector(gal_data_t *list)
   free(name);
   return out;
 }
+
+
+
+
+
+static void
+table_apply_permutation(gal_data_t *table, size_t *permutation,
+                        size_t permsize, int inverse)
+{
+  gal_data_t *tmp;
+
+  for(tmp=table;tmp!=NULL;tmp=tmp->next)
+    {
+      /* Apply the permutation. */
+      if(inverse)
+        {
+          if(tmp->ndim==1)
+            gal_permutation_apply_inverse(tmp, permutation);
+          else
+            error(EXIT_FAILURE, 0, "%s: inverse permutation on "
+                  "vector columns is not yet supported. Please "
+                  "get in touch with us at '%s' to add this "
+                  "feature", __func__, PACKAGE_BUGREPORT);
+        }
+      else
+        {
+          if(tmp->ndim==1) gal_permutation_apply(tmp, permutation);
+          else    gal_permutation_apply_onlydim0(tmp, permutation);
+        }
+
+      /* Correct the size. */
+      tmp->dsize[0]=permsize;
+      tmp->size = tmp->dsize[0] * (tmp->ndim==1 ? 1 : tmp->dsize[1]);
+    }
+}
+
+
+
+
+
+void
+gal_table_sort(gal_data_t *table, gal_data_t *sortcol, uint8_t descending)
+{
+  gal_data_t *perm;
+  size_t c=0, *s, *sf, dsize0=table->dsize[0];
+  int (*qsortfn)(const void *, const void *)=NULL;
+
+  /* In case there are no columns to sort, skip this function. */
+  if(table->size==0 || table->array==NULL || table->dsize==NULL)
+    return;
+
+  /* Allocate the permutation array and fill it. Note that we need 'dsize0'
+     because the first column may be a vector column (which is 2D). */
+  perm=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, &dsize0, NULL, 0,
+                      table->minmapsize, table->quietmmap, NULL, NULL, NULL);
+  sf=(s=perm->array)+perm->size; do *s=c++; while(++s<sf);
+
+  /* For string columns, print a descriptive message. Note that some FITS
+     tables were found that do actually have numbers stored in string
+     types! */
+  if(sortcol->type==GAL_TYPE_STRING)
+    error(EXIT_FAILURE, 0, "sort column has a string type, but it can "
+          "(currently) only work on numbers.\n\n"
+          "TIP: if you know the columns contents are all numbers that are "
+          "just stored as strings, you can use this program to save the "
+          "table as a text file, modify the column meta-data (for example "
+          "to type 'i32' or 'f32' instead of 'strN'), then use this "
+          "program again to save it as a FITS table.\n\n"
+          "For more on column metadata in plain text format, please run "
+          "the following command (or see the 'Gnuastro text table format "
+          "section of the book/manual):\n\n"
+          "    $ info gnuastro \"gnuastro text table format\"");
+
+  /* Set the proper qsort function. */
+  if(descending)
+    switch(sortcol->type)
+      {
+      case GAL_TYPE_UINT8:   qsortfn=gal_qsort_index_single_uint8_d;   break;
+      case GAL_TYPE_INT8:    qsortfn=gal_qsort_index_single_int8_d;    break;
+      case GAL_TYPE_UINT16:  qsortfn=gal_qsort_index_single_uint16_d;  break;
+      case GAL_TYPE_INT16:   qsortfn=gal_qsort_index_single_int16_d;   break;
+      case GAL_TYPE_UINT32:  qsortfn=gal_qsort_index_single_uint32_d;  break;
+      case GAL_TYPE_INT32:   qsortfn=gal_qsort_index_single_int32_d;   break;
+      case GAL_TYPE_UINT64:  qsortfn=gal_qsort_index_single_uint64_d;  break;
+      case GAL_TYPE_INT64:   qsortfn=gal_qsort_index_single_int64_d;   break;
+      case GAL_TYPE_FLOAT32: qsortfn=gal_qsort_index_single_float32_d; break;
+      case GAL_TYPE_FLOAT64: qsortfn=gal_qsort_index_single_float64_d; break;
+      default:
+        error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
+              "the problem. The code '%u' wasn't recognized as a data type",
+              __func__, PACKAGE_BUGREPORT, sortcol->type);
+      }
+  else
+    switch(sortcol->type)
+      {
+      case GAL_TYPE_UINT8:   qsortfn=gal_qsort_index_single_uint8_i;   break;
+      case GAL_TYPE_INT8:    qsortfn=gal_qsort_index_single_int8_i;    break;
+      case GAL_TYPE_UINT16:  qsortfn=gal_qsort_index_single_uint16_i;  break;
+      case GAL_TYPE_INT16:   qsortfn=gal_qsort_index_single_int16_i;   break;
+      case GAL_TYPE_UINT32:  qsortfn=gal_qsort_index_single_uint32_i;  break;
+      case GAL_TYPE_INT32:   qsortfn=gal_qsort_index_single_int32_i;   break;
+      case GAL_TYPE_UINT64:  qsortfn=gal_qsort_index_single_uint64_i;  break;
+      case GAL_TYPE_INT64:   qsortfn=gal_qsort_index_single_int64_i;   break;
+      case GAL_TYPE_FLOAT32: qsortfn=gal_qsort_index_single_float32_i; break;
+      case GAL_TYPE_FLOAT64: qsortfn=gal_qsort_index_single_float64_i; break;
+      default:
+        error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
+              "the problem. The code '%u' wasn't recognized as a data type",
+              __func__, PACKAGE_BUGREPORT, sortcol->type);
+      }
+
+  /* Sort the indexs from the values. */
+  gal_qsort_index_single=sortcol->array;
+  qsort(perm->array, perm->size, sizeof *s, qsortfn);
+
+  /* For a check (only on float32 type 'sortcol'):
+  {
+    float *f=sortcol->array;
+    sf=(s=perm->array)+perm->size;
+    do printf("%f\n", f[*s]); while(++s<sf);
+    exit(0);
+  }
+  */
+
+  /* Sort all the output columns with this permutation. */
+  table_apply_permutation(table, perm->array, perm->size, 0);
+
+  /* Clean up. */
+  gal_data_free(perm);
+}
