@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2023 Free Software Foundation, Inc.
+Copyright (C) 2017-2025 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -35,6 +35,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/pointer.h>
 #include <gnuastro/convolve.h>
 #include <gnuastro/dimension.h>
+#include <gnuastro/statistics.h>
 #include <gnuastro/interpolate.h>
 #include <gnuastro/permutation.h>
 
@@ -120,7 +121,7 @@ gal_tile_start_end_coord(gal_data_t *tile, size_t *start_end, int rel_block)
     }
 
   /* Add the dimensions of the tile to the starting coordinate. Note that
-     the ending coordinates are stored immediately after the start.*/
+     the ending coordinates are stored immediately after the start. */
   gal_dimension_add_coords(start_end, tile->dsize, end, ndim);
 }
 
@@ -146,13 +147,13 @@ gal_tile_start_end_ind_inclusive(gal_data_t *tile, gal_data_t *work,
 
   /* The starting index can be found from the distance of the 'tile->array'
      pointer and 'block->array' pointer. IMPORTANT: with the type of the
-     block array.  */
+     block array. */
   start_end_inc[0]=gal_pointer_num_between(block->array, tile->array,
                                            block->type);
 
 
   /* To find the end index, we need to know the coordinates of the starting
-     point in the allocated block.  */
+     point in the allocated block. */
   gal_dimension_index_to_coord(start_end_inc[0], ndim, block->dsize,
                               start_coord);
 
@@ -161,7 +162,7 @@ gal_tile_start_end_ind_inclusive(gal_data_t *tile, gal_data_t *work,
      dimension. To have less potential for bugs, we will remove that extra
      value, so we get the coordinates of the last pixel in the tile
      (inclusive). We will finally, increment that value by one to get to
-     the pixel immediately outside of the tile.*/
+     the pixel immediately outside of the tile. */
   e=end_coord;
   l=tile->dsize;
   sf=(s=start_coord)+ndim; do *e++ = *s + *l++ - 1; while(++s<sf);
@@ -234,13 +235,18 @@ gal_tile_series_from_minmax(gal_data_t *block, size_t *minmax, size_t number)
   size_t i, d, ind, size, width=2*ndim;
   gal_data_t *tiles=gal_data_array_calloc(number);
 
-  /* Fill the tile information.  */
+  /* Fill the tile information. */
   for(i=0;i<number;++i)
     {
       /* To make things more readable. */
       min = &minmax[ i * width        ];
       max = &minmax[ i * width + ndim ];
 
+      /* For a check.
+      printf("%s: [%zu,%zu], [%zu,%zu]\n", __func__, min[0], max[0],
+             min[1], max[1]);
+      */
+
       /* Tile types should be invalid (we shouldn't use tiles directly),
          also se the other simple values. */
       tiles[i].flag  = 0;
@@ -248,17 +254,28 @@ gal_tile_series_from_minmax(gal_data_t *block, size_t *minmax, size_t number)
       tiles[i].type  = GAL_TYPE_INVALID;
       tiles[i].next  = i==number-1 ? NULL : &tiles[i+1];
 
-      /* Set the size related constants. */
-      size = 1;
-      tiles[i].ndim  = ndim;
-      tiles[i].dsize = gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0,
-                                            __func__, "tiles[i].dsize");
-      for(d=0;d<ndim;++d) size *= tiles[i].dsize[d] = max[d] - min[d] + 1;
-      tiles[i].size  = size;
+      /* In case any of the values is blank, do not set the size or
+         pointers. */
+      if(    min[0]!=GAL_BLANK_SIZE_T
+          && max[0]!=GAL_BLANK_SIZE_T
+          && min[1]!=GAL_BLANK_SIZE_T
+          && max[1]!=GAL_BLANK_SIZE_T )
+        {
 
-      /* Tile's array pointer. */
-      ind=gal_dimension_coord_to_index(ndim, block->dsize, min);
-      tiles[i].array = gal_pointer_increment(block->array, ind, block->type);
+          /* Set the size related constants. */
+          size=1;
+          tiles[i].ndim=ndim;
+          tiles[i].dsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0,
+                                              __func__, "tiles[i].dsize");
+          for(d=0;d<ndim;++d)
+            size *= tiles[i].dsize[d] = max[d] - min[d] + 1;
+          tiles[i].size  = size;
+
+          /* Tile's array pointer. */
+          ind=gal_dimension_coord_to_index(ndim, block->dsize, min);
+          tiles[i].array = gal_pointer_increment(block->array, ind,
+                                                 block->type);
+        }
     }
 
   /* For a check (put all the objects in an extension of a test file).
@@ -280,6 +297,287 @@ gal_tile_series_from_minmax(gal_data_t *block, size_t *minmax, size_t number)
 
 
 
+static size_t
+gal_tile_per_label_number(gal_data_t *labels)
+{
+  size_t out;
+  gal_data_t *tmp;
+
+  /* Get the maximum of the labels.*/
+  tmp=gal_statistics_maximum(labels);
+  out=*((int32_t *)(tmp->array)); /*numobjects is int32_t.*/
+
+  /* In case the input is all blank, the maximum will be blank, so in
+     effect, there we no objects. */
+  if(out==GAL_BLANK_INT32) out=0;
+
+  /* Clean up and return. */
+  gal_data_free(tmp);
+  return out;
+}
+
+
+
+
+
+static size_t *
+gal_tile_per_label_minmax(gal_data_t *labs, size_t *maxlabel)
+{
+  int32_t *l, *lf, *start;
+  size_t *min, *max, maxlab=*maxlabel;
+  size_t i, d, width, *minmax, *coord, ndim=labs->ndim;
+
+  /* If the number of labels is not set (it is blank), find the largest
+     label within the image and write it in that value. */
+  if(maxlab==GAL_BLANK_SIZE_T)
+    *maxlabel=maxlab=gal_tile_per_label_number(labs);
+
+  /* Allocate the necessary arrays. */
+  width=2*ndim;
+  minmax=gal_pointer_allocate(GAL_TYPE_SIZE_T, width*maxlab, 0,
+                              __func__, "minmax");
+  coord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,"coord");
+
+  /* Initialize the minimum and maximum position for each tile/object. So,
+     we'll initialize the minimum coordinates to the maximum possible
+     'size_t' value (in 'GAL_BLANK_SIZE_T') and the maximums to zero. */
+  for(i=0;i<maxlab;++i)
+    for(d=0;d<ndim;++d)
+      {
+        minmax[ i * width +        d ] = GAL_BLANK_SIZE_T; /* Minimum. */
+        minmax[ i * width + ndim + d ] = 0;                /* Maximum. */
+      }
+
+  /* Go over the objects label image and correct the minimum and maximum
+     coordinates. */
+  start=labs->array;
+  lf=(l=labs->array)+labs->size;
+  do
+    {
+      /* Small sanity check: the objects image shouldn't have negative
+         values (can happen when the user gives a clump image). */
+      if(*l!=GAL_BLANK_INT32 && *l<0)
+        error(EXIT_FAILURE, 0, "the main labeled image ('OBJECTS' HDU "
+              "when using Segment), shouldn't contain negative values. "
+              "This can happen when you mistakenly give Segment's "
+              "clumps as the main input labeled image. If you want to "
+              "treat clumps as objects, please use 'astarithmetic' to "
+              "convert all negative pixels to 0 before calling "
+              "MakeCatalog. For example this command: "
+              "'astarithmetic file.fits -hCLUMPS set-i i i 0 lt "
+              "0 where'. For this scenario, a better solution is to "
+              "only keep the clumps and give them each a separate "
+              "label with a command like this: 'astarithmetic file.fits "
+              "-hCLUMPS 0 gt 2 connected-components'");
+
+      /* We are on an object. */
+      if(*l>0)
+        {
+          /* Get the coordinates of this pixel. */
+          gal_dimension_index_to_coord(l-start, ndim, labs->dsize,
+                                       coord);
+
+          /* Check to see this coordinate is the smallest/largest found so
+             far for this label. Note that labels start from 1, while indexs
+             here start from zero. */
+          min = &minmax[ (*l-1) * width        ];
+          max = &minmax[ (*l-1) * width + ndim ];
+          for(d=0;d<ndim;++d)
+            {
+              if( coord[d] < min[d] ) min[d] = coord[d];
+              if( coord[d] > max[d] ) max[d] = coord[d];
+            }
+        }
+    }
+  while(++l<lf);
+
+  /* Clean up and return. */
+  free(coord);
+  return minmax;
+}
+
+
+
+
+
+static gal_data_t *
+gal_tile_per_label_minmax_adjust(gal_data_t *labs, size_t *maxlabel,
+                                 uint8_t inbetweenints, size_t *minmax,
+                                 size_t *numexist)
+{
+  int exists;
+  uint8_t *farray=NULL;
+  gal_data_t *labs_not_present=NULL;
+  size_t i, d, width, nexist=0, ndim=labs->ndim, maxlab=*maxlabel;
+
+  /* If a label doesn't exist in the image, then write over it and define
+     the unique labels to use for the next steps. To over-write, we have
+     two counters: 'i' (for the position in the input array) and 'nexist'
+     (for the counter in the output table). In the end, 'nexist' counts the
+     total number of unique labels in the input. */
+  width=2*ndim;
+  for(i=0;i<maxlab;++i)
+    {
+      /* Make sure a pixel with this label exists in all dimensions. */
+      exists=0;
+      for(d=0;d<ndim;++d)
+        if (    minmax[ i * width + d        ] == GAL_BLANK_SIZE_T
+             && minmax[ i * width + ndim + d ] == 0                )
+          {
+            /* When the label doesn't exist, but the user wants a row
+               anyway, make all the minimums and maximums of all
+               coordinates 0, note that the maximum is already zero. */
+            if(inbetweenints) minmax[i*width+ndim+d] = GAL_BLANK_SIZE_T;
+          }
+        else
+          {
+            /* Write over the blank elements when necessary
+               (i!=j). When i==j, then these statements are
+               redundant. */
+            minmax[nexist*width+d]=minmax[i*width+d];
+            minmax[nexist*width+ndim+d]=minmax[i*width+ndim+d];
+
+            /* Set the checked flag. */
+            exists=1;
+          }
+
+      /* If the label exists (or if the user wants to keep all integers),
+         then increment the output counter. Otherwise, write the keep a
+         flag that the given label didn't exist. */
+      if(inbetweenints || exists) ++nexist;
+      else
+        {
+          /* If 'farray' isn't defined yet, define it. */
+          if(farray==NULL)
+            {
+              /* Note that by initializing with zeros, all (the possibly
+                 existing) previous rows that shouldn't be removed are
+                 flagged as zero in this array. */
+              labs_not_present=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1,
+                                              &maxlab, NULL, 1,
+                                              labs->minmapsize,
+                                              labs->quietmmap,
+                                              NULL, NULL, NULL);
+              farray=(labs_not_present)->array;
+            }
+          farray[i]=1;
+        }
+    }
+
+  /* If we re-arranged the minimum and maximums, set all the remaining
+     minimum and maximums blank (so no tile is created).  */
+  if(inbetweenints==0)
+    {
+      for(i=nexist;i<maxlab;++i)
+        for(d=0;d<ndim;++d)
+          {
+            minmax[ i * width +        d ] = GAL_BLANK_SIZE_T; /* Min. */
+            minmax[ i * width + ndim + d ] = GAL_BLANK_SIZE_T; /* Max. */
+          }
+    }
+
+  /* Return the flags table (identifying which labels were not present in
+     the input. */
+  *numexist=nexist;
+  return labs_not_present;
+}
+
+
+
+
+static gal_data_t *
+gal_tile_per_label_match(gal_data_t *labs_not_present, size_t ntiles)
+{
+  gal_data_t *out=NULL;
+  int32_t *lab_from_tile;
+  size_t *t, *tf, *lab_to_tile;
+  uint8_t *farray=labs_not_present->array;
+  size_t i, j, maxlab=labs_not_present->size, maxlabp1=maxlab+1;
+
+  /* Column to map labels to tiles. Note that because labels start counting
+     from 1 and we want a direct mapping, it is important to allocate one
+     more than 'maxlab'. */
+  gal_list_data_add_alloc(&out, NULL, GAL_TYPE_SIZE_T, 1, &maxlabp1, NULL,
+                          0, labs_not_present->minmapsize,
+                          labs_not_present->quietmmap, "LAB-TO-TILE",
+                          NULL, NULL);
+  lab_to_tile=out->array; /* Must be after the '_alloc' above. */
+
+  /* Column to map tiles to labels. */
+  gal_list_data_add_alloc(&out, NULL, GAL_TYPE_INT32, 1, &ntiles, NULL,
+                          0, labs_not_present->minmapsize,
+                          labs_not_present->quietmmap, "LAB-FROM-TILE",
+                          NULL, NULL);
+  lab_from_tile=out->array; /* Must be after the '_alloc' above. */
+
+  /* Tile index (counting from zero) to input label (counting from 1). */
+  j=0;
+  for(i=0;i<maxlab;++i) if(farray[i]==0) lab_from_tile[j++]=i+1;
+
+  /* Input label to tile index; but first initialize it to blank (so labels
+     do not exist give bad crashes in case they are not properly
+     checked!). */
+  tf=(t=lab_to_tile)+maxlabp1; do *t++=GAL_BLANK_UINT32; while(t<tf);
+  for(i=0;i<ntiles;++i) lab_to_tile[ lab_from_tile[i] ] = i;
+
+  /* For a check.
+  printf("%s: lab_from_tile:\n", __func__);
+  for(i=0;i<ntiles;++i)
+    printf("\ttile-%zu: lab-%d\n", i, lab_from_tile[i]);
+  printf("%s: lab_to_tile:\n", __func__);
+  for(i=0;i<maxlabp1;++i)
+    printf("\tlab-%zu: tile-%d\n", i, lab_to_tile[i]);
+  exit(0);
+  */
+
+  /* Return the list. */
+  return out;
+}
+
+
+
+
+
+/* Define a tile over each group of labeled pixels within the input
+   image. */
+gal_data_t *
+gal_tile_per_label(gal_data_t *labels, size_t *maxlabel,
+                   uint8_t inbetweenints, gal_data_t **lab_fromto_tile)
+{
+  size_t ntiles, *minmax;
+  gal_data_t *labs, *tiles, *labs_not_present;
+
+  /* Make sure the input labels has the correct type and if not, convert to
+     the expected 'int32_t'. */
+  labs=gal_checkset_labels_to_int32(labels, __func__);
+
+  /* Find the minimum and maximum position of each label (necessary to
+     build tiles and adjust them (in case there are non-existant labels. */
+  minmax=gal_tile_per_label_minmax(labs, maxlabel);
+  labs_not_present=gal_tile_per_label_minmax_adjust(labs, maxlabel,
+                                                    inbetweenints,
+                                                    minmax, &ntiles);
+
+  /* Make the tiles for each existing/unique label. */
+  tiles=gal_tile_series_from_minmax(labs, minmax, ntiles);
+
+  /* If a mapping of labels to tiles is necessary add it here. We know it
+     is necessary when 'labs_not_present' exists.  */
+  *lab_fromto_tile = ( labs_not_present
+                       ? gal_tile_per_label_match(labs_not_present, ntiles)
+                       : NULL );
+
+  /* Clean up and return. */
+  if(labs!=labels) gal_data_free(labs);
+  gal_data_free(labs_not_present);
+  free(minmax);
+  return tiles;
+}
+
+
+
+
+
 
 
 
@@ -851,7 +1149,7 @@ gal_tile_full(gal_data_t *input, size_t *regular,
 
       /* Set the block structure for this tile to the 'input', and set the
          next pointer as the next tile. Note that only when we are dealing
-         with the last tile should the 'next' pointer be set to NULL.*/
+         with the last tile should the 'next' pointer be set to NULL. */
       tiles[i].flag  = 0;
       tiles[i].block = input;
       tiles[i].next  = i==numtiles-1 ? NULL : &tiles[i+1];
@@ -934,7 +1232,7 @@ gal_tile_full_sanity_check(char *filename, char *hdu, gal_data_t *input,
   for(i=0;i<ndim;++i)
     {
       /* Check if the number of channels is not more than the size of the
-         image. Note that the reported dimension must be in FITS format.*/
+         image. Note that the reported dimension must be in FITS format. */
       if( input->dsize[i] < tl->numchannels[i] )
         error(EXIT_FAILURE, 0, "the number of channels in dimension %zu "
               "(%zu) is more than the size of the '%s' (hdu: %s) in that "
@@ -983,7 +1281,7 @@ gal_tile_full_two_layers(gal_data_t *input,
   gal_data_t *t;
   size_t i, *junk, *junk2, ndim=tl->ndim=input->ndim;
 
-  /* Initialize.  */
+  /* Initialize. */
   tl->channels=tl->tiles=NULL;
 
   /* Initialize necessary values and do the channels tessellation. */
@@ -1014,7 +1312,7 @@ gal_tile_full_two_layers(gal_data_t *input,
       tl->tiles[ i * tl->tottilesinch - 1 ].next = t;
 
       /* Fill in the information for all the tiles in this channel. Note
-         that we already have the returned value, so it isn't important.*/
+         that we already have the returned value, so it isn't important. */
       junk=gal_tile_full(&tl->channels[i], tl->tilesize, tl->remainderfrac,
                          &t, 1, &junk2);
       free(junk);
@@ -1110,7 +1408,7 @@ void
 gal_tile_full_values_write(gal_data_t *tilevalues,
                            struct gal_tile_two_layer_params *tl,
                            int withblank, char *filename,
-                           gal_fits_list_key_t *keys, char *program_string)
+                           gal_fits_list_key_t *keys, int freekeys)
 {
   gal_data_t *disp;
 
@@ -1121,8 +1419,8 @@ gal_tile_full_values_write(gal_data_t *tilevalues,
         {
           /* A small sanity check. */
           if(tl->permutation==NULL)
-            error(EXIT_FAILURE, 0, "%s: no permutation defined for the input "
-                  "tessellation", __func__);
+            error(EXIT_FAILURE, 0, "%s: no permutation defined for the "
+                  "input tessellation", __func__);
 
           /* Writing tile values to disk is not done for checking, not for
              efficiency. So to be safe (allow the caller to work on
@@ -1138,7 +1436,7 @@ gal_tile_full_values_write(gal_data_t *tilevalues,
                                           withblank, 0);
 
   /* Write the array as a file and then clean up (if necessary). */
-  gal_fits_img_write(disp, filename, keys, program_string);
+  gal_fits_img_write(disp, filename, keys, freekeys);
   if(disp!=tilevalues) gal_data_free(disp);
 }
 
@@ -1185,7 +1483,7 @@ gal_tile_full_values_smooth(gal_data_t *tilevalues,
 
   /* Do the smoothing. */
   if(tl->workoverch)
-    smoothed=gal_convolve_spatial(tilevalues, kernel, numthreads, 1, 1);
+    smoothed=gal_convolve_spatial(tilevalues, kernel, numthreads, 1, 1, 0);
   else
     {
       /* Create the tile structure. */
@@ -1195,7 +1493,7 @@ gal_tile_full_values_smooth(gal_data_t *tilevalues,
       gal_tile_full_two_layers(tilevalues, &ttl);
 
       /* Do the convolution separately on each channel. */
-      smoothed=gal_convolve_spatial(ttl.tiles, kernel, numthreads, 1, 0);
+      smoothed=gal_convolve_spatial(ttl.tiles, kernel, numthreads, 1, 0, 0);
 
       /* Clean up. */
       ttl.tilesize=ttl.numchannels=NULL;
@@ -1205,7 +1503,7 @@ gal_tile_full_values_smooth(gal_data_t *tilevalues,
   /* Reverse the permutation. */
   if(permute) gal_permutation_apply_inverse(smoothed, tl->permutation);
 
-  /* Clean up and return; */
+  /* Clean up and return. */
   free(kdsize);
   gal_data_free(kernel);
   return smoothed;
