@@ -41,7 +41,7 @@
 # Original author:
 #     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
-# Copyright (C) 2016-2021, Free Software Foundation, Inc.
+# Copyright (C) 2016-2025 Free Software Foundation, Inc.
 #
 # Gnuastro is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free
@@ -82,6 +82,15 @@ outdir=
 # script, and once for the utility. In such cases it might be easier to
 # just add the argument/option to the final script that runs the utility
 # rather than these variables.
+#
+# TEST ASTSCRIPTs: When testing a script (e.g., astscript-zeropoint), set
+# utilname to 'script-<name>' (e.g., 'utilname=script-zeropoint'). This
+# ensures their correct identification as a script (not a compiled
+# program). If the thing you are debugging/adding is in the compiled
+# programs used within the script, you have to add a line under this line
+# of the script to make sure that program is also re-built: 'if [ -f
+# "$utility" ]; then rm "$utility"; fi' that will delete that particular
+# program.
 utilname=
 arguments=
 options=
@@ -127,6 +136,11 @@ fi
 longprefix="${utilname:0:6}"
 if [ x"$longprefix" = x"script" ]; then
     execdir="script"
+
+    # We need to delete all the compiled programs so they are recompiled
+    # (for usage in the script, if this slows down your tests, comment it
+    # and only delete the particular program that is causing problems).
+    find "$builddir/bin" -type f -executable -exec rm "{}" \;
 else
     execdir="$utilname"
 fi
